clear
clc
close all
path([pwd, '/subprograms/'], path)


%% SETTINGS AND PREALLOCATION

% Simulation settings
S = 10000; % Number of Monte Carlo replicas
safe = ceil(0.05*S); % Number of replicas per saving at each stage
title = 'S621';

S_bootstrap = 10000; % Number of bootstrap samples in each iteration
S1_grid = 17;
a_min = 0.1; a_max = 0.9; a_step = (a_max-a_min)/(S1_grid-1);
S2_grid = 9;
R2_min = 0.35; R2_max = 0.75; R2_step = (R2_max-R2_min)/(S2_grid-1);
S3_grid = 3;
ratio_min = 1; ratio_max = 1.4; ratio_step = (ratio_max-ratio_min)/(S3_grid-1);
ax_grid = linspace(a_min,a_max,S1_grid);
R2_grid = linspace(R2_min,R2_max,S2_grid);
ratio_grid = linspace(ratio_min,ratio_max,S3_grid);

options = optimoptions('fmincon','Algorithm','interior-point',...
    'GradObj','off','Hessian','off','Display','off',...
    'MaxFunEvals',1000,'MaxIter',1000,'TolCon',0.0,'TolFun',10e-9);
maxerror = 1e-4; % Tolerance criterion for numerical convergence
maxiter = 1e2; % Maximum number of iterations for numerical convergence

% Model settings
T = 100; % Sample size
hypothesis = 0; % Distribution of innovations
N = 2;
M = 4;
K = 3;

% Preallocation
Stat_k0_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                           % Case-0 statistic of kurtosis component at ML estimate
Stat_k1_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                           % Case-1 statistic of kurtosis component at ML estimate
Stat_k2_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                           % Case-2 statistic of kurtosis component at ML estimate
Stat_k3_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid,K);                         % Case-3 statistic of kurtosis component at ML estimate
Stat_k4_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                           % Case-4 statistic of kurtosis component at ML estimate

Stat_s0_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                           % Case-0 statistic of skewness component at ML estimate
Stat_s1_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                           % Case-1 statistic of skewness component at ML estimate
Stat_s2_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                           % Case-2 statistic of skewness component at ML estimate
Stat_s3_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid,K);                         % Case-3 statistic of skewness component at ML estimate
Stat_s4_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                           % Case-4 statistic of skewness component at ML estimate

Stat_GH0_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                          % Case-0 full statistic at ML estimate
Stat_GH1_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                          % Case-1 full statistic at ML estimate
Stat_GH2_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                          % Case-2 full statistic at ML estimate
Stat_GH3_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid,K);                        % Case-3 full statistic at ML estimate
Stat_GH4_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                          % Case-4 full statistic at ML estimate

StatKT_k0_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                         % Case-0 KT statistic of kurtosis component at ML estimate
StatKT_k1_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                         % Case-1 KT statistic of kurtosis component at ML estimate
StatKT_k2_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                         % Case-2 KT statistic of kurtosis component at ML estimate
StatKT_k3_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid,K);                       % Case-3 KT statistic of kurtosis component at ML estimate
StatKT_k4_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                         % Case-4 KT statistic of kurtosis component at ML estimate

StatKT_GH0_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                        % Case-0 full KT statistic at ML estimate
StatKT_GH1_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                        % Case-1 full KT statistic at ML estimate
StatKT_GH2_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                        % Case-2 full KT statistic at ML estimate
StatKT_GH3_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid,K);                      % Case-3 full KT statistic at ML estimate
StatKT_GH4_ML = zeros(S_bootstrap,S1_grid,S2_grid,S3_grid);                        % Case-4 full KT statistic at ML estimate

innovations = normrnd(0,1,K,S_bootstrap,T);

Stat_k0_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
Stat_k1_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
Stat_k2_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
Stat_k3_cv = zeros(S1_grid, S2_grid, S3_grid, 3, K);
Stat_k4_cv = zeros(S1_grid, S2_grid, S3_grid, 3);

Stat_s0_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
Stat_s1_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
Stat_s2_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
Stat_s3_cv = zeros(S1_grid, S2_grid, S3_grid, 3, K);
Stat_s4_cv = zeros(S1_grid, S2_grid, S3_grid, 3);

Stat_GH0_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
Stat_GH1_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
Stat_GH2_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
Stat_GH3_cv = zeros(S1_grid, S2_grid, S3_grid, 3, K);
Stat_GH4_cv = zeros(S1_grid, S2_grid, S3_grid, 3);

StatKT_k0_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
StatKT_k1_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
StatKT_k2_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
StatKT_k3_cv = zeros(S1_grid, S2_grid, S3_grid, 3, K);
StatKT_k4_cv = zeros(S1_grid, S2_grid, S3_grid, 3);

StatKT_GH0_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
StatKT_GH1_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
StatKT_GH2_cv = zeros(S1_grid, S2_grid, S3_grid, 3);
StatKT_GH3_cv = zeros(S1_grid, S2_grid, S3_grid, 3, K);
StatKT_GH4_cv = zeros(S1_grid, S2_grid, S3_grid, 3);

rej_k = zeros(7,3,S);
rej_s = zeros(7,3,S);
rej_GH = zeros(7,3,S);
rej_KTk = zeros(7,3,S);
rej_KTGH = zeros(7,3,S);

rej_k_ML = zeros(7,3,S);
rej_s_ML = zeros(7,3,S);
rej_GH_ML = zeros(7,3,S);
rej_KTk_ML = zeros(7,3,S);
rej_KTGH_ML = zeros(7,3,S);

reject = zeros(7,3,3);
reject_ML = zeros(7,3,3);
rejectKT = zeros(7,3,2);
rejectKT_ML = zeros(7,3,2);


%% BOOTSTRAP CALCULATIONS

for s1_grid = 1:S1_grid
    for s2_grid = 1:S2_grid
        for s3_grid = 1:S3_grid
            
            % Simulation settings
            rng('default')
            rng(280990)
            
            % Model settings
            cx = 0;
            cv = 0;
            ax = ax_grid(s1_grid);
            R2 = R2_grid(s2_grid);
            gx = 1-ax^2;
            ratio = ratio_grid(s3_grid);
            g1 = (1/ratio+1)*(1-R2)/(2*R2);
            g2 = ratio*g1;
            parameters = [ax;gx;g1;g2];
            
            kappa = 1e5;
            C = [(1-parameters(1))*cx;0;cv/2;-cv/2];
            F = [[1+parameters(1),-parameters(1);1,0],zeros(2,2); ...
                zeros(2,4)];
            G = [[sqrt(parameters(2));0],zeros(2,2); ...
                zeros(2,1),diag(sqrt([parameters(3),parameters(4)]))];
            H = [ones(2,1), zeros(2,1), eye(2)];
            
            % Bootstrap calculations
            h = waitbar(0,['BOOTSTRAP. ax: ',num2str(s1_grid),' out of ',num2str(S1_grid),' - R2: ',num2str(s2_grid),' out of ',num2str(S2_grid),' - Ratio: ',num2str(s3_grid),' out of ',num2str(S3_grid)]);
            
            for s_bootstrap = 1:S_bootstrap
                
                X = zeros(M,T);
                Y = zeros(N,T);
                
                X(:,1) = zeros(M,1);
                Y(:,1) = H*X(:,1);
                
                for t = 2:T
                    X(:,t) = C + F*X(:,t-1) + G*[innovations(1,s_bootstrap,t);innovations(2,s_bootstrap,t);innovations(3,s_bootstrap,t)];
                    Y(:,t) = H*X(:,t);
                end
                
                MLestimate = zeros(6,1);
                Z = [Y(1,2:T)-Y(2,2:T);(Y(1,2:T)-Y(1,1:T-1)+Y(2,2:T)-Y(2,1:T-1))/2];
                MLestimate(1:2) = [mean(Z(2,:),2);mean(Z(1,:),2)];
                Zy = fft((Z-mean(Z,2)*ones(1,T-1))');
                
                lower = [-.9999  0.0001  0.0001  0.0001]';
                upper = [ .9999 10.0000 10.0000 10.0000]';
                
                MLestimate(3:6) = fmincon('Bivariate_Likelihood',...
                    parameters,[],[],[],[],lower,upper,[],options,(Z-mean(Z,2)*ones(1,T-1)));
                
                C_ML = [(1-MLestimate(3))*MLestimate(1);0;MLestimate(2)/2;-MLestimate(2)/2;zeros(3,1)];
                F_ML = [[[1+MLestimate(3),-MLestimate(3);1,0],zeros(2,2); ...
                    zeros(2,4)],...
                    zeros(4,3);...
                    zeros(3,4),zeros(3,3)];
                G_ML = [[[sqrt(MLestimate(4));0],zeros(2,2); ...
                    zeros(2,1),diag(sqrt([MLestimate(5),MLestimate(6)]))];...
                    eye(3)];
                H_ML = [[ones(2,1), zeros(2,1), eye(2)],...
                    zeros(2,3)];
                
                EX1 = [zeros(M,1);...
                    zeros(3,1)];
                VX1_ML = [[kappa*eye(2),zeros(2,2);...
                    zeros(2,2),...
                    diag([MLestimate(5),MLestimate(6)])],...
                    [[[sqrt(MLestimate(4));0],zeros(2,2)];...
                    zeros(2,1),diag(sqrt([MLestimate(5),MLestimate(6)]))];...
                    [[[sqrt(MLestimate(4));0],zeros(2,2)];...
                    zeros(2,1),diag(sqrt([MLestimate(5),MLestimate(6)]))]',...
                    eye(3)];
                
                [Xf_ML,Pf_ML,Xs_ML,Ps_ML] = KalmanFS(EX1,VX1_ML,F_ML,G_ML,H_ML,Y,C_ML);
                
                e0_ML = zeros(N,T-2);
                P0_ML = zeros(N,N,T-2);
                for t = 2:T-1
                    e0_ML(:,t-1) = Y(:,t) - H_ML*Xf_ML(:,t);
                    P0_ML(:,:,t-1) = H_ML*Pf_ML(:,:,t)*H_ML';
                end
                e1_ML = Xs_ML(M+1:M+K,2:T-1);
                P1_ML = Ps_ML(M+1:M+K,M+1:M+K,2:T-1);
                e2_ML = Xs_ML(M+2:M+K,2:T-1);
                P2_ML = Ps_ML(M+2:M+K,M+2:M+K,2:T-1);
                e3_ML(:,:,1) = Xs_ML(M+1,2:T-1);
                P3_ML(:,:,:,1) = Ps_ML(M+1,M+1,2:T-1);
                e3_ML(:,:,2) = Xs_ML(M+2,2:T-1);
                P3_ML(:,:,:,2) = Ps_ML(M+2,M+2,2:T-1);
                e3_ML(:,:,3) = Xs_ML(M+3,2:T-1);
                P3_ML(:,:,:,3) = Ps_ML(M+3,M+3,2:T-1);
                
                sk0_ML = ScoreFK(e0_ML,P0_ML);
                sk1_ML = ScoreSK(e1_ML,P1_ML);
                sk2_ML = ScoreSK(e2_ML,P2_ML);
                sk3_ML(:,:,1) = ScoreSK(e3_ML(:,:,1),P3_ML(:,:,:,1));
                sk3_ML(:,:,2) = ScoreSK(e3_ML(:,:,2),P3_ML(:,:,:,2));
                sk3_ML(:,:,3) = ScoreSK(e3_ML(:,:,3),P3_ML(:,:,:,3));
                
                ss0_ML = ScoreFS(e0_ML,P0_ML);
                ss1_ML = ScoreSS(e1_ML,P1_ML);
                ss2_ML = ScoreSS(e2_ML,P2_ML);
                ss3_ML(:,:,1) = ScoreSS(e3_ML(:,:,1),P3_ML(:,:,:,1));
                ss3_ML(:,:,2) = ScoreSS(e3_ML(:,:,2),P3_ML(:,:,:,2));
                ss3_ML(:,:,3) = ScoreSS(e3_ML(:,:,3),P3_ML(:,:,:,3));
                
                Scr_k0_ML = mean(sk0_ML,2);
                Scr_k1_ML = mean(sk1_ML,2);
                Scr_k2_ML = mean(sk2_ML,2);
                Scr_k3_ML(:,1) = mean(sk3_ML(:,:,1),2);
                Scr_k3_ML(:,2) = mean(sk3_ML(:,:,2),2);
                Scr_k3_ML(:,3) = mean(sk3_ML(:,:,3),2);
                
                MSE_ML = mean(P0_ML,3);
                
                Scr_s0_ML = mean(ss0_ML,2);
                Scr_s1_ML = mean(ss1_ML,2);
                Scr_s2_ML = mean(ss2_ML,2);
                Scr_s3_ML(:,1) = mean(ss3_ML(:,:,1),2);
                Scr_s3_ML(:,2) = mean(ss3_ML(:,:,2),2);
                Scr_s3_ML(:,3) = mean(ss3_ML(:,:,3),2);
                
                ax = MLestimate(3);
                gx = MLestimate(4);
                g1 = MLestimate(5);
                g2 = MLestimate(6);
                
                [PHI,THETA] = Bivariate_WKfilter(ax,gx,g1,g2);
                auxGAMMA0 = AutocovarianceWK(PHI,THETA,eye(K),0);
                
                r = K;
                [~,aux_bs,~] = ScoreSS(zeros(K,1),eye(K) - auxGAMMA0);
                [~,aux_bk,~] = ScoreSK(zeros(K,1),eye(K) - auxGAMMA0);
                
                B_e1_1 = aux_bs(1:r,:);
                B_e2_1 = aux_bk(2:1+r^2);
                B_e3_1 = aux_bs(r+1:end,:);
                B_e4_1 = aux_bk(2+r^2:end);
                
                r = K-1;
                E2 = [zeros(K-1,1),eye(K-1)];
                [~,aux_bs,~] = ScoreSS(zeros(K-1,1),eye(K-1) - auxGAMMA0(2:K,2:K));
                [~,aux_bk,~] = ScoreSK(zeros(K-1,1),eye(K-1) - auxGAMMA0(2:K,2:K));
                
                B_e1_2 = E2'*aux_bs(1:r,:);
                B_e2_2 = kron(E2',E2')*aux_bk(2:1+r^2);
                B_e3_2 = kron(E2',E2')*aux_bs(r+1:end,:);
                B_e4_2 = kron(E2',E2')*aux_bk(2+r^2:end);
                
                r = 1;
                E3(:,:,1) = [1,0,0];
                [~,aux_bs,~] = ScoreSS(0,1 - auxGAMMA0(1,1));
                [~,aux_bk,~] = ScoreSK(0,1 - auxGAMMA0(1,1));
                
                B_e1_3(:,:,1) = E3(:,:,1)'*aux_bs(1:r,:);
                B_e2_3(:,:,1) = kron(E3(:,:,1)',E3(:,:,1)')*aux_bk(2:1+r^2);
                B_e3_3(:,:,1) = kron(E3(:,:,1)',E3(:,:,1)')*aux_bs(r+1:end,:);
                B_e4_3(:,:,1) = kron(E3(:,:,1)',E3(:,:,1)')*aux_bk(2+r^2:end);
                
                E3(:,:,2) = [0,1,0];
                [~,aux_bs,~] = ScoreSS(0,1 - auxGAMMA0(2,2));
                [~,aux_bk,~] = ScoreSK(0,1 - auxGAMMA0(2,2));
                
                B_e1_3(:,:,2) = E3(:,:,2)'*aux_bs(1:r,:);
                B_e2_3(:,:,2) = kron(E3(:,:,2)',E3(:,:,2)')*aux_bk(2:1+r^2);
                B_e3_3(:,:,2) = kron(E3(:,:,2)',E3(:,:,2)')*aux_bs(r+1:end,:);
                B_e4_3(:,:,2) = kron(E3(:,:,2)',E3(:,:,2)')*aux_bk(2+r^2:end);
                
                E3(:,:,3) = [0,0,1];
                [~,aux_bs,~] = ScoreSS(0,1 - auxGAMMA0(3,3));
                [~,aux_bk,~] = ScoreSK(0,1 - auxGAMMA0(3,3));
                
                B_e1_3(:,:,3) = E3(:,:,3)'*aux_bs(1:r,:);
                B_e2_3(:,:,3) = kron(E3(:,:,3)',E3(:,:,3)')*aux_bk(2:1+r^2);
                B_e3_3(:,:,3) = kron(E3(:,:,3)',E3(:,:,3)')*aux_bs(r+1:end,:);
                B_e4_3(:,:,3) = kron(E3(:,:,3)',E3(:,:,3)')*aux_bk(2+r^2:end);
                
                B_e1_4 = [B_e1_3(:,:,1),B_e1_3(:,:,2),B_e1_3(:,:,3)];
                B_e2_4 = [B_e2_3(:,:,1),B_e2_3(:,:,2),B_e2_3(:,:,3)];
                B_e3_4 = [B_e3_3(:,:,1),B_e3_3(:,:,2),B_e3_3(:,:,3)];
                B_e4_4 = [B_e4_3(:,:,1),B_e4_3(:,:,2),B_e4_3(:,:,3)];
                
                [CMe1e1,CMe2e2,CMe3e3,CMe4e4] = AutocovarianceMM(auxGAMMA0,auxGAMMA0);
                
                AGFe1e1 = CMe1e1;
                AGFe2e2 = CMe2e2;
                AGFe3e3 = CMe3e3;
                AGFe4e4 = CMe4e4;
                
                j = 0;
                error = 1;
                
                while ((j < maxiter) && (error >= maxerror))
                    
                    auxGAMMAj = AutocovarianceWK(PHI,THETA,eye(K),j+1);
                    [CMe1e1,CMe2e2,CMe3e3,CMe4e4] = ...
                        AutocovarianceMM(auxGAMMA0,auxGAMMAj);
                    [CMe1e1m,CMe2e2m,CMe3e3m,CMe4e4m] = ...
                        AutocovarianceMM(auxGAMMA0,auxGAMMAj');
                    
                    AGFe1e1 = AGFe1e1 + CMe1e1 + CMe1e1m;
                    AGFe2e2 = AGFe2e2 + CMe2e2 + CMe2e2m;
                    AGFe3e3 = AGFe3e3 + CMe3e3 + CMe3e3m;
                    AGFe4e4 = AGFe4e4 + CMe4e4 + CMe4e4m;
                    
                    j = j+1;
                    
                    de1e1 = max(max(abs(CMe1e1 + CMe1e1m)));
                    de2e2 = max(max(abs(CMe2e2 + CMe2e2m)));
                    de3e3 = max(max(abs(CMe3e3 + CMe3e3m)));
                    de4e4 = max(max(abs(CMe4e4 + CMe4e4m)));
                    error = max([de1e1,de2e2,de3e3,de4e4]);
                    
                end
                
                Var_k1_ML = (B_e4_1')*AGFe4e4*(B_e4_1) - (B_e2_1')*AGFe2e2*(B_e2_1);
                Var_k2_ML = (B_e4_2')*AGFe4e4*(B_e4_2) - (B_e2_2')*AGFe2e2*(B_e2_2);
                Var_k3_ML(:,1) = (B_e4_3(:,:,1)')*AGFe4e4*(B_e4_3(:,:,1)) - ...
                    (B_e2_3(:,:,1)')*AGFe2e2*(B_e2_3(:,:,1));
                Var_k3_ML(:,2) = (B_e4_3(:,:,2)')*AGFe4e4*(B_e4_3(:,:,2)) - ...
                    (B_e2_3(:,:,2)')*AGFe2e2*(B_e2_3(:,:,2));
                Var_k3_ML(:,3) = (B_e4_3(:,:,3)')*AGFe4e4*(B_e4_3(:,:,3)) - ...
                    (B_e2_3(:,:,3)')*AGFe2e2*(B_e2_3(:,:,3));
                Var_k4_ML = (B_e4_4')*AGFe4e4*(B_e4_4) - (B_e2_4')*AGFe2e2*(B_e2_4);
                
                Var_s1_ML = (B_e3_1')*AGFe3e3*(B_e3_1) - (B_e1_1')*AGFe1e1*(B_e1_1);
                Var_s2_ML = (B_e3_2')*AGFe3e3*(B_e3_2) - (B_e1_2')*AGFe1e1*(B_e1_2);
                Var_s3_ML(:,:,1) = (B_e3_3(:,:,1)')*AGFe3e3*(B_e3_3(:,:,1)) - ...
                    (B_e1_3(:,:,1)')*AGFe1e1*(B_e1_3(:,:,1));
                Var_s3_ML(:,:,2) = (B_e3_3(:,:,2)')*AGFe3e3*(B_e3_3(:,:,2)) - ...
                    (B_e1_3(:,:,2)')*AGFe1e1*(B_e1_3(:,:,2));
                Var_s3_ML(:,:,3) = (B_e3_3(:,:,3)')*AGFe3e3*(B_e3_3(:,:,3)) - ...
                    (B_e1_3(:,:,3)')*AGFe1e1*(B_e1_3(:,:,3));
                Var_s4_ML = (B_e3_4')*AGFe3e3*(B_e3_4) - (B_e1_4')*AGFe1e1*(B_e1_4);
                
                Stat_k0_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = T*(Scr_k0_ML.^2)/(N*(N+2)/2);
                Stat_k1_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = T*(Scr_k1_ML.^2)/(Var_k1_ML);
                Stat_k2_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = T*(Scr_k2_ML.^2)/(Var_k2_ML);
                Stat_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,1) = T*(Scr_k3_ML(:,1).^2)/(Var_k3_ML(:,1));
                Stat_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,2) = T*(Scr_k3_ML(:,2).^2)/(Var_k3_ML(:,2));
                Stat_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,3) = T*(Scr_k3_ML(:,3).^2)/(Var_k3_ML(:,3));
                Scr_k4_ML = [Scr_k3_ML(:,1);Scr_k3_ML(:,2);Scr_k3_ML(:,3)];
                Stat_k4_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = T*Scr_k4_ML'*((Var_k4_ML)\Scr_k4_ML);
                
                Stat_s0_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = T*Scr_s0_ML'*pinv(MSE_ML)...
                    *Scr_s0_ML/(2*(N+2));
                Stat_s1_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = T*Scr_s1_ML'*((Var_s1_ML)\Scr_s1_ML);
                Stat_s2_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = T*Scr_s2_ML'*((Var_s2_ML)\Scr_s2_ML);
                Stat_s3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,1) = T*Scr_s3_ML(:,1)'*((Var_s3_ML(:,:,1))\Scr_s3_ML(:,1));
                Stat_s3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,2) = T*Scr_s3_ML(:,2)'*((Var_s3_ML(:,:,2))\Scr_s3_ML(:,2));
                Stat_s3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,3) = T*Scr_s3_ML(:,3)'*((Var_s3_ML(:,:,3))\Scr_s3_ML(:,3));
                Scr_s4_ML = [Scr_s3_ML(:,1);Scr_s3_ML(:,2);Scr_s3_ML(:,3)];
                Stat_s4_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = T*Scr_s4_ML'*((Var_s4_ML)\Scr_s4_ML);
                
                Stat_GH0_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = Stat_k0_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) + Stat_s0_ML(s_bootstrap,s1_grid,s2_grid,s3_grid);
                Stat_GH1_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = Stat_k1_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) + Stat_s1_ML(s_bootstrap,s1_grid,s2_grid,s3_grid);
                Stat_GH2_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = Stat_k2_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) + Stat_s2_ML(s_bootstrap,s1_grid,s2_grid,s3_grid);
                Stat_GH3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,1) = Stat_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,1) + Stat_s3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,1);
                Stat_GH3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,2) = Stat_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,2) + Stat_s3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,2);
                Stat_GH3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,3) = Stat_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,3) + Stat_s3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,3);
                Stat_GH4_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = Stat_k4_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) + Stat_s4_ML(s_bootstrap,s1_grid,s2_grid,s3_grid);
                
                StatKT_k0_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = Stat_k0_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) * (Scr_k0_ML > 0);
                StatKT_k1_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = Stat_k1_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) * (Scr_k1_ML > 0);
                StatKT_k2_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = Stat_k2_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) * (Scr_k2_ML > 0);
                StatKT_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,1) = Stat_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,1) * (Scr_k3_ML(:,1) > 0);
                StatKT_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,2) = Stat_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,2) * (Scr_k3_ML(:,2) > 0);
                StatKT_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,3) = Stat_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,3) * (Scr_k3_ML(:,3) > 0);
                Scr_k4_ML = Scr_k4_ML .* (Scr_k4_ML > 0);
                StatKT_k4_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = T*Scr_k4_ML'*((Var_k4_ML)\Scr_k4_ML);
                
                StatKT_GH0_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = StatKT_k0_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) + Stat_s0_ML(s_bootstrap,s1_grid,s2_grid,s3_grid);
                StatKT_GH1_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = StatKT_k1_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) + Stat_s1_ML(s_bootstrap,s1_grid,s2_grid,s3_grid);
                StatKT_GH2_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = StatKT_k2_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) + Stat_s2_ML(s_bootstrap,s1_grid,s2_grid,s3_grid);
                StatKT_GH3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,1) = StatKT_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,1) + Stat_s3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,1);
                StatKT_GH3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,2) = StatKT_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,2) + Stat_s3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,2);
                StatKT_GH3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,3) = StatKT_k3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,3) + Stat_s3_ML(s_bootstrap,s1_grid,s2_grid,s3_grid,3);
                StatKT_GH4_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) = StatKT_k4_ML(s_bootstrap,s1_grid,s2_grid,s3_grid) + Stat_s4_ML(s_bootstrap,s1_grid,s2_grid,s3_grid);
                
                waitbar(s_bootstrap/S_bootstrap,h)
                
            end
            close(h)
            
            save bootstrap.mat ax_grid R2_grid Stat_k0_ML Stat_k1_ML Stat_k2_ML Stat_k3_ML Stat_k4_ML ...
                Stat_s0_ML Stat_s1_ML Stat_s2_ML Stat_s3_ML Stat_s4_ML...
                Stat_GH0_ML Stat_GH1_ML Stat_GH2_ML Stat_GH3_ML Stat_GH4_ML ...
                StatKT_k0_ML StatKT_k1_ML StatKT_k2_ML StatKT_k3_ML StatKT_k4_ML ...
                StatKT_GH0_ML StatKT_GH1_ML StatKT_GH2_ML StatKT_GH3_ML StatKT_GH4_ML
        end
    end
    
end

movefile('bootstrap.mat',['IO/bootstrap/', title,'_bootstrap_T',num2str(T),'.mat']);


%% BOOTSTRAP CRITICAL VALUES

load(['IO/bootstrap/', title,'_bootstrap_T',num2str(T),'.mat'])

for s1_grid = 1:S1_grid
    for s2_grid = 1:S2_grid
        
        for a = 1:3
            
            level = (a == 1)*0.99 + (a == 2)*0.95 + (a == 3)*0.90;
            
            % Test statistic
            Stat_k0_cv(s1_grid, s2_grid, s3_grid, a) = quantile(Stat_k0_ML(:, s1_grid, s2_grid, s3_grid), level);
            Stat_k1_cv(s1_grid, s2_grid, s3_grid, a) = quantile(Stat_k1_ML(:, s1_grid, s2_grid, s3_grid), level);
            Stat_k2_cv(s1_grid, s2_grid, s3_grid, a) = quantile(Stat_k2_ML(:, s1_grid, s2_grid, s3_grid), level);
            for k = 1:K
                Stat_k3_cv(s1_grid, s2_grid, s3_grid, a, k) = quantile(Stat_k3_ML(:, s1_grid, s2_grid, s3_grid, k), level);
            end
            Stat_k4_cv(s1_grid, s2_grid, s3_grid, a) = quantile(Stat_k4_ML(:, s1_grid, s2_grid, s3_grid), level);
            
            Stat_s0_cv(s1_grid, s2_grid, s3_grid, a) = quantile(Stat_s0_ML(:, s1_grid, s2_grid, s3_grid), level);
            Stat_s1_cv(s1_grid, s2_grid, s3_grid, a) = quantile(Stat_s1_ML(:, s1_grid, s2_grid, s3_grid), level);
            Stat_s2_cv(s1_grid, s2_grid, s3_grid, a) = quantile(Stat_s2_ML(:, s1_grid, s2_grid, s3_grid), level);
            for k = 1:K
                Stat_s3_cv(s1_grid, s2_grid, s3_grid, a, k) = quantile(Stat_s3_ML(:, s1_grid, s2_grid, s3_grid, k), level);
            end
            Stat_s4_cv(s1_grid, s2_grid, s3_grid, a) = quantile(Stat_s4_ML(:, s1_grid, s2_grid, s3_grid), level);
            
            Stat_GH0_cv(s1_grid, s2_grid, s3_grid, a) = quantile(Stat_GH0_ML(:, s1_grid, s2_grid, s3_grid), level);
            Stat_GH1_cv(s1_grid, s2_grid, s3_grid, a) = quantile(Stat_GH1_ML(:, s1_grid, s2_grid, s3_grid), level);
            Stat_GH2_cv(s1_grid, s2_grid, s3_grid, a) = quantile(Stat_GH2_ML(:, s1_grid, s2_grid, s3_grid), level);
            for k = 1:K
                Stat_GH3_cv(s1_grid, s2_grid, s3_grid, a, k) = quantile(Stat_GH3_ML(:, s1_grid, s2_grid, s3_grid, k), level);
            end
            Stat_GH4_cv(s1_grid, s2_grid, s3_grid, a) = quantile(Stat_GH4_ML(:, s1_grid, s2_grid, s3_grid), level);
            
            StatKT_k0_cv(s1_grid, s2_grid, s3_grid, a) = quantile(StatKT_k0_ML(:, s1_grid, s2_grid, s3_grid), level);
            StatKT_k1_cv(s1_grid, s2_grid, s3_grid, a) = quantile(StatKT_k1_ML(:, s1_grid, s2_grid, s3_grid), level);
            StatKT_k2_cv(s1_grid, s2_grid, s3_grid, a) = quantile(StatKT_k2_ML(:, s1_grid, s2_grid, s3_grid), level);
            for k = 1:K
                StatKT_k3_cv(s1_grid, s2_grid, s3_grid, a, k) = quantile(StatKT_k3_ML(:, s1_grid, s2_grid, s3_grid, k), level);
            end
            StatKT_k4_cv(s1_grid, s2_grid, s3_grid, a) = quantile(StatKT_k4_ML(:, s1_grid, s2_grid, s3_grid), level);
            
            StatKT_GH0_cv(s1_grid, s2_grid, s3_grid, a) = quantile(StatKT_GH0_ML(:, s1_grid, s2_grid, s3_grid), level);
            StatKT_GH1_cv(s1_grid, s2_grid, s3_grid, a) = quantile(StatKT_GH1_ML(:, s1_grid, s2_grid, s3_grid), level);
            StatKT_GH2_cv(s1_grid, s2_grid, s3_grid, a) = quantile(StatKT_GH2_ML(:, s1_grid, s2_grid, s3_grid), level);
            for k = 1:K
                StatKT_GH3_cv(s1_grid, s2_grid, s3_grid, a, k) = quantile(StatKT_GH3_ML(:, s1_grid, s2_grid, s3_grid, k), level);
            end
            StatKT_GH4_cv(s1_grid, s2_grid, s3_grid, a) = quantile(StatKT_GH4_ML(:, s1_grid, s2_grid, s3_grid), level);
            
        end
        
    end
end

save crit.mat ax_grid R2_grid ...
    Stat_k0_cv Stat_k1_cv Stat_k2_cv Stat_k3_cv Stat_k4_cv ...
    Stat_s0_cv Stat_s1_cv Stat_s2_cv Stat_s3_cv Stat_s4_cv...
    Stat_GH0_cv Stat_GH1_cv Stat_GH2_cv Stat_GH3_cv Stat_GH4_cv ...
    StatKT_k0_cv StatKT_k1_cv StatKT_k2_cv StatKT_k3_cv StatKT_k4_cv ...
    StatKT_GH0_cv StatKT_GH1_cv StatKT_GH2_cv StatKT_GH3_cv StatKT_GH4_cv

movefile('crit.mat',['IO/bootstrap/', title,'_critical_T',num2str(T),'.mat']);


%% BOOTSTRAP TEST

h = waitbar(0,'Computation of test statistic');
load(['IO/bootstrap/', title,'_critical_T',num2str(T),'.mat'])
load(['IO/estimate/', title,'_estimate_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat'])
load(['IO/test/', title,'_test_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat'])

cx = 0;
cv = 0;
ax = 0.5;
gx = 1-ax^2;
g1 = 1;
g2 = 1;
parameters = [cx;cv;ax;gx;g1;g2];

R2 = 1/(1+2*(1-parameters(3)^2)/parameters(4)*...
    (parameters(5)*parameters(6)/(parameters(5)+parameters(6))));
ratio = max(parameters(5)/parameters(6),parameters(6)/parameters(5)) + 0.0001;

if (ax <= a_min)
    id_a = 1;
    w_a = 1;
elseif (ax >= a_max)
    id_a = S1_grid-1;
    w_a = 0;
else
    id_a = ceil((ax-a_min)/a_step);
    w_a = id_a - (ax-a_min)/a_step;
end

if (R2 <= R2_min)
    id_R2 = 1;
    w_R2 = 1;
elseif (R2 >= R2_max)
    id_R2 = S2_grid-1;
    w_R2 = 0;
else
    id_R2 = ceil((R2-R2_min)/R2_step);
    w_R2 = id_R2 - (R2-R2_min)/R2_step;
end

if (ratio <= ratio_min)
    id_ratio = 1;
    w_ratio = 1;
elseif (ratio >= ratio_max)
    id_ratio = S3_grid-1;
    w_ratio = 0;
else
    id_ratio = ceil((ratio-ratio_min)/ratio_step);
    w_ratio = id_ratio - (ratio-ratio_min)/ratio_step;
end

for s = 1:S
    
    a_ML = MLestimate(3,s);
    R2_ML = 1/(1+2*(1-MLestimate(3,s)^2)/MLestimate(4,s)*...
        (MLestimate(5,s)*MLestimate(6,s)/(MLestimate(5,s)+MLestimate(6,s))));
    ratio_ML = max(MLestimate(5,s)/MLestimate(6,s),MLestimate(6,s)/MLestimate(5,s)) + 0.0001;
    
    if (a_ML <= a_min)
        id_a_ML = 1;
        w_a_ML = 1;
    elseif (a_ML >= a_max)
        id_a_ML = S1_grid-1;
        w_a_ML = 0;
    else
        id_a_ML = ceil((a_ML-a_min)/a_step);
        w_a_ML = id_a_ML - (a_ML-a_min)/a_step;
    end
    
    if (R2_ML <= R2_min)
        id_R2_ML = 1;
        w_R2_ML = 1;
    elseif (R2_ML >= R2_max)
        id_R2_ML = S2_grid-1;
        w_R2_ML = 0;
    else
        id_R2_ML = ceil((R2_ML-R2_min)/R2_step);
        w_R2_ML = id_R2_ML - (R2_ML-R2_min)/R2_step;
    end
    
    if (ratio_ML <= ratio_min)
        id_ratio_ML = 1;
        w_ratio_ML = 1;
    elseif (ratio_ML >= ratio_max)
        id_ratio_ML = S3_grid-1;
        w_ratio_ML = 0;
    else
        id_ratio_ML = ceil((ratio_ML-ratio_min)/ratio_step);
        w_ratio_ML = id_ratio_ML - (ratio_ML-ratio_min)/ratio_step;
    end
    
    for index = [1,2,3]
        rej_k(1,index,s) = (Stat_k0(1,s) > w_ratio*[w_a,1-w_a]*...
            Stat_k0_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2] ...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_k0_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_k(2,index,s) = (Stat_k1(1,s) > w_ratio*[w_a,1-w_a]*...
            Stat_k1_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_k1_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_k(3,index,s) = (Stat_k2(1,s) > w_ratio*[w_a,1-w_a]*...
            Stat_k2_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_k2_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_k(4,index,s) = (Stat_k3(1,s,1) > w_ratio*[w_a,1-w_a]*...
            Stat_k3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,1)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_k3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,1)*[w_R2;1-w_R2]);
        rej_k(5,index,s) = (Stat_k3(1,s,2) > w_ratio*[w_a,1-w_a]*...
            Stat_k3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,2)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_k3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,2)*[w_R2;1-w_R2]);
        rej_k(6,index,s) = (Stat_k3(1,s,3) > w_ratio*[w_a,1-w_a]*...
            Stat_k3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,3)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_k3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,3)*[w_R2;1-w_R2]);
        rej_k(7,index,s) = (Stat_k4(1,s) > w_ratio*[w_a,1-w_a]*...
            Stat_k4_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_k4_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        
        rej_s(1,index,s) = (Stat_s0(1,s) > w_ratio*[w_a,1-w_a]*...
            Stat_s0_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_s0_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_s(2,index,s) = (Stat_s1(1,s) > w_ratio*[w_a,1-w_a]*...
            Stat_s1_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_s1_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_s(3,index,s) = (Stat_s2(1,s) > w_ratio*[w_a,1-w_a]*...
            Stat_s2_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_s2_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_s(4,index,s) = (Stat_s3(1,s,1) > w_ratio*[w_a,1-w_a]*...
            Stat_s3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,1)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_s3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,1)*[w_R2;1-w_R2]);
        rej_s(5,index,s) = (Stat_s3(1,s,2) > w_ratio*[w_a,1-w_a]*...
            Stat_s3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,2)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_s3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,2)*[w_R2;1-w_R2]);
        rej_s(6,index,s) = (Stat_s3(1,s,3) > w_ratio*[w_a,1-w_a]*...
            Stat_s3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,3)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_s3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,3)*[w_R2;1-w_R2]);
        rej_s(7,index,s) = (Stat_s4(1,s) > w_ratio*[w_a,1-w_a]*...
            Stat_s4_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_s4_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        
        rej_GH(1,index,s) = (Stat_GH0(1,s) > w_ratio*[w_a,1-w_a]*...
            Stat_GH0_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_GH0_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_GH(2,index,s) = (Stat_GH1(1,s) > w_ratio*[w_a,1-w_a]*...
            Stat_GH1_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_GH1_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_GH(3,index,s) = (Stat_GH2(1,s) > w_ratio*[w_a,1-w_a]*...
            Stat_GH2_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_GH2_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_GH(4,index,s) = (Stat_GH3(1,s,1) > w_ratio*[w_a,1-w_a]*...
            Stat_GH3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,1)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_GH3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,1)*[w_R2;1-w_R2]);
        rej_GH(5,index,s) = (Stat_GH3(1,s,2) > w_ratio*[w_a,1-w_a]*...
            Stat_GH3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,2)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_GH3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,2)*[w_R2;1-w_R2]);
        rej_GH(6,index,s) = (Stat_GH3(1,s,3) > w_ratio*[w_a,1-w_a]*...
            Stat_GH3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,3)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_GH3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,3)*[w_R2;1-w_R2]);
        rej_GH(7,index,s) = (Stat_GH4(1,s) > w_ratio*[w_a,1-w_a]*...
            Stat_GH4_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            Stat_GH4_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        
        rej_KTk(1,index,s) = (StatKT_k0(1,s) > w_ratio*[w_a,1-w_a]*...
            StatKT_k0_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            StatKT_k0_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_KTk(2,index,s) = (StatKT_k1(1,s) > w_ratio*[w_a,1-w_a]*...
            StatKT_k1_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            StatKT_k1_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_KTk(3,index,s) = (StatKT_k2(1,s) > w_ratio*[w_a,1-w_a]*...
            StatKT_k2_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            StatKT_k2_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_KTk(4,index,s) = (StatKT_k3(1,s,1) > w_ratio*[w_a,1-w_a]*...
            StatKT_k3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,1)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            StatKT_k3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,1)*[w_R2;1-w_R2]);
        rej_KTk(5,index,s) = (StatKT_k3(1,s,2) > w_ratio*[w_a,1-w_a]*...
            StatKT_k3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,2)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            StatKT_k3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,2)*[w_R2;1-w_R2]);
        rej_KTk(6,index,s) = (StatKT_k3(1,s,3) > w_ratio*[w_a,1-w_a]*...
            StatKT_k3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,3)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            StatKT_k3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,3)*[w_R2;1-w_R2]);
        rej_KTk(7,index,s) = (StatKT_k4(1,s) > w_ratio*[w_a,1-w_a]*...
            StatKT_k4_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            StatKT_k4_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        
        rej_KTGH(1,index,s) = (StatKT_GH0(1,s) > w_ratio*[w_a,1-w_a]*...
            StatKT_GH0_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            StatKT_GH0_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_KTGH(2,index,s) = (StatKT_GH1(1,s) > w_ratio*[w_a,1-w_a]*...
            StatKT_GH1_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            StatKT_GH1_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_KTGH(3,index,s) = (StatKT_GH2(1,s) > w_ratio*[w_a,1-w_a]*...
            StatKT_GH2_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            StatKT_GH2_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        rej_KTGH(4,index,s) = (StatKT_GH3(1,s,1) > w_ratio*[w_a,1-w_a]*...
            StatKT_GH3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,1)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            StatKT_GH3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,1)*[w_R2;1-w_R2]);
        rej_KTGH(5,index,s) = (StatKT_GH3(1,s,2) > w_ratio*[w_a,1-w_a]*...
            StatKT_GH3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,2)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            StatKT_GH3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,2)*[w_R2;1-w_R2]);
        rej_KTGH(6,index,s) = (StatKT_GH3(1,s,3) > w_ratio*[w_a,1-w_a]*...
            StatKT_GH3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index,3)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            StatKT_GH3_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index,3)*[w_R2;1-w_R2]);
        rej_KTGH(7,index,s) = (StatKT_GH4(1,s) > w_ratio*[w_a,1-w_a]*...
            StatKT_GH4_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio,index)*[w_R2;1-w_R2]...
            +(1-w_ratio)*[w_a,1-w_a]*...
            StatKT_GH4_cv(id_a:id_a+1,id_R2:id_R2+1,id_ratio+1,index)*[w_R2;1-w_R2]);
        
        rej_k_ML(1,index,s) = (Stat_k0_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_k0_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML] ...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_k0_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_k_ML(2,index,s) = (Stat_k1_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_k1_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_k1_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_k_ML(3,index,s) = (Stat_k2_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_k2_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_k2_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_k_ML(4,index,s) = (Stat_k3_ML(1,s,1) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_k3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,1)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_k3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,1)*[w_R2_ML;1-w_R2_ML]);
        rej_k_ML(5,index,s) = (Stat_k3_ML(1,s,2) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_k3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,2)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_k3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,2)*[w_R2_ML;1-w_R2_ML]);
        rej_k_ML(6,index,s) = (Stat_k3_ML(1,s,3) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_k3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,3)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_k3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,3)*[w_R2_ML;1-w_R2_ML]);
        rej_k_ML(7,index,s) = (Stat_k4_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_k4_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_k4_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        
        rej_s_ML(1,index,s) = (Stat_s0_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_s0_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_s0_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_s_ML(2,index,s) = (Stat_s1_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_s1_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_s1_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_s_ML(3,index,s) = (Stat_s2_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_s2_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_s2_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_s_ML(4,index,s) = (Stat_s3_ML(1,s,1) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_s3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,1)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_s3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,1)*[w_R2_ML;1-w_R2_ML]);
        rej_s_ML(5,index,s) = (Stat_s3_ML(1,s,2) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_s3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,2)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_s3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,2)*[w_R2_ML;1-w_R2_ML]);
        rej_s_ML(6,index,s) = (Stat_s3_ML(1,s,3) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_s3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,3)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_s3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,3)*[w_R2_ML;1-w_R2_ML]);
        rej_s_ML(7,index,s) = (Stat_s4_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_s4_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_s4_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        
        rej_GH_ML(1,index,s) = (Stat_GH0_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_GH0_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_GH0_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_GH_ML(2,index,s) = (Stat_GH1_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_GH1_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_GH1_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_GH_ML(3,index,s) = (Stat_GH2_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_GH2_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_GH2_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_GH_ML(4,index,s) = (Stat_GH3_ML(1,s,1) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_GH3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,1)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_GH3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,1)*[w_R2_ML;1-w_R2_ML]);
        rej_GH_ML(5,index,s) = (Stat_GH3_ML(1,s,2) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_GH3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,2)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_GH3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,2)*[w_R2_ML;1-w_R2_ML]);
        rej_GH_ML(6,index,s) = (Stat_GH3_ML(1,s,3) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_GH3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,3)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_GH3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,3)*[w_R2_ML;1-w_R2_ML]);
        rej_GH_ML(7,index,s) = (Stat_GH4_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            Stat_GH4_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            Stat_GH4_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        
        rej_KTk_ML(1,index,s) = (StatKT_k0_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            StatKT_k0_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            StatKT_k0_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_KTk_ML(2,index,s) = (StatKT_k1_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            StatKT_k1_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            StatKT_k1_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_KTk_ML(3,index,s) = (StatKT_k2_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            StatKT_k2_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            StatKT_k2_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_KTk_ML(4,index,s) = (StatKT_k3_ML(1,s,1) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            StatKT_k3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,1)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            StatKT_k3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,1)*[w_R2_ML;1-w_R2_ML]);
        rej_KTk_ML(5,index,s) = (StatKT_k3_ML(1,s,2) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            StatKT_k3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,2)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            StatKT_k3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,2)*[w_R2_ML;1-w_R2_ML]);
        rej_KTk_ML(6,index,s) = (StatKT_k3_ML(1,s,3) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            StatKT_k3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,3)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            StatKT_k3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,3)*[w_R2_ML;1-w_R2_ML]);
        rej_KTk_ML(7,index,s) = (StatKT_k4_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            StatKT_k4_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            StatKT_k4_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        
        rej_KTGH_ML(1,index,s) = (StatKT_GH0_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            StatKT_GH0_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            StatKT_GH0_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_KTGH_ML(2,index,s) = (StatKT_GH1_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            StatKT_GH1_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            StatKT_GH1_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_KTGH_ML(3,index,s) = (StatKT_GH2_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            StatKT_GH2_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            StatKT_GH2_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
        rej_KTGH_ML(4,index,s) = (StatKT_GH3_ML(1,s,1) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            StatKT_GH3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,1)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            StatKT_GH3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,1)*[w_R2_ML;1-w_R2_ML]);
        rej_KTGH_ML(5,index,s) = (StatKT_GH3_ML(1,s,2) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            StatKT_GH3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,2)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            StatKT_GH3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,2)*[w_R2_ML;1-w_R2_ML]);
        rej_KTGH_ML(6,index,s) = (StatKT_GH3_ML(1,s,3) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            StatKT_GH3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index,3)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            StatKT_GH3_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index,3)*[w_R2_ML;1-w_R2_ML]);
        rej_KTGH_ML(7,index,s) = (StatKT_GH4_ML(1,s) > w_ratio_ML*[w_a_ML,1-w_a_ML]*...
            StatKT_GH4_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML,index)*[w_R2_ML;1-w_R2_ML]...
            +(1-w_ratio_ML)*[w_a_ML,1-w_a_ML]*...
            StatKT_GH4_cv(id_a_ML:id_a_ML+1,id_R2_ML:id_R2_ML+1,id_ratio_ML+1,index)*[w_R2_ML;1-w_R2_ML]);
    end
    
    if (rem(s, safe) == 0 || s == S)
        for index = [1,2,3]
            reject(1,index,1) = mean(rej_k(1,index,:),3);
            reject(2,index,1) = mean(rej_k(2,index,:),3);
            reject(3,index,1) = mean(rej_k(3,index,:),3);
            reject(4,index,1) = mean(rej_k(4,index,:),3);
            reject(5,index,1) = mean(rej_k(5,index,:),3);
            reject(6,index,1) = mean(rej_k(6,index,:),3);
            reject(7,index,1) = mean(rej_k(7,index,:),3);
            
            reject(1,index,2) = mean(rej_s(1,index,:),3);
            reject(2,index,2) = mean(rej_s(2,index,:),3);
            reject(3,index,2) = mean(rej_s(3,index,:),3);
            reject(4,index,2) = mean(rej_s(4,index,:),3);
            reject(5,index,2) = mean(rej_s(5,index,:),3);
            reject(6,index,2) = mean(rej_s(6,index,:),3);
            reject(7,index,2) = mean(rej_s(7,index,:),3);
            
            reject(1,index,3) = mean(rej_GH(1,index,:),3);
            reject(2,index,3) = mean(rej_GH(2,index,:),3);
            reject(3,index,3) = mean(rej_GH(3,index,:),3);
            reject(4,index,3) = mean(rej_GH(4,index,:),3);
            reject(5,index,3) = mean(rej_GH(5,index,:),3);
            reject(6,index,3) = mean(rej_GH(6,index,:),3);
            reject(7,index,3) = mean(rej_GH(7,index,:),3);
            
            reject_ML(1,index,1) = mean(rej_k_ML(1,index,:),3);
            reject_ML(2,index,1) = mean(rej_k_ML(2,index,:),3);
            reject_ML(3,index,1) = mean(rej_k_ML(3,index,:),3);
            reject_ML(4,index,1) = mean(rej_k_ML(4,index,:),3);
            reject_ML(5,index,1) = mean(rej_k_ML(5,index,:),3);
            reject_ML(6,index,1) = mean(rej_k_ML(6,index,:),3);
            reject_ML(7,index,1) = mean(rej_k_ML(7,index,:),3);
            
            reject_ML(1,index,2) = mean(rej_s_ML(1,index,:),3);
            reject_ML(2,index,2) = mean(rej_s_ML(2,index,:),3);
            reject_ML(3,index,2) = mean(rej_s_ML(3,index,:),3);
            reject_ML(4,index,2) = mean(rej_s_ML(4,index,:),3);
            reject_ML(5,index,2) = mean(rej_s_ML(5,index,:),3);
            reject_ML(6,index,2) = mean(rej_s_ML(6,index,:),3);
            reject_ML(7,index,2) = mean(rej_s_ML(7,index,:),3);
            
            reject_ML(1,index,3) = mean(rej_GH_ML(1,index,:),3);
            reject_ML(2,index,3) = mean(rej_GH_ML(2,index,:),3);
            reject_ML(3,index,3) = mean(rej_GH_ML(3,index,:),3);
            reject_ML(4,index,3) = mean(rej_GH_ML(4,index,:),3);
            reject_ML(5,index,3) = mean(rej_GH_ML(5,index,:),3);
            reject_ML(6,index,3) = mean(rej_GH_ML(6,index,:),3);
            reject_ML(7,index,3) = mean(rej_GH_ML(7,index,:),3);
            
            rejectKT(1,index,1) = mean(rej_KTk(1,index,:),3);
            rejectKT(2,index,1) = mean(rej_KTk(2,index,:),3);
            rejectKT(3,index,1) = mean(rej_KTk(3,index,:),3);
            rejectKT(4,index,1) = mean(rej_KTk(4,index,:),3);
            rejectKT(5,index,1) = mean(rej_KTk(5,index,:),3);
            rejectKT(6,index,1) = mean(rej_KTk(6,index,:),3);
            rejectKT(7,index,1) = mean(rej_KTk(7,index,:),3);
            
            rejectKT(1,index,2) = mean(rej_KTGH(1,index,:),3);
            rejectKT(2,index,2) = mean(rej_KTGH(2,index,:),3);
            rejectKT(3,index,2) = mean(rej_KTGH(3,index,:),3);
            rejectKT(4,index,2) = mean(rej_KTGH(4,index,:),3);
            rejectKT(5,index,2) = mean(rej_KTGH(5,index,:),3);
            rejectKT(6,index,2) = mean(rej_KTGH(6,index,:),3);
            rejectKT(7,index,2) = mean(rej_KTGH(7,index,:),3);
            
            rejectKT_ML(1,index,1) = mean(rej_KTk_ML(1,index,:),3);
            rejectKT_ML(2,index,1) = mean(rej_KTk_ML(2,index,:),3);
            rejectKT_ML(3,index,1) = mean(rej_KTk_ML(3,index,:),3);
            rejectKT_ML(4,index,1) = mean(rej_KTk_ML(4,index,:),3);
            rejectKT_ML(5,index,1) = mean(rej_KTk_ML(5,index,:),3);
            rejectKT_ML(6,index,1) = mean(rej_KTk_ML(6,index,:),3);
            rejectKT_ML(7,index,1) = mean(rej_KTk_ML(7,index,:),3);
            
            rejectKT_ML(1,index,2) = mean(rej_KTGH_ML(1,index,:),3);
            rejectKT_ML(2,index,2) = mean(rej_KTGH_ML(2,index,:),3);
            rejectKT_ML(3,index,2) = mean(rej_KTGH_ML(3,index,:),3);
            rejectKT_ML(4,index,2) = mean(rej_KTGH_ML(4,index,:),3);
            rejectKT_ML(5,index,2) = mean(rej_KTGH_ML(5,index,:),3);
            rejectKT_ML(6,index,2) = mean(rej_KTGH_ML(6,index,:),3);
            rejectKT_ML(7,index,2) = mean(rej_KTGH_ML(7,index,:),3);
            
        end
        
        save btest.mat s reject reject_ML rejectKT rejectKT_ML
    end
    waitbar(s/S,h)
end
close(h)

movefile('btest.mat',['IO/bootstrap/', title,'_bootstrap_test_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat']);