function epsilon = Asym_t_rnd(param,T,N,dist)
% epsilon = simul_epsilonN2(param,T,R,dist)
%
% Inputs: [param,N,T,dist]
%       param are the parameters that characterize the distribution
%       N is the cross sectional dimension of the model (scalar)
%       T is the times/series dimension of the model (scalar)
%       dist defines the distribution to be simulated
%           if dist = 1, then Gaussian innovations and 'param' is zero
%           if dist = 2, then Student t innovations and 'param' is 'nu'
%           if dist = 3, then Asymmetric t distribution and 'param' is '[nu,beta]'
%
% Output: epsilon, a T x N vector of unit variance
% Author: Julio Crego
% I modified the SqRR matrix and replaced the r argument by N.
% This function needs to be tested for N >= 3.

SqRR = eye(N);
if dist == 1
    % Gaussian innovations, 'param' is zero
	epsilon = randn(T,N);
    epsilon = epsilon*SqRR; 
elseif dist == 2
    % Student t innovations, 'param' is 'nu'
    v = param;
	epsilon = (((v-2)/v)^0.5)*mvtrnd_gaminv(eye(N),v,T);
    epsilon = epsilon*SqRR; 
elseif dist == 3
	% Asymmetric t distribution, 'param' is '[nu,beta]'
    v2 = param(1)/2;   
    beta = (param(2:N+1)*SqRR)';   
    eta = 0.5/v2;
    c = (-(1-4*eta) + sqrt(((1-4*eta)^2) + ...
        8*(beta'*beta)*(1-4*eta)*eta))/(4*(beta'*beta)*eta);
    upsilon = (eye(N) + beta*beta'*((c-1)/(beta'*beta))) * (1-2*eta)/eta;
    alfa = -c*beta;
    ut = randn(T,N); 
    u = rand(T,1);
    xsi = gaminv(u,v2,2);
    epsilon = zeros(T,N);
    
    for j = 1:T  
        epsilon(j,:) = (alfa' + beta'*upsilon'*(xsi(j)^(-1)) + ...
                       (1/sqrt(xsi(j)))*(ut(j,:)*(upsilon^0.5)'))*SqRR;   
    end
end