function s = ScoreFS(e,V)
% DESCRIPTION
% This function computes the skewness component of the filtered score with
% respect to the shape parameters.
%
% INPUT
% e = [e_(1|0) ... e_(T|T-1)] is (N,T) and contains filtered errors, all
% non-Gaussian under the alternative, e_(t|t-1) = LSP[e_t|y_1 ... y_(t-1)],
% V = [V_(1|0) ... V_(T|T-1)] is (N,N,T) and contains the filter
% mean squared errors, V_(t|t-1) = E[(e_t-e_(t|t-1))(e_t-e_(t|t-1))'].
%
% OUTPUT
% s = [s_(1|0) ... s_(T|T-1)] is (N,T) and contains filtered contributions
% to the skewness component of the score with respect to shape parameters
% in each of the asymmetry directions.
%
% INFORMATION
% Authors: Almuzara, Amengual, Sentana.
% Date: January 2016.


%% 01. Settings and pre-allocation

[N,T] = size(e);

c3 = -(N+2);

s = zeros(N,T);


%% 02. Computation of the filtered contributions to the skewness-score

for t = 1:T
    aux_sigma = e(:,t)'*pinv(V(:,:,t))*e(:,t);
    s(:,t) = e(:,t) * (aux_sigma + c3);
end