clear
clc
close all
path([pwd, '/subprograms/'], path)


%% SETTINGS AND PREALLOCATION

% Simulation settings
S = 10000; % Number of Monte Carlo replicas
safe = ceil(0.05*S); % Number of replicas per saving at each stage
title = 'S622';

rng(261009)
options = optimoptions('fmincon','Algorithm','interior-point',...
    'GradObj','off','Hessian','off','Display','iter',...
    'MaxFunEvals',1000,'MaxIter',1000,'TolCon',0.0,'TolFun',10e-9);
maxerror = 1e-4; % Tolerance criterion for numerical convergence
maxiter = 1e2; % Maximum number of iterations for numerical convergence

% Model settings
T = 100; % Sample size
hypothesis = 0; % Distribution of innovations

c = 0;
q = 2;
parameters = [c;1;1/q];

nu = 8; % Degrees of freedom
beta = [-1,-1]; % Direction of asymmetry

N = 1;
M = 2;
K = 2;
d = size(parameters,1);

% Preallocation
Data = zeros(N,T,S);                                                       % Simulated data processes
MLestimate = zeros(d+2,S);                                                 % ML estimates
Var_MLestimate = zeros(d+2,d+2,S);                                         % Estimated variance of ML estimates

Scr_k0 = zeros(1,S);                                                       % Case-0 score of kurtosis component at true parameter value
Scr_k1 = zeros(1,S);                                                       % Case-1 score of kurtosis component at true parameter value
Scr_k2 = zeros(1,S);                                                       % Case-2 score of kurtosis component at true parameter value
Scr_k3 = zeros(1,S);                                                       % Case-3 score of kurtosis component at true parameter value

MSE = zeros(N,N,S);
Scr_s0 = zeros(N,S);                                                       % Case-0 score of skewness component at true parameter value
Scr_s1 = zeros(K,S);                                                       % Case-1 score of skewness component at true parameter value
Scr_s2 = zeros(K-1,S);                                                     % Case-2 score of skewness component at true parameter value
Scr_s3 = zeros(1,S);                                                       % Case-3 score of skewness component at true parameter value

Stat_k0 = zeros(1,S);                                                      % Case-0 statistic of kurtosis component at true parameter value
Stat_k1 = zeros(1,S);                                                      % Case-1 statistic of kurtosis component at true parameter value
Stat_k2 = zeros(1,S);                                                      % Case-2 statistic of kurtosis component at true parameter value
Stat_k3 = zeros(1,S);                                                      % Case-3 statistic of kurtosis component at true parameter value
Stat_k4 = zeros(1,S);                                                      % Case-4 statistic of kurtosis component at true parameter value

Stat_s0 = zeros(1,S);                                                      % Case-0 statistic of skewness component at true parameter value
Stat_s1 = zeros(1,S);                                                      % Case-1 statistic of skewness component at true parameter value
Stat_s2 = zeros(1,S);                                                      % Case-2 statistic of skewness component at true parameter value
Stat_s3 = zeros(1,S);                                                      % Case-3 statistic of skewness component at true parameter value
Stat_s4 = zeros(1,S);                                                      % Case-4 statistic of skewness component at true parameter value

Stat_GH0 = zeros(1,S);                                                     % Case-0 full statistic at true parameter value
Stat_GH1 = zeros(1,S);                                                     % Case-1 full statistic at true parameter value
Stat_GH2 = zeros(1,S);                                                     % Case-2 full statistic at true parameter value
Stat_GH3 = zeros(1,S);                                                     % Case-3 full statistic at true parameter value
Stat_GH4 = zeros(1,S);                                                     % Case-4 full statistic at true parameter value

StatKT_k0 = zeros(1,S);                                                    % Case-0 KT statistic of kurtosis component at true parameter value
StatKT_k1 = zeros(1,S);                                                    % Case-1 KT statistic of kurtosis component at true parameter value
StatKT_k2 = zeros(1,S);                                                    % Case-2 KT statistic of kurtosis component at true parameter value
StatKT_k3 = zeros(1,S);                                                    % Case-3 KT statistic of kurtosis component at true parameter value
StatKT_k4 = zeros(1,S);                                                    % Case-4 KT statistic of kurtosis component at true parameter value

StatKT_GH0 = zeros(1,S);                                                   % Case-0 KT full statistic at true parameter value
StatKT_GH1 = zeros(1,S);                                                   % Case-1 KT full statistic at true parameter value
StatKT_GH2 = zeros(1,S);                                                   % Case-2 KT full statistic at true parameter value
StatKT_GH3 = zeros(1,S);                                                   % Case-3 KT full statistic at true parameter value
StatKT_GH4 = zeros(1,S);                                                   % Case-4 KT full statistic at true parameter value

Scr_k0_ML = zeros(1,S);                                                    % Case-0 score of kurtosis component at ML estimate
Scr_k1_ML = zeros(1,S);                                                    % Case-1 score of kurtosis component at ML estimate
Scr_k2_ML = zeros(1,S);                                                    % Case-2 score of kurtosis component at ML estimate
Scr_k3_ML = zeros(1,S);                                                    % Case-3 score of kurtosis component at ML estimate

MSE_ML = zeros(N,N,S);
Scr_s0_ML = zeros(N,S);                                                    % Case-0 score of skewness component at ML estimate
Scr_s1_ML = zeros(K,S);                                                    % Case-1 score of skewness component at ML estimate
Scr_s2_ML = zeros(K-1,S);                                                  % Case-2 score of skewness component at ML estimate
Scr_s3_ML = zeros(1,S);                                                    % Case-3 score of skewness component at ML estimate

Var_k1_ML = zeros(1,S);                                                    % Case-1 asymptotic variance of kurtosis component at ML estimate
Var_k2_ML = zeros(1,S);                                                    % Case-2 asymptotic variance of kurtosis component at ML estimate
Var_k3_ML = zeros(1,S);                                                    % Case-3 asymptotic variance of kurtosis component at ML estimate
Var_k4_ML = zeros(K,K,S);                                                  % Case-4 asymptotic variance of kurtosis component at ML estimate

Var_s1_ML = zeros(K,K,S);                                                  % Case-1 asymptotic variance of skewness component at ML estimate
Var_s2_ML = zeros(K-1,K-1,S);                                              % Case-2 asymptotic variance of skewness component at ML estimate
Var_s3_ML = zeros(1,1,S);                                                  % Case-3 asymptotic variance of skewness component at ML estimate
Var_s4_ML = zeros(K,K,S);                                                  % Case-4 asymptotic variance of skewness component at ML estimate

Stat_k0_ML = zeros(1,S);                                                   % Case-0 statistic of kurtosis component at ML estimate
Stat_k1_ML = zeros(1,S);                                                   % Case-1 statistic of kurtosis component at ML estimate
Stat_k2_ML = zeros(1,S);                                                   % Case-2 statistic of kurtosis component at ML estimate
Stat_k3_ML = zeros(1,S);                                                   % Case-3 statistic of kurtosis component at ML estimate
Stat_k4_ML = zeros(1,S);                                                   % Case-4 statistic of kurtosis component at ML estimate

Stat_s0_ML = zeros(1,S);                                                   % Case-0 statistic of skewness component at ML estimate
Stat_s1_ML = zeros(1,S);                                                   % Case-1 statistic of skewness component at ML estimate
Stat_s2_ML = zeros(1,S);                                                   % Case-2 statistic of skewness component at ML estimate
Stat_s3_ML = zeros(1,S);                                                   % Case-3 statistic of skewness component at ML estimate
Stat_s4_ML = zeros(1,S);                                                   % Case-4 statistic of skewness component at ML estimate

Stat_GH0_ML = zeros(1,S);                                                  % Case-0 full statistic at ML estimate
Stat_GH1_ML = zeros(1,S);                                                  % Case-1 full statistic at ML estimate
Stat_GH2_ML = zeros(1,S);                                                  % Case-2 full statistic at ML estimate
Stat_GH3_ML = zeros(1,S);                                                  % Case-3 full statistic at ML estimate
Stat_GH4_ML = zeros(1,S);                                                  % Case-4 full statistic at ML estimate

StatKT_k0_ML = zeros(1,S);                                                 % Case-0 KT statistic of kurtosis component at ML estimate
StatKT_k1_ML = zeros(1,S);                                                 % Case-1 KT statistic of kurtosis component at ML estimate
StatKT_k2_ML = zeros(1,S);                                                 % Case-2 KT statistic of kurtosis component at ML estimate
StatKT_k3_ML = zeros(1,S);                                                 % Case-3 KT statistic of kurtosis component at ML estimate
StatKT_k4_ML = zeros(1,S);                                                 % Case-4 KT statistic of kurtosis component at ML estimate

StatKT_GH0_ML = zeros(1,S);                                                % Case-0 full KT statistic at ML estimate
StatKT_GH1_ML = zeros(1,S);                                                % Case-1 full KT statistic at ML estimate
StatKT_GH2_ML = zeros(1,S);                                                % Case-2 full KT statistic at ML estimate
StatKT_GH3_ML = zeros(1,S);                                                % Case-3 full KT statistic at ML estimate
StatKT_GH4_ML = zeros(1,S);                                                % Case-4 full KT statistic at ML estimate

HK_k2 = zeros(1,S);                                                        % Harvey-Koopmans case-2 mean of kurtosis at true parameter value
HK_k3 = zeros(1,S);                                                        % Harvey-Koopmans case-2 mean of kurtosis at true parameter value
HK_s2 = zeros(1,S);                                                        % Harvey-Koopmans case-3 mean of skewness at true parameter value
HK_s3 = zeros(1,S);                                                        % Harvey-Koopmans case-3 mean of skewness at true parameter value

Stat_HKk2 = zeros(1,S);                                                    % Harvey-Koopmans case-2 kurtosis statistic at true parameter value
Stat_HKk3 = zeros(1,S);                                                    % Harvey-Koopmans case-3 kurtosis statistic at true parameter value
Stat_HKs2 = zeros(1,S);                                                    % Harvey-Koopmans case-2 skewness statistic at true parameter value
Stat_HKs3 = zeros(1,S);                                                    % Harvey-Koopmans case-3 skewness statistic at true parameter value
Stat_HKGH2 = zeros(1,S);                                                   % Harvey-Koopmans case-2 full statistic at true parameter value
Stat_HKGH3 = zeros(1,S);                                                   % Harvey-Koopmans case-3 full statistic at true parameter value

HK_k2_ML = zeros(1,S);                                                     % Harvey-Koopmans case-2 mean of kurtosis at ML estimate
HK_k3_ML = zeros(1,S);                                                     % Harvey-Koopmans case-3 mean of kurtosis at ML estimate
HK_s2_ML = zeros(1,S);                                                     % Harvey-Koopmans case-2 mean of skewness at ML estimate
HK_s3_ML = zeros(1,S);                                                     % Harvey-Koopmans case-3 mean of skewness at ML estimate

Var_HKk2_ML = zeros(1,S);                                                  % Harvey-Koopmans case-2 variance of kurtosis at ML estimate
Var_HKk3_ML = zeros(1,S);                                                  % Harvey-Koopmans case-3 variance of kurtosis at ML estimate
Var_HKs2_ML = zeros(1,S);                                                  % Harvey-Koopmans case-2 variance of skewness at ML estimate
Var_HKs3_ML = zeros(1,S);                                                  % Harvey-Koopmans case-3 variance of skewness at ML estimate

Stat_HKk2_ML = zeros(1,S);                                                 % Harvey-Koopmans case-2 kurtosis statistic at ML estimate
Stat_HKk3_ML = zeros(1,S);                                                 % Harvey-Koopmans case-3 kurtosis statistic at ML estimate
Stat_HKs2_ML = zeros(1,S);                                                 % Harvey-Koopmans case-2 skewness statistic at ML estimate
Stat_HKs3_ML = zeros(1,S);                                                 % Harvey-Koopmans case-3 skewness statistic at ML estimate
Stat_HKGH2_ML = zeros(1,S);                                                % Harvey-Koopmans case-2 full statistic at ML estimate
Stat_HKGH3_ML = zeros(1,S);                                                % Harvey-Koopmans case-3 full statistic at ML estimate

rejection = zeros(7,3,3);
rejection_ML = zeros(7,3,3);
rejectionKT = zeros(5,3,2);
rejectionKT_ML = zeros(5,3,2);


%% SIMULATION AND ESTIMATION

h = waitbar(0,'Simulation and estimation');

for s = 1:S
    
    display(s,'Replica')
    
    kappa = 10e5;
    C = [parameters(1);...
        0];
    F = [1,0;...
        0,0];
    G = diag([sqrt(parameters(2)),sqrt(parameters(3))]);
    H = [1,1];
    EX1 = zeros(2,1);
    VX1 = diag([kappa*parameters(2),parameters(3)]);
    
    X = zeros(M,T);
    Y = zeros(N,T);
    
    X1 = mvnrnd(EX1,VX1)';
    
    X(:,1) = X1;
    Y(:,1) = H*X1;
    
    if (hypothesis == 0)
        for t = 2:T
            X(:,t) = C + F*X(:,t-1) + G*randn(K,1);
            Y(:,t) = H*X(:,t);
        end
    elseif (hypothesis == 1)
        for t = 2:T
            X(:,t) = C + F*X(:,t-1) + ...
                G*(sqrt((nu-2)/nu))*mvtrnd(eye(K),nu)';
            Y(:,t) = H*X(:,t);
        end
    elseif (hypothesis == 2)
        for t = 2:T
            X(:,t) = C + F*X(:,t-1) + ...
                G*[randn(1,1);sqrt((nu-2)/nu)*mvtrnd(1,nu)];
            Y(:,t) = H*X(:,t);
        end
    elseif (hypothesis == 3)
        for t = 2:T
            X(:,t) = C + F*X(:,t-1) + ...
                G*[sqrt((nu-2)/nu)*mvtrnd(1,nu);randn(1,1)];
            Y(:,t) = H*X(:,t);
        end
    elseif (hypothesis == 4)
        for t = 2:T
            X(:,t) = C + F*X(:,t-1) + ...
                G*sqrt((nu-2)/nu)*[mvtrnd(1,nu);mvtrnd(1,nu)];
            Y(:,t) = H*X(:,t);
        end
    elseif (hypothesis == 5)
        e = Asym_t_rnd([nu,beta],T-1,2,3);
        for t = 2:T
            X(:,t) = C + F*X(:,t-1) + ...
                G*e(t-1,:)';
            Y(:,t) = H*X(:,t);
        end
    elseif (hypothesis == 6)
        e = Asym_t_rnd([nu,beta(1)],T-1,1,3);
        for t = 2:T
            X(:,t) = C + F*X(:,t-1) + ...
                G*[randn(1,1);e(t-1,:)'];
            Y(:,t) = H*X(:,t);
        end
    elseif (hypothesis == 7)
        e = Asym_t_rnd([nu,beta(2)],T-1,1,3);
        for t = 2:T
            X(:,t) = C + F*X(:,t-1) + ...
                G*[e(t-1,:)';randn(1,1)];
            Y(:,t) = H*X(:,t);
        end
    elseif (hypothesis == 8)
        e = [Asym_t_rnd([nu,beta(1)],T-1,1,3),...
            Asym_t_rnd([nu,beta(2)],T-1,1,3)];
        for t = 2:T
            X(:,t) = C + F*X(:,t-1) + ...
                G*e(t-1,:)';
            Y(:,t) = H*X(:,t);
        end
    else
        disp('WARNING: hypothesis not valid.')
        close(h)
        return
    end
    
    Data(:,:,s) = Y;
    
    [MLestimate(:,s),Var_MLestimate(:,:,s)] = UN_Estimation(...
        parameters(1),parameters(2),parameters(3),Y,options);
    
    waitbar(s/S,h)
    
    % Saving results
    if (rem(s, safe) == 0 || s == S)
        save estimate.mat s ...
            Data MLestimate Var_MLestimate
    end
    
end
close(h)

movefile('estimate.mat',['IO/estimate/', title,'_estimate_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat']);


%% COMPUTATION OF SCORE

h = waitbar(0,'Computation of score');
load(['IO/estimate/', title,'_estimate_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat'])

for s = 1:S
    
    display(s,'Replica')
    
    Y = Data(:,:,s);
    
    C = [parameters(1);...
        zeros(3,1)];
    F = [1,zeros(1,3);...
        zeros(3,4)];
    G = [diag([sqrt(parameters(2)),sqrt(parameters(3))]);...
        1,0;...
        0,1];
    H = [1,1,0,0];
    EX1 = zeros(4,1);
    VX1 = [diag([kappa*parameters(2),parameters(3)]),zeros(2,2);...
        zeros(2,2),eye(2)];
    
    C_ML = [MLestimate(1,s);...
        zeros(3,1)];
    G_ML = [diag([sqrt(MLestimate(2,s)),sqrt(MLestimate(3,s))]);...
        1,0;...
        0,1];
    VX1_ML = [diag([kappa*MLestimate(2,s),MLestimate(3,s)]),...
        zeros(2,2);...
        zeros(2,2),...
        eye(2)];
    
    [Xf,Pf,Xs,Ps] = KalmanFS(EX1,VX1,F,G,H,Y,C);
    
    e0 = zeros(N,T-2);
    P0 = zeros(N,N,T-2);
    for t = 2:T-1
        e0(:,t-1) = Y(:,t) - H*Xf(:,t);
        P0(:,:,t-1) = H*Pf(:,:,t)*H';
    end
    e1 = Xs(M+1:M+K,2:T-1);
    P1 = Ps(M+1:M+K,M+1:M+K,2:T-1);
    e2 = Xs(M+2:M+K,2:T-1);
    P2 = Ps(M+2:M+K,M+2:M+K,2:T-1);
    e3 = Xs(M+1,2:T-1);
    P3 = Ps(M+1,M+1,2:T-1);
    
    [Xf_ML,Pf_ML,Xs_ML,Ps_ML] = KalmanFS(EX1,VX1_ML,F,G_ML,H,Y,C_ML);
    
    e0_ML = zeros(N,T-2);
    P0_ML = zeros(N,N,T-2);
    for t = 2:T-1
        e0_ML(:,t-1) = Y(:,t) - H*Xf_ML(:,t);
        P0_ML(:,:,t-1) = H*Pf_ML(:,:,t)*H';
    end
    e1_ML = Xs_ML(M+1:M+K,2:T-1);
    P1_ML = Ps_ML(M+1:M+K,M+1:M+K,2:T-1);
    e2_ML = Xs_ML(M+2:M+K,2:T-1);
    P2_ML = Ps_ML(M+2:M+K,M+2:M+K,2:T-1);
    e3_ML = Xs_ML(M+1,2:T-1);
    P3_ML = Ps_ML(M+1,M+1,2:T-1);
    
    % Computation of score
    sk0 = ScoreFK(e0,P0);
    sk1 = ScoreSK(e1,P1);
    sk2 = ScoreSK(e2,P2);
    sk3 = ScoreSK(e3,P3);
    
    ss0 = ScoreFS(e0,P0);
    ss1 = ScoreSS(e1,P1);
    ss2 = ScoreSS(e2,P2);
    ss3 = ScoreSS(e3,P3);
    
    Scr_k0(:,s) = mean(sk0,2);
    Scr_k1(:,s) = mean(sk1,2);
    Scr_k2(:,s) = mean(sk2,2);
    Scr_k3(:,s) = mean(sk3,2);
    
    MSE(:,:,s) = mean(P0,3);
    
    Scr_s0(:,s) = mean(ss0,2);
    Scr_s1(:,s) = mean(ss1,2);
    Scr_s2(:,s) = mean(ss2,2);
    Scr_s3(:,s) = mean(ss3,2);
    
    sk0_ML = ScoreFK(e0_ML,P0_ML);
    sk1_ML = ScoreSK(e1_ML,P1_ML);
    sk2_ML = ScoreSK(e2_ML,P2_ML);
    sk3_ML = ScoreSK(e3_ML,P3_ML);
    
    ss0_ML = ScoreFS(e0_ML,P0_ML);
    ss1_ML = ScoreSS(e1_ML,P1_ML);
    ss2_ML = ScoreSS(e2_ML,P2_ML);
    ss3_ML = ScoreSS(e3_ML,P3_ML);
    
    Scr_k0_ML(:,s) = mean(sk0_ML,2);
    Scr_k1_ML(:,s) = mean(sk1_ML,2);
    Scr_k2_ML(:,s) = mean(sk2_ML,2);
    Scr_k3_ML(:,s) = mean(sk3_ML,2);
    
    MSE_ML(:,:,s) = mean(P0_ML,3);
    
    Scr_s0_ML(:,s) = mean(ss0_ML,2);
    Scr_s1_ML(:,s) = mean(ss1_ML,2);
    Scr_s2_ML(:,s) = mean(ss2_ML,2);
    Scr_s3_ML(:,s) = mean(ss3_ML,2);
    
    HK_k2(:,s) = StatHKK(e2);
    HK_k3(:,s) = StatHKK(e3);
    
    HK_s2(:,s) = StatHKS(e2);
    HK_s3(:,s) = StatHKS(e3);
    
    HK_k2_ML(:,s) = StatHKK(e2_ML);
    HK_k3_ML(:,s) = StatHKK(e3_ML);
    
    HK_s2_ML(:,s) = StatHKS(e2_ML);
    HK_s3_ML(:,s) = StatHKS(e3_ML);
    
    waitbar(s/S,h)
    
    % Saving results
    if (rem(s, safe) == 0 || s == S)
        save score.mat s ...
            Scr_k0 Scr_k1 Scr_k2 Scr_k3 ...
            MSE Scr_s0 Scr_s1 Scr_s2 Scr_s3 ...
            Scr_k0_ML Scr_k1_ML Scr_k2_ML Scr_k3_ML ...
            MSE_ML Scr_s0_ML Scr_s1_ML Scr_s2_ML Scr_s3_ML ...
            HK_k2 HK_k3 HK_s2 HK_s3 ...
            HK_k2_ML HK_k3_ML HK_s2_ML HK_s3_ML
    end
    
end
close(h)

movefile('score.mat',['IO/score/', title,'_score_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat']);


%% COMPUTATION OF ASYMPTOTIC VARIANCE

h = waitbar(0,'Computation of asymptotic variance');
load(['IO/estimate/', title,'_estimate_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat'])

gx = parameters(2);
gv = parameters(3);

[PHI,THETA] = UN_WKfilter(gx,gv);
auxGAMMA0 = AutocovarianceWK(PHI,THETA,eye(K),0);

r = K;
[~,aux_bs,~] = ScoreSS(zeros(K,1),eye(K) - auxGAMMA0);
[~,aux_bk,~] = ScoreSK(zeros(K,1),eye(K) - auxGAMMA0);

B_e1_1 = aux_bs(1:r,:);
B_e2_1 = aux_bk(2:1+r^2);
B_e3_1 = aux_bs(r+1:end,:);
B_e4_1 = aux_bk(2+r^2:end);

r = K-1;
E2 = [zeros(K-1,1),eye(K-1)];
[~,aux_bs,~] = ScoreSS(zeros(K-1,1),eye(K-1) - auxGAMMA0(2:K,2:K));
[~,aux_bk,~] = ScoreSK(zeros(K-1,1),eye(K-1) - auxGAMMA0(2:K,2:K));

B_e1_2 = E2'*aux_bs(1:r,:);
B_e2_2 = kron(E2',E2')*aux_bk(2:1+r^2);
B_e3_2 = kron(E2',E2')*aux_bs(r+1:end,:);
B_e4_2 = kron(E2',E2')*aux_bk(2+r^2:end);

E3 = [1,zeros(1,K-1)];
[~,aux_bs,~] = ScoreSS(0,1 - auxGAMMA0(1,1));
[~,aux_bk,~] = ScoreSK(0,1 - auxGAMMA0(1,1));

B_e1_3 = E3'*aux_bs(1:r,:);
B_e2_3 = kron(E3',E3')*aux_bk(2:1+r^2);
B_e3_3 = kron(E3',E3')*aux_bs(r+1:end,:);
B_e4_3 = kron(E3',E3')*aux_bk(2+r^2:end);

B_e1_4 = [B_e1_2,B_e1_3];
B_e2_4 = [B_e2_2,B_e2_3];
B_e3_4 = [B_e3_2,B_e3_3];
B_e4_4 = [B_e4_2,B_e4_3];

[CMe1e1,CMe2e2,CMe3e3,CMe4e4] = AutocovarianceMM(auxGAMMA0,auxGAMMA0);

AGFe1e1 = CMe1e1;
AGFe2e2 = CMe2e2;
AGFe3e3 = CMe3e3;
AGFe4e4 = CMe4e4;

j = 0;
error = 1;

while ((j < maxiter) && (error >= maxerror))
    
    auxGAMMAj = AutocovarianceWK(PHI,THETA,eye(K),j+1);
    [CMe1e1,CMe2e2,CMe3e3,CMe4e4] = ...
        AutocovarianceMM(auxGAMMA0,auxGAMMAj);
    [CMe1e1m,CMe2e2m,CMe3e3m,CMe4e4m] = ...
        AutocovarianceMM(auxGAMMA0,auxGAMMAj');
    
    AGFe1e1 = AGFe1e1 + CMe1e1 + CMe1e1m;
    AGFe2e2 = AGFe2e2 + CMe2e2 + CMe2e2m;
    AGFe3e3 = AGFe3e3 + CMe3e3 + CMe3e3m;
    AGFe4e4 = AGFe4e4 + CMe4e4 + CMe4e4m;
    
    j = j+1;
    
    de1e1 = max(max(abs(CMe1e1 + CMe1e1m)));
    de2e2 = max(max(abs(CMe2e2 + CMe2e2m)));
    de3e3 = max(max(abs(CMe3e3 + CMe3e3m)));
    de4e4 = max(max(abs(CMe4e4 + CMe4e4m)));
    error = max([de1e1,de2e2,de3e3,de4e4]);
    
    display([j,error],'Iteration')
end

Var_k1 = (B_e4_1')*AGFe4e4*(B_e4_1) - (B_e2_1')*AGFe2e2*(B_e2_1);
Var_k2 = (B_e4_2')*AGFe4e4*(B_e4_2) - (B_e2_2')*AGFe2e2*(B_e2_2);
Var_k3 = (B_e4_3')*AGFe4e4*(B_e4_3) - (B_e2_3')*AGFe2e2*(B_e2_3);
Var_k4 = (B_e4_4')*AGFe4e4*(B_e4_4) - (B_e2_4')*AGFe2e2*(B_e2_4);
Var_s1 = (B_e3_1')*AGFe3e3*(B_e3_1) - (B_e1_1')*AGFe1e1*(B_e1_1);
Var_s2 = (B_e3_2')*AGFe3e3*(B_e3_2) - (B_e1_2')*AGFe1e1*(B_e1_2);
Var_s3 = (B_e3_3')*AGFe3e3*(B_e3_3) - (B_e1_3')*AGFe1e1*(B_e1_3);
Var_s4 = (B_e3_4')*AGFe3e3*(B_e3_4) - (B_e1_4')*AGFe1e1*(B_e1_4);

kappa_k2 = auxGAMMA0(2,2)^4;
kappa_k3 = auxGAMMA0(1,1)^4;
kappa_s2 = auxGAMMA0(2,2)^3;
kappa_s3 = auxGAMMA0(1,1)^3;

j = 0;
error = 1;

while ((j < maxiter) && (error >= maxerror))
    
    auxGAMMAj = AutocovarianceWK(PHI,THETA,eye(K),j+1);
    kappa_k2 = kappa_k2 + 2*auxGAMMAj(2,2)^4;
    kappa_k3 = kappa_k3 + 2*auxGAMMAj(1,1)^4;
    kappa_s2 = kappa_s2 + 2*auxGAMMAj(2,2)^3;
    kappa_s3 = kappa_s3 + 2*auxGAMMAj(1,1)^3;
    
    j = j+1;
    error = max(trace(auxGAMMAj.^4),trace(auxGAMMAj.^3));
    display([j,error],'Iteration')
end

Var_HKk2 = kappa_k2/auxGAMMA0(2,2)^4;
Var_HKk3 = kappa_k3/auxGAMMA0(1,1)^4;
Var_HKs2 = kappa_s2/auxGAMMA0(2,2)^3;
Var_HKs3 = kappa_s3/auxGAMMA0(1,1)^3;

% Computation of asymptotic variance
for s = 1:S
    
    display(s,'Replica')
    
    gx = MLestimate(2,s);
    gv = MLestimate(3,s);
    
    [PHI,THETA] = UN_WKfilter(gx,gv);
    auxGAMMA0 = AutocovarianceWK(PHI,THETA,eye(K),0);
    
    r = K;
    [~,aux_bs,~] = ScoreSS(zeros(K,1),eye(K) - auxGAMMA0);
    [~,aux_bk,~] = ScoreSK(zeros(K,1),eye(K) - auxGAMMA0);
    
    B_e1_1 = aux_bs(1:r,:);
    B_e2_1 = aux_bk(2:1+r^2);
    B_e3_1 = aux_bs(r+1:end,:);
    B_e4_1 = aux_bk(2+r^2:end);
    
    r = K-1;
    E2 = [zeros(K-1,1),eye(K-1)];
    [~,aux_bs,~] = ScoreSS(zeros(K-1,1),eye(K-1) - auxGAMMA0(2:K,2:K));
    [~,aux_bk,~] = ScoreSK(zeros(K-1,1),eye(K-1) - auxGAMMA0(2:K,2:K));
    
    B_e1_2 = E2'*aux_bs(1:r,:);
    B_e2_2 = kron(E2',E2')*aux_bk(2:1+r^2);
    B_e3_2 = kron(E2',E2')*aux_bs(r+1:end,:);
    B_e4_2 = kron(E2',E2')*aux_bk(2+r^2:end);
    
    E3 = [1,zeros(1,K-1)];
    [~,aux_bs,~] = ScoreSS(0,1 - auxGAMMA0(1,1));
    [~,aux_bk,~] = ScoreSK(0,1 - auxGAMMA0(1,1));
    
    B_e1_3 = E3'*aux_bs(1:r,:);
    B_e2_3 = kron(E3',E3')*aux_bk(2:1+r^2);
    B_e3_3 = kron(E3',E3')*aux_bs(r+1:end,:);
    B_e4_3 = kron(E3',E3')*aux_bk(2+r^2:end);
    
    B_e1_4 = [B_e1_2,B_e1_3];
    B_e2_4 = [B_e2_2,B_e2_3];
    B_e3_4 = [B_e3_2,B_e3_3];
    B_e4_4 = [B_e4_2,B_e4_3];
    
    [CMe1e1,CMe2e2,CMe3e3,CMe4e4] = AutocovarianceMM(auxGAMMA0,auxGAMMA0);
    
    AGFe1e1 = CMe1e1;
    AGFe2e2 = CMe2e2;
    AGFe3e3 = CMe3e3;
    AGFe4e4 = CMe4e4;
    
    j = 0;
    error = 1;
    
    while ((j < maxiter) && (error >= maxerror))
        
        auxGAMMAj = AutocovarianceWK(PHI,THETA,eye(K),j+1);
        [CMe1e1,CMe2e2,CMe3e3,CMe4e4] = ...
            AutocovarianceMM(auxGAMMA0,auxGAMMAj);
        [CMe1e1m,CMe2e2m,CMe3e3m,CMe4e4m] = ...
            AutocovarianceMM(auxGAMMA0,auxGAMMAj');
        
        AGFe1e1 = AGFe1e1 + CMe1e1 + CMe1e1m;
        AGFe2e2 = AGFe2e2 + CMe2e2 + CMe2e2m;
        AGFe3e3 = AGFe3e3 + CMe3e3 + CMe3e3m;
        AGFe4e4 = AGFe4e4 + CMe4e4 + CMe4e4m;
        
        j = j+1;
        
        de1e1 = max(max(abs(CMe1e1 + CMe1e1m)));
        de2e2 = max(max(abs(CMe2e2 + CMe2e2m)));
        de3e3 = max(max(abs(CMe3e3 + CMe3e3m)));
        de4e4 = max(max(abs(CMe4e4 + CMe4e4m)));
        error = max([de1e1,de2e2,de3e3,de4e4]);
        
        display([j,error],'Iteration')
    end
    
    Var_k1_ML(:,s) = (B_e4_1')*AGFe4e4*(B_e4_1) - (B_e2_1')*AGFe2e2*(B_e2_1);
    Var_k2_ML(:,s) = (B_e4_2')*AGFe4e4*(B_e4_2) - (B_e2_2')*AGFe2e2*(B_e2_2);
    Var_k3_ML(:,s) = (B_e4_3')*AGFe4e4*(B_e4_3) - (B_e2_3')*AGFe2e2*(B_e2_3);
    Var_k4_ML(:,:,s) = (B_e4_4')*AGFe4e4*(B_e4_4) - (B_e2_4')*AGFe2e2*(B_e2_4);
    Var_s1_ML(:,:,s) = (B_e3_1')*AGFe3e3*(B_e3_1) - (B_e1_1')*AGFe1e1*(B_e1_1);
    Var_s2_ML(:,:,s) = (B_e3_2')*AGFe3e3*(B_e3_2) - (B_e1_2')*AGFe1e1*(B_e1_2);
    Var_s3_ML(:,:,s) = (B_e3_3')*AGFe3e3*(B_e3_3) - (B_e1_3')*AGFe1e1*(B_e1_3);
    Var_s4_ML(:,:,s) = (B_e3_4')*AGFe3e3*(B_e3_4) - (B_e1_4')*AGFe1e1*(B_e1_4);
    
    kappa_k2 = auxGAMMA0(2,2)^4;
    kappa_k3 = auxGAMMA0(1,1)^4;
    kappa_s2 = auxGAMMA0(2,2)^3;
    kappa_s3 = auxGAMMA0(1,1)^3;
    
    j = 0;
    error = 1;
    
    while ((j < maxiter) && (error >= maxerror))
        
        auxGAMMAj = AutocovarianceWK(PHI,THETA,eye(K),j+1);
        kappa_k2 = kappa_k2 + 2*auxGAMMAj(2,2)^4;
        kappa_k3 = kappa_k3 + 2*auxGAMMAj(1,1)^4;
        kappa_s2 = kappa_s2 + 2*auxGAMMAj(2,2)^3;
        kappa_s3 = kappa_s3 + 2*auxGAMMAj(1,1)^3;
        
        j = j+1;
        error = max(trace(auxGAMMAj.^4),trace(auxGAMMAj.^3));
        display([j,error],'Iteration')
    end
    
    Var_HKk2_ML(:,s) = kappa_k2/auxGAMMA0(2,2)^4;
    Var_HKk3_ML(:,s) = kappa_k3/auxGAMMA0(1,1)^4;
    Var_HKs2_ML(:,s) = kappa_s2/auxGAMMA0(2,2)^3;
    Var_HKs3_ML(:,s) = kappa_s3/auxGAMMA0(1,1)^3;
    
    waitbar(s/S,h)
    
    % Saving results
    if (rem(s, safe) == 0 || s == S)
        save stderr.mat s ...
            Var_k1 Var_k2 Var_k3 Var_k4 ...
            Var_s1 Var_s2 Var_s3 Var_s4 ...
            Var_HKk2 Var_HKk3 ...
            Var_HKs2 Var_HKs3 ...
            Var_k1_ML Var_k2_ML Var_k3_ML Var_k4_ML ...
            Var_s1_ML Var_s2_ML Var_s3_ML Var_s4_ML ...
            Var_HKk2_ML Var_HKk3_ML ...
            Var_HKs2_ML Var_HKs3_ML
    end
    
end
close(h)

movefile('stderr.mat',['IO/stderr/', title,'_stderr_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat']);


%% COMPUTATION OF TEST STATISTICS

h = waitbar(0,'Computation of test statistic');
load(['IO/score/', title,'_score_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat'])
load(['IO/stderr/', title,'_stderr_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat'])

for s = 1:S
    
    display(s,'Replica')
    
    Stat_k0(:,s) = T*(Scr_k0(:,s).^2)/(N*(N+2)/2);
    Stat_k1(:,s) = T*(Scr_k1(:,s).^2)/(Var_k1);
    Stat_k2(:,s) = T*(Scr_k2(:,s).^2)/(Var_k2);
    Stat_k3(:,s) = T*(Scr_k3(:,s).^2)/(Var_k3);
    Scr_k4 = [Scr_k2(:,s);Scr_k3(:,s)];
    Stat_k4(:,s) = T*Scr_k4'*((Var_k4)\Scr_k4);
    
    Stat_s0(:,s) = T*Scr_s0(:,s)'*pinv(MSE(:,:,s))*Scr_s0(:,s)/(2*(N+2));
    Stat_s1(:,s) = T*Scr_s1(:,s)'*((Var_s1)\Scr_s1(:,s));
    Stat_s2(:,s) = T*Scr_s2(:,s)'*((Var_s2)\Scr_s2(:,s));
    Stat_s3(:,s) = T*Scr_s3(:,s)'*((Var_s3)\Scr_s3(:,s));
    Scr_s4 = [Scr_s2(:,s);Scr_s3(:,s)];
    Stat_s4(:,s) = T*Scr_s4'*((Var_s4)\Scr_s4);
    
    Stat_GH0(:,s) = Stat_k0(:,s) + Stat_s0(:,s);
    Stat_GH1(:,s) = Stat_k1(:,s) + Stat_s1(:,s);
    Stat_GH2(:,s) = Stat_k2(:,s) + Stat_s2(:,s);
    Stat_GH3(:,s) = Stat_k3(:,s) + Stat_s3(:,s);
    Stat_GH4(:,s) = Stat_k4(:,s) + Stat_s4(:,s);
    
    StatKT_k0(:,s) = Stat_k0(:,s) * (Scr_k0(:,s) > 0);
    StatKT_k1(:,s) = Stat_k1(:,s) * (Scr_k1(:,s) > 0);
    StatKT_k2(:,s) = Stat_k2(:,s) * (Scr_k2(:,s) > 0);
    StatKT_k3(:,s) = Stat_k3(:,s) * (Scr_k3(:,s) > 0);
    Scr_k4 = Scr_k4 .* (Scr_k4 > 0);
    StatKT_k4(:,s) = T*Scr_k4'*((Var_k4)\Scr_k4);
    
    StatKT_GH0(:,s) = StatKT_k0(:,s) + Stat_s0(:,s);
    StatKT_GH1(:,s) = StatKT_k1(:,s) + Stat_s1(:,s);
    StatKT_GH2(:,s) = StatKT_k2(:,s) + Stat_s2(:,s);
    StatKT_GH3(:,s) = StatKT_k3(:,s) + Stat_s3(:,s);
    StatKT_GH4(:,s) = StatKT_k4(:,s) + Stat_s4(:,s);
    
    Stat_k0_ML(:,s) = T*(Scr_k0_ML(:,s).^2)/(N*(N+2)/2);
    Stat_k1_ML(:,s) = T*(Scr_k1_ML(:,s).^2)/(Var_k1_ML(:,s));
    Stat_k2_ML(:,s) = T*(Scr_k2_ML(:,s).^2)/(Var_k2_ML(:,s));
    Stat_k3_ML(:,s) = T*(Scr_k3_ML(:,s).^2)/(Var_k3_ML(:,s));
    Scr_k4_ML = [Scr_k2_ML(:,s);Scr_k3(:,s)];
    Stat_k4_ML(:,s) = T*Scr_k4_ML'*((Var_k4_ML(:,:,s))\Scr_k4_ML);
    
    Stat_s0_ML(:,s) = T*Scr_s0_ML(:,s)'*pinv(MSE_ML(:,:,s))...
        *Scr_s0_ML(:,s)/(2*(N+2));
    Stat_s1_ML(:,s) = T*Scr_s1_ML(:,s)'*((Var_s1_ML(:,:,s))\Scr_s1_ML(:,s));
    Stat_s2_ML(:,s) = T*Scr_s2_ML(:,s)'*((Var_s2_ML(:,:,s))\Scr_s2_ML(:,s));
    Stat_s3_ML(:,s) = T*Scr_s3_ML(:,s)'*((Var_s3_ML(:,:,s))\Scr_s3_ML(:,s));
    Scr_s4_ML = [Scr_s2_ML(:,s);Scr_s3_ML(:,s)];
    Stat_s4_ML(:,s) = T*Scr_s4_ML'*((Var_s4_ML(:,:,s))\Scr_s4_ML);
    
    Stat_GH0_ML(:,s) = Stat_k0_ML(:,s) + Stat_s0_ML(:,s);
    Stat_GH1_ML(:,s) = Stat_k1_ML(:,s) + Stat_s1_ML(:,s);
    Stat_GH2_ML(:,s) = Stat_k2_ML(:,s) + Stat_s2_ML(:,s);
    Stat_GH3_ML(:,s) = Stat_k3_ML(:,s) + Stat_s3_ML(:,s);
    Stat_GH4_ML(:,s) = Stat_k4_ML(:,s) + Stat_s4_ML(:,s);
    
    StatKT_k0_ML(:,s) = Stat_k0_ML(:,s) * (Scr_k0_ML(:,s) > 0);
    StatKT_k1_ML(:,s) = Stat_k1_ML(:,s) * (Scr_k1_ML(:,s) > 0);
    StatKT_k2_ML(:,s) = Stat_k2_ML(:,s) * (Scr_k2_ML(:,s) > 0);
    StatKT_k3_ML(:,s) = Stat_k3_ML(:,s) * (Scr_k3_ML(:,s) > 0);
    Scr_k4_ML = Scr_k4_ML .* (Scr_k4_ML > 0);
    StatKT_k4_ML(:,s) = T*Scr_k4_ML'*((Var_k4_ML(:,:,s))\Scr_k4_ML);
    
    StatKT_GH0_ML(:,s) = StatKT_k0_ML(:,s) + Stat_s0_ML(:,s);
    StatKT_GH1_ML(:,s) = StatKT_k1_ML(:,s) + Stat_s1_ML(:,s);
    StatKT_GH2_ML(:,s) = StatKT_k2_ML(:,s) + Stat_s2_ML(:,s);
    StatKT_GH3_ML(:,s) = StatKT_k3_ML(:,s) + Stat_s3_ML(:,s);
    StatKT_GH4_ML(:,s) = StatKT_k4_ML(:,s) + Stat_s4_ML(:,s);
    
    Stat_HKk2(:,s) = HK_k2(:,s)/(Var_HKk2);
    Stat_HKk3(:,s) = HK_k3(:,s)/(Var_HKk3);
    
    Stat_HKs2(:,s) = HK_s2(:,s)/(Var_HKs2);
    Stat_HKs3(:,s) = HK_s3(:,s)/(Var_HKs3);
    
    Stat_HKGH2(:,s) = Stat_HKk2(:,s) + Stat_HKs2(:,s);
    Stat_HKGH3(:,s) = Stat_HKk3(:,s) + Stat_HKs3(:,s);
    
    Stat_HKk2_ML(:,s) = HK_k2_ML(:,s)/(Var_HKk2_ML(:,s));
    Stat_HKk3_ML(:,s) = HK_k3_ML(:,s)/(Var_HKk3_ML(:,s));
    
    Stat_HKs2_ML(:,s) = HK_s2_ML(:,s)/(Var_HKs2_ML(:,s));
    Stat_HKs3_ML(:,s) = HK_s3_ML(:,s)/(Var_HKs3_ML(:,s));
    
    Stat_HKGH2_ML(:,s) = Stat_HKk2_ML(:,s) + Stat_HKs2_ML(:,s);
    Stat_HKGH3_ML(:,s) = Stat_HKk3_ML(:,s) + Stat_HKs3_ML(:,s);
    
    waitbar(s/S,h)
    
    % Saving results
    if (rem(s, safe) == 0 || s == S)
        
        for level = [1,5,10]
            if (level == 1)
                index = 1;
            elseif (level == 5)
                index = 2;
            elseif (level == 10)
                index = 3;
            end
            
            c1 = chi2inv(1-level/100,1);
            c2 = chi2inv(1-level/100,2);
            c3 = chi2inv(1-level/100,3);
            c4 = chi2inv(1-level/100,4);
            
            cKT1 = (level == 1)*5.41189 + (level == 5)*2.70554 ...
                + (level == 10)*1.64237;
            cKT2 = (level == 1)*8.27325 + (level == 5)*5.13838 ...
                + (level == 10)*3.80781;
            cKT3 = (level == 1)*10.5009 + (level == 5)*7.04506 ...
                + (level == 10)*5.52814;
            
            if (q < 1.5) % q = 0.5
                cKTcase4t = (level == 1)*7.33623 + (level == 5)*4.26121 ...
                    + (level == 10)*2.97309;
                cKTcase4GH = (level == 1)*11.7016 + (level == 5)*8.03853 ...
                    + (level == 10)*6.40778;
            else % q = 2.0
                cKTcase4t = (level == 1)*7.38755 + (level == 5)*4.29556 ...
                    + (level == 10)*2.99655;
                cKTcase4GH = (level == 1)*11.7294 + (level == 5)*8.02211 ...
                    + (level == 10)*6.42092;
            end
            
            
            rejection(1,index,1) = sum(Stat_k0 > c1)/S;
            rejection(2,index,1) = sum(Stat_k1 > c1)/S;
            rejection(3,index,1) = sum(Stat_k2 > c1)/S;
            rejection(4,index,1) = sum(Stat_k3 > c1)/S;
            rejection(5,index,1) = sum(Stat_k4 > c2)/S;
            rejection(6,index,1) = sum(Stat_HKk2 > c1)/S;
            rejection(7,index,1) = sum(Stat_HKk3 > c1)/S;
            
            rejection(1,index,2) = sum(Stat_s0 > c1)/S;
            rejection(2,index,2) = sum(Stat_s1 > c2)/S;
            rejection(3,index,2) = sum(Stat_s2 > c1)/S;
            rejection(4,index,2) = sum(Stat_s3 > c1)/S;
            rejection(5,index,2) = sum(Stat_s4 > c2)/S;
            rejection(6,index,2) = sum(Stat_HKs2 > c1)/S;
            rejection(7,index,2) = sum(Stat_HKs3 > c1)/S;
            
            rejection(1,index,3) = sum(Stat_GH0 > c2)/S;
            rejection(2,index,3) = sum(Stat_GH1 > c3)/S;
            rejection(3,index,3) = sum(Stat_GH2 > c2)/S;
            rejection(4,index,3) = sum(Stat_GH3 > c2)/S;
            rejection(5,index,3) = sum(Stat_GH4 > c4)/S;
            rejection(6,index,3) = sum(Stat_HKGH2 > c2)/S;
            rejection(7,index,3) = sum(Stat_HKGH3 > c2)/S;
            
            rejection_ML(1,index,1) = sum(Stat_k0_ML > c1)/S;
            rejection_ML(2,index,1) = sum(Stat_k1_ML > c1)/S;
            rejection_ML(3,index,1) = sum(Stat_k2_ML > c1)/S;
            rejection_ML(4,index,1) = sum(Stat_k3_ML > c1)/S;
            rejection_ML(5,index,1) = sum(Stat_k4_ML > c2)/S;
            rejection_ML(6,index,1) = sum(Stat_HKk2_ML > c1)/S;
            rejection_ML(7,index,1) = sum(Stat_HKk3_ML > c1)/S;
            
            rejection_ML(1,index,2) = sum(Stat_s0_ML > c1)/S;
            rejection_ML(2,index,2) = sum(Stat_s1_ML > c2)/S;
            rejection_ML(3,index,2) = sum(Stat_s2_ML > c1)/S;
            rejection_ML(4,index,2) = sum(Stat_s3_ML > c1)/S;
            rejection_ML(5,index,2) = sum(Stat_s4_ML > c2)/S;
            rejection_ML(6,index,2) = sum(Stat_HKs2_ML > c1)/S;
            rejection_ML(7,index,2) = sum(Stat_HKs3_ML > c1)/S;
            
            rejection_ML(1,index,3) = sum(Stat_GH0_ML > c2)/S;
            rejection_ML(2,index,3) = sum(Stat_GH1_ML > c3)/S;
            rejection_ML(3,index,3) = sum(Stat_GH2_ML > c2)/S;
            rejection_ML(4,index,3) = sum(Stat_GH3_ML > c2)/S;
            rejection_ML(5,index,3) = sum(Stat_GH4_ML > c4)/S;
            rejection_ML(6,index,3) = sum(Stat_HKGH2_ML > c2)/S;
            rejection_ML(7,index,3) = sum(Stat_HKGH3_ML > c2)/S;
            
            rejectionKT(1,index,1) = sum(StatKT_k0 > cKT1)/S;
            rejectionKT(2,index,1) = sum(StatKT_k1 > cKT1)/S;
            rejectionKT(3,index,1) = sum(StatKT_k2 > cKT1)/S;
            rejectionKT(4,index,1) = sum(StatKT_k3 > cKT1)/S;
            rejectionKT(5,index,1) = sum(StatKT_k4 > cKTcase4t)/S;
            
            rejectionKT(1,index,2) = sum(StatKT_GH0 > cKT2)/S;
            rejectionKT(2,index,2) = sum(StatKT_GH1 > cKT3)/S;
            rejectionKT(3,index,2) = sum(StatKT_GH2 > cKT2)/S;
            rejectionKT(4,index,2) = sum(StatKT_GH3 > cKT2)/S;
            rejectionKT(5,index,2) = sum(StatKT_GH4 > cKTcase4GH)/S;
            
            rejectionKT_ML(1,index,1) = sum(StatKT_k0_ML > cKT1)/S;
            rejectionKT_ML(2,index,1) = sum(StatKT_k1_ML > cKT1)/S;
            rejectionKT_ML(3,index,1) = sum(StatKT_k2_ML > cKT1)/S;
            rejectionKT_ML(4,index,1) = sum(StatKT_k3_ML > cKT1)/S;
            rejectionKT_ML(5,index,1) = sum(StatKT_k4_ML > cKTcase4t)/S;
            
            rejectionKT_ML(1,index,2) = sum(StatKT_GH0_ML > cKT2)/S;
            rejectionKT_ML(2,index,2) = sum(StatKT_GH1_ML > cKT3)/S;
            rejectionKT_ML(3,index,2) = sum(StatKT_GH2_ML > cKT2)/S;
            rejectionKT_ML(4,index,2) = sum(StatKT_GH3_ML > cKT2)/S;
            rejectionKT_ML(5,index,2) = sum(StatKT_GH4_ML > cKTcase4GH)/S;
        end
        
        save test.mat s rejection rejection_ML rejectionKT rejectionKT_ML ...
            Stat_k0 Stat_k1 Stat_k2 Stat_k3 Stat_k4 ...
            Stat_s0 Stat_s1 Stat_s2 Stat_s3 Stat_s4 ...
            Stat_GH0 Stat_GH1 Stat_GH2 Stat_GH3 Stat_GH4 ...
            StatKT_k0 StatKT_k1 StatKT_k2 StatKT_k3 StatKT_k4 ...
            StatKT_GH0 StatKT_GH1 StatKT_GH2 StatKT_GH3 StatKT_GH4 ...
            Stat_HKk2 Stat_HKk3 ...
            Stat_HKs2 Stat_HKs3 ...
            Stat_HKGH2 Stat_HKGH3 ...
            Stat_k0_ML Stat_k1_ML Stat_k2_ML Stat_k3_ML Stat_k4_ML ...
            Stat_s0_ML Stat_s1_ML Stat_s2_ML Stat_s3_ML Stat_s4_ML...
            Stat_GH0_ML Stat_GH1_ML Stat_GH2_ML Stat_GH3_ML Stat_GH4_ML ...
            StatKT_k0_ML StatKT_k1_ML StatKT_k2_ML StatKT_k3_ML StatKT_k4_ML ...
            StatKT_GH0_ML StatKT_GH1_ML StatKT_GH2_ML StatKT_GH3_ML StatKT_GH4_ML ...
            Stat_HKk2_ML Stat_HKk3_ML ...
            Stat_HKs2_ML Stat_HKs3_ML ...
            Stat_HKGH2_ML Stat_HKGH3_ML
    end
    
end
close(h)

movefile('test.mat',['IO/test/', title,'_test_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat']);