clear
clc
close all
path([pwd, '/subprograms/'], path)


%% SETTINGS AND PREALLOCATION

% Simulation settings
S = 10000; % Number of Monte Carlo replicas
safe = ceil(0.05*S); % Number of replicas per saving at each stage
title = 'S622';

S_bootstrap = 10000; % Number of bootstrap samples in each iteration
S_grid = 50; % Number of grid points
R2_grid = linspace(0.05,0.95,S_grid);

options = optimoptions('fmincon','Algorithm','interior-point',...
    'GradObj','off','Hessian','off','Display','off',...
    'MaxFunEvals',1000,'MaxIter',1000,'TolCon',0.0,'TolFun',10e-9);
maxerror = 1e-4; % Tolerance criterion for numerical convergence
maxiter = 1e2; % Maximum number of iterations for numerical convergence

% Model settings
T = 100; % Sample size
hypothesis = 0; % Distribution of innovations

N = 1;
M = 2;
K = 2;

% Preallocation
Stat_k0_ML = zeros(S_bootstrap,S_grid);                                    % Case-0 statistic of kurtosis component at ML estimate
Stat_k1_ML = zeros(S_bootstrap,S_grid);                                    % Case-1 statistic of kurtosis component at ML estimate
Stat_k2_ML = zeros(S_bootstrap,S_grid);                                    % Case-2 statistic of kurtosis component at ML estimate
Stat_k3_ML = zeros(S_bootstrap,S_grid);                                    % Case-3 statistic of kurtosis component at ML estimate
Stat_k4_ML = zeros(S_bootstrap,S_grid);                                    % Case-4 statistic of kurtosis component at ML estimate

Stat_s0_ML = zeros(S_bootstrap,S_grid);                                    % Case-0 statistic of skewness component at ML estimate
Stat_s1_ML = zeros(S_bootstrap,S_grid);                                    % Case-1 statistic of skewness component at ML estimate
Stat_s2_ML = zeros(S_bootstrap,S_grid);                                    % Case-2 statistic of skewness component at ML estimate
Stat_s3_ML = zeros(S_bootstrap,S_grid);                                    % Case-3 statistic of skewness component at ML estimate
Stat_s4_ML = zeros(S_bootstrap,S_grid);                                    % Case-4 statistic of skewness component at ML estimate

Stat_GH0_ML = zeros(S_bootstrap,S_grid);                                   % Case-0 full statistic at ML estimate
Stat_GH1_ML = zeros(S_bootstrap,S_grid);                                   % Case-1 full statistic at ML estimate
Stat_GH2_ML = zeros(S_bootstrap,S_grid);                                   % Case-2 full statistic at ML estimate
Stat_GH3_ML = zeros(S_bootstrap,S_grid);                                   % Case-3 full statistic at ML estimate
Stat_GH4_ML = zeros(S_bootstrap,S_grid);                                   % Case-4 full statistic at ML estimate

StatKT_k0_ML = zeros(S_bootstrap,S_grid);                                  % Case-0 KT statistic of kurtosis component at ML estimate
StatKT_k1_ML = zeros(S_bootstrap,S_grid);                                  % Case-1 KT statistic of kurtosis component at ML estimate
StatKT_k2_ML = zeros(S_bootstrap,S_grid);                                  % Case-2 KT statistic of kurtosis component at ML estimate
StatKT_k3_ML = zeros(S_bootstrap,S_grid);                                  % Case-3 KT statistic of kurtosis component at ML estimate
StatKT_k4_ML = zeros(S_bootstrap,S_grid);                                  % Case-4 KT statistic of kurtosis component at ML estimate

StatKT_GH0_ML = zeros(S_bootstrap,S_grid);                                 % Case-0 full KT statistic at ML estimate
StatKT_GH1_ML = zeros(S_bootstrap,S_grid);                                 % Case-1 full KT statistic at ML estimate
StatKT_GH2_ML = zeros(S_bootstrap,S_grid);                                 % Case-2 full KT statistic at ML estimate
StatKT_GH3_ML = zeros(S_bootstrap,S_grid);                                 % Case-3 full KT statistic at ML estimate
StatKT_GH4_ML = zeros(S_bootstrap,S_grid);                                 % Case-4 full KT statistic at ML estimate

Stat_HKk2_ML = zeros(S_bootstrap,S_grid);                                  % Harvey-Koopmans case-2 kurtosis statistic at ML estimate
Stat_HKk3_ML = zeros(S_bootstrap,S_grid);                                  % Harvey-Koopmans case-3 kurtosis statistic at ML estimate
Stat_HKs2_ML = zeros(S_bootstrap,S_grid);                                  % Harvey-Koopmans case-2 skewness statistic at ML estimate
Stat_HKs3_ML = zeros(S_bootstrap,S_grid);                                  % Harvey-Koopmans case-3 skewness statistic at ML estimate
Stat_HKGH2_ML = zeros(S_bootstrap,S_grid);                                 % Harvey-Koopmans case-2 full statistic at ML estimate
Stat_HKGH3_ML = zeros(S_bootstrap,S_grid);                                 % Harvey-Koopmans case-3 full statistic at ML estimate

rej_k = zeros(5,3,S);
rej_s = zeros(5,3,S);
rej_GH = zeros(5,3,S);
rej_KTk = zeros(5,3,S);
rej_KTGH = zeros(5,3,S);

rej_k_ML = zeros(5,3,S);
rej_s_ML = zeros(5,3,S);
rej_GH_ML = zeros(5,3,S);
rej_KTk_ML = zeros(5,3,S);
rej_KTGH_ML = zeros(5,3,S);

rej_HK = zeros(6,3,S);
rej_HK_ML = zeros(6,3,S);

reject = zeros(5,3,3);
reject_ML = zeros(5,3,3);
rejectKT = zeros(5,3,2);
rejectKT_ML = zeros(5,3,2);
rejectHK = zeros(2,3,3);
rejectHK_ML = zeros(2,3,3);


%% BOOTSTRAP CALCULATIONS

for s_grid = 1:S_grid
    
    % Simulation settings
    rng('default')
    rng(280990)
    
    % Model settings
    c = 0;
    q = (4*R2_grid(s_grid)^2)/(1-R2_grid(s_grid)^2);
    parameters = [c;1;1/q];
    
    C = [parameters(1);0];
    F = [1,0;0,0];
    G = diag([sqrt(parameters(2)),sqrt(parameters(3))]);
    H = [1,1];
    kappa = 1e5;
    
    % Bootstrap calculations
    h = waitbar(0,['Bootstrap: ',num2str(s_grid),' out of ',num2str(S_grid)]);
    
    for s_bootstrap = 1:S_bootstrap
        
        display(s_bootstrap,'Replica')
        
        X = zeros(M,T);
        Y = zeros(N,T);
        
        X(:,1) = zeros(2,1);
        Y(:,1) = H*X(:,1);
        
        for t = 2:T
            X(:,t) = C + F*X(:,t-1) + G*normrnd(0,1,2,1);
            Y(:,t) = H*X(:,t);
        end
        
        [MLestimate,Var_MLestimate] = UN_Estimation(...
            parameters(1),parameters(2),parameters(3),Y,options);
        
        C_ML = [MLestimate(1);zeros(3,1)];
        F_ML = [1,zeros(1,3);zeros(3,4)];
        G_ML = [diag([sqrt(MLestimate(2)),sqrt(MLestimate(3))]);1,0;0,1];
        H_ML = [1,1,0,0];
        EX1 = zeros(4,1);
        VX1_ML = [diag([kappa*MLestimate(2),MLestimate(3)]),...
            zeros(2,2);zeros(2,2),eye(2)];
        
        [Xf_ML,Pf_ML,Xs_ML,Ps_ML] = KalmanFS(EX1,VX1_ML,F_ML,G_ML,H_ML,Y,C_ML);
        
        e0_ML = zeros(N,T-2);
        P0_ML = zeros(N,N,T-2);
        for t = 2:T-1
            e0_ML(:,t-1) = Y(:,t) - H_ML*Xf_ML(:,t);
            P0_ML(:,:,t-1) = H_ML*Pf_ML(:,:,t)*H_ML';
        end
        e1_ML = Xs_ML(M+1:M+K,2:T-1);
        P1_ML = Ps_ML(M+1:M+K,M+1:M+K,2:T-1);
        e2_ML = Xs_ML(M+2:M+K,2:T-1);
        P2_ML = Ps_ML(M+2:M+K,M+2:M+K,2:T-1);
        e3_ML = Xs_ML(M+1,2:T-1);
        P3_ML = Ps_ML(M+1,M+1,2:T-1);
        
        sk0_ML = ScoreFK(e0_ML,P0_ML);
        sk1_ML = ScoreSK(e1_ML,P1_ML);
        sk2_ML = ScoreSK(e2_ML,P2_ML);
        sk3_ML = ScoreSK(e3_ML,P3_ML);
        
        ss0_ML = ScoreFS(e0_ML,P0_ML);
        ss1_ML = ScoreSS(e1_ML,P1_ML);
        ss2_ML = ScoreSS(e2_ML,P2_ML);
        ss3_ML = ScoreSS(e3_ML,P3_ML);
        
        Scr_k0_ML = mean(sk0_ML,2);
        Scr_k1_ML = mean(sk1_ML,2);
        Scr_k2_ML = mean(sk2_ML,2);
        Scr_k3_ML = mean(sk3_ML,2);
        
        MSE_ML = mean(P0_ML,3);
        
        Scr_s0_ML = mean(ss0_ML,2);
        Scr_s1_ML = mean(ss1_ML,2);
        Scr_s2_ML = mean(ss2_ML,2);
        Scr_s3_ML = mean(ss3_ML,2);
        
        HK_k2_ML = StatHKK(e2_ML);
        HK_k3_ML = StatHKK(e3_ML);
        
        HK_s2_ML = StatHKS(e2_ML);
        HK_s3_ML = StatHKS(e3_ML);
        
        gx = MLestimate(2);
        gv = MLestimate(3);
        
        [PHI,THETA] = UN_WKfilter(gx,gv);
        auxGAMMA0 = AutocovarianceWK(PHI,THETA,eye(K),0);
        
        r = K;
        [~,aux_bs,~] = ScoreSS(zeros(K,1),eye(K) - auxGAMMA0);
        [~,aux_bk,~] = ScoreSK(zeros(K,1),eye(K) - auxGAMMA0);
        
        B_e1_1 = aux_bs(1:r,:);
        B_e2_1 = aux_bk(2:1+r^2);
        B_e3_1 = aux_bs(r+1:end,:);
        B_e4_1 = aux_bk(2+r^2:end);
        
        r = K-1;
        E2 = [zeros(K-1,1),eye(K-1)];
        [~,aux_bs,~] = ScoreSS(zeros(K-1,1),eye(K-1) - auxGAMMA0(2:K,2:K));
        [~,aux_bk,~] = ScoreSK(zeros(K-1,1),eye(K-1) - auxGAMMA0(2:K,2:K));
        
        B_e1_2 = E2'*aux_bs(1:r,:);
        B_e2_2 = kron(E2',E2')*aux_bk(2:1+r^2);
        B_e3_2 = kron(E2',E2')*aux_bs(r+1:end,:);
        B_e4_2 = kron(E2',E2')*aux_bk(2+r^2:end);
        
        E3 = [1,zeros(1,K-1)];
        [~,aux_bs,~] = ScoreSS(0,1 - auxGAMMA0(1,1));
        [~,aux_bk,~] = ScoreSK(0,1 - auxGAMMA0(1,1));
        
        B_e1_3 = E3'*aux_bs(1:r,:);
        B_e2_3 = kron(E3',E3')*aux_bk(2:1+r^2);
        B_e3_3 = kron(E3',E3')*aux_bs(r+1:end,:);
        B_e4_3 = kron(E3',E3')*aux_bk(2+r^2:end);
        
        B_e1_4 = [B_e1_2,B_e1_3];
        B_e2_4 = [B_e2_2,B_e2_3];
        B_e3_4 = [B_e3_2,B_e3_3];
        B_e4_4 = [B_e4_2,B_e4_3];
        
        [CMe1e1,CMe2e2,CMe3e3,CMe4e4] = AutocovarianceMM(auxGAMMA0,auxGAMMA0);
        
        AGFe1e1 = CMe1e1;
        AGFe2e2 = CMe2e2;
        AGFe3e3 = CMe3e3;
        AGFe4e4 = CMe4e4;
        
        j = 0;
        error = 1;
        
        while ((j < maxiter) && (error >= maxerror))
            
            auxGAMMAj = AutocovarianceWK(PHI,THETA,eye(K),j+1);
            [CMe1e1,CMe2e2,CMe3e3,CMe4e4] = ...
                AutocovarianceMM(auxGAMMA0,auxGAMMAj);
            [CMe1e1m,CMe2e2m,CMe3e3m,CMe4e4m] = ...
                AutocovarianceMM(auxGAMMA0,auxGAMMAj');
            
            AGFe1e1 = AGFe1e1 + CMe1e1 + CMe1e1m;
            AGFe2e2 = AGFe2e2 + CMe2e2 + CMe2e2m;
            AGFe3e3 = AGFe3e3 + CMe3e3 + CMe3e3m;
            AGFe4e4 = AGFe4e4 + CMe4e4 + CMe4e4m;
            
            j = j+1;
            
            de1e1 = max(max(abs(CMe1e1 + CMe1e1m)));
            de2e2 = max(max(abs(CMe2e2 + CMe2e2m)));
            de3e3 = max(max(abs(CMe3e3 + CMe3e3m)));
            de4e4 = max(max(abs(CMe4e4 + CMe4e4m)));
            error = max([de1e1,de2e2,de3e3,de4e4]);
            
        end
        
        Var_k1_ML = (B_e4_1')*AGFe4e4*(B_e4_1) - (B_e2_1')*AGFe2e2*(B_e2_1);
        Var_k2_ML = (B_e4_2')*AGFe4e4*(B_e4_2) - (B_e2_2')*AGFe2e2*(B_e2_2);
        Var_k3_ML = (B_e4_3')*AGFe4e4*(B_e4_3) - (B_e2_3')*AGFe2e2*(B_e2_3);
        Var_k4_ML = (B_e4_4')*AGFe4e4*(B_e4_4) - (B_e2_4')*AGFe2e2*(B_e2_4);
        Var_s1_ML = (B_e3_1')*AGFe3e3*(B_e3_1) - (B_e1_1')*AGFe1e1*(B_e1_1);
        Var_s2_ML = (B_e3_2')*AGFe3e3*(B_e3_2) - (B_e1_2')*AGFe1e1*(B_e1_2);
        Var_s3_ML = (B_e3_3')*AGFe3e3*(B_e3_3) - (B_e1_3')*AGFe1e1*(B_e1_3);
        Var_s4_ML = (B_e3_4')*AGFe3e3*(B_e3_4) - (B_e1_4')*AGFe1e1*(B_e1_4);
        
        kappa_k2 = auxGAMMA0(2,2)^4;
        kappa_k3 = auxGAMMA0(1,1)^4;
        kappa_s2 = auxGAMMA0(2,2)^3;
        kappa_s3 = auxGAMMA0(1,1)^3;
        
        j = 0;
        error = 1;
        
        while ((j < maxiter) && (error >= maxerror))
            
            auxGAMMAj = AutocovarianceWK(PHI,THETA,eye(K),j+1);
            kappa_k2 = kappa_k2 + 2*auxGAMMAj(2,2)^4;
            kappa_k3 = kappa_k3 + 2*auxGAMMAj(1,1)^4;
            kappa_s2 = kappa_s2 + 2*auxGAMMAj(2,2)^3;
            kappa_s3 = kappa_s3 + 2*auxGAMMAj(1,1)^3;
            
            j = j+1;
            error = max(trace(auxGAMMAj.^4),trace(auxGAMMAj.^3));
            
        end
        
        Var_HKk2_ML = kappa_k2/auxGAMMA0(2,2)^4;
        Var_HKk3_ML = kappa_k3/auxGAMMA0(1,1)^4;
        Var_HKs2_ML = kappa_s2/auxGAMMA0(2,2)^3;
        Var_HKs3_ML = kappa_s3/auxGAMMA0(1,1)^3;
        
        Stat_k0_ML(s_bootstrap,s_grid) = T*(Scr_k0_ML.^2)/(N*(N+2)/2);
        Stat_k1_ML(s_bootstrap,s_grid) = T*(Scr_k1_ML.^2)/(Var_k1_ML);
        Stat_k2_ML(s_bootstrap,s_grid) = T*(Scr_k2_ML.^2)/(Var_k2_ML);
        Stat_k3_ML(s_bootstrap,s_grid) = T*(Scr_k3_ML.^2)/(Var_k3_ML);
        Scr_k4_ML = [Scr_k2_ML;Scr_k3_ML];
        Stat_k4_ML(s_bootstrap,s_grid) = T*Scr_k4_ML'*((Var_k4_ML)\Scr_k4_ML);
        
        Stat_s0_ML(s_bootstrap,s_grid) = T*Scr_s0_ML'*pinv(MSE_ML)...
            *Scr_s0_ML/(2*(N+2));
        Stat_s1_ML(s_bootstrap,s_grid) = T*Scr_s1_ML'*((Var_s1_ML)\Scr_s1_ML);
        Stat_s2_ML(s_bootstrap,s_grid) = T*Scr_s2_ML'*((Var_s2_ML)\Scr_s2_ML);
        Stat_s3_ML(s_bootstrap,s_grid) = T*Scr_s3_ML'*((Var_s3_ML)\Scr_s3_ML);
        Scr_s4_ML = [Scr_s2_ML;Scr_s3_ML];
        Stat_s4_ML(s_bootstrap,s_grid) = T*Scr_s4_ML'*((Var_s4_ML)\Scr_s4_ML);
        
        Stat_GH0_ML(s_bootstrap,s_grid) = Stat_k0_ML(s_bootstrap,s_grid) + Stat_s0_ML(s_bootstrap,s_grid);
        Stat_GH1_ML(s_bootstrap,s_grid) = Stat_k1_ML(s_bootstrap,s_grid) + Stat_s1_ML(s_bootstrap,s_grid);
        Stat_GH2_ML(s_bootstrap,s_grid) = Stat_k2_ML(s_bootstrap,s_grid) + Stat_s2_ML(s_bootstrap,s_grid);
        Stat_GH3_ML(s_bootstrap,s_grid) = Stat_k3_ML(s_bootstrap,s_grid) + Stat_s3_ML(s_bootstrap,s_grid);
        Stat_GH4_ML(s_bootstrap,s_grid) = Stat_k4_ML(s_bootstrap,s_grid) + Stat_s4_ML(s_bootstrap,s_grid);
        
        StatKT_k0_ML(s_bootstrap,s_grid) = Stat_k0_ML(s_bootstrap,s_grid) * (Scr_k0_ML > 0);
        StatKT_k1_ML(s_bootstrap,s_grid) = Stat_k1_ML(s_bootstrap,s_grid) * (Scr_k1_ML > 0);
        StatKT_k2_ML(s_bootstrap,s_grid) = Stat_k2_ML(s_bootstrap,s_grid) * (Scr_k2_ML > 0);
        StatKT_k3_ML(s_bootstrap,s_grid) = Stat_k3_ML(s_bootstrap,s_grid) * (Scr_k3_ML > 0);
        Scr_k4_ML = Scr_k4_ML .* (Scr_k4_ML > 0);
        StatKT_k4_ML(s_bootstrap,s_grid) = T*Scr_k4_ML'*((Var_k4_ML)\Scr_k4_ML);
        
        StatKT_GH0_ML(s_bootstrap,s_grid) = StatKT_k0_ML(s_bootstrap,s_grid) + Stat_s0_ML(s_bootstrap,s_grid);
        StatKT_GH1_ML(s_bootstrap,s_grid) = StatKT_k1_ML(s_bootstrap,s_grid) + Stat_s1_ML(s_bootstrap,s_grid);
        StatKT_GH2_ML(s_bootstrap,s_grid) = StatKT_k2_ML(s_bootstrap,s_grid) + Stat_s2_ML(s_bootstrap,s_grid);
        StatKT_GH3_ML(s_bootstrap,s_grid) = StatKT_k3_ML(s_bootstrap,s_grid) + Stat_s3_ML(s_bootstrap,s_grid);
        StatKT_GH4_ML(s_bootstrap,s_grid) = StatKT_k4_ML(s_bootstrap,s_grid) + Stat_s4_ML(s_bootstrap,s_grid);
        
        Stat_HKk2_ML(s_bootstrap,s_grid) = HK_k2_ML/(Var_HKk2_ML);
        Stat_HKk3_ML(s_bootstrap,s_grid) = HK_k3_ML/(Var_HKk3_ML);
        
        Stat_HKs2_ML(s_bootstrap,s_grid) = HK_s2_ML/(Var_HKs2_ML);
        Stat_HKs3_ML(s_bootstrap,s_grid) = HK_s3_ML/(Var_HKs3_ML);
        
        Stat_HKGH2_ML(s_bootstrap,s_grid) = Stat_HKk2_ML(s_bootstrap,s_grid) + Stat_HKs2_ML(s_bootstrap,s_grid);
        Stat_HKGH3_ML(s_bootstrap,s_grid) = Stat_HKk3_ML(s_bootstrap,s_grid) + Stat_HKs3_ML(s_bootstrap,s_grid);
        
        waitbar(s_bootstrap/S_bootstrap,h)
        
    end
    close(h)
    
    save bootstrap.mat R2_grid Stat_k0_ML Stat_k1_ML Stat_k2_ML Stat_k3_ML Stat_k4_ML ...
        Stat_s0_ML Stat_s1_ML Stat_s2_ML Stat_s3_ML Stat_s4_ML...
        Stat_GH0_ML Stat_GH1_ML Stat_GH2_ML Stat_GH3_ML Stat_GH4_ML ...
        StatKT_k0_ML StatKT_k1_ML StatKT_k2_ML StatKT_k3_ML StatKT_k4_ML ...
        StatKT_GH0_ML StatKT_GH1_ML StatKT_GH2_ML StatKT_GH3_ML StatKT_GH4_ML ...
        Stat_HKk2_ML Stat_HKk3_ML ...
        Stat_HKs2_ML Stat_HKs3_ML ...
        Stat_HKGH2_ML Stat_HKGH3_ML
    
end

movefile('bootstrap.mat',['IO/bootstrap/', title,'_bootstrap_T',num2str(T),'.mat']);
    

%% BOOTSTRAP CRITICAL VALUES

load(['IO/bootstrap/', title,'_bootstrap_T',num2str(T),'.mat'])

cv_k0 = quantile(Stat_k0_ML, [.99; .95; .90])';
cv_k1 = quantile(Stat_k1_ML, [.99; .95; .90])';
cv_k2 = quantile(Stat_k2_ML, [.99; .95; .90])';
cv_k3 = quantile(Stat_k3_ML, [.99; .95; .90])';
cv_k4 = quantile(Stat_k4_ML, [.99; .95; .90])';

cv_s0 = quantile(Stat_s0_ML, [.99; .95; .90])';
cv_s1 = quantile(Stat_s1_ML, [.99; .95; .90])';
cv_s2 = quantile(Stat_s2_ML, [.99; .95; .90])';
cv_s3 = quantile(Stat_s3_ML, [.99; .95; .90])';
cv_s4 = quantile(Stat_s4_ML, [.99; .95; .90])';

cv_GH0 = quantile(Stat_GH0_ML, [.99; .95; .90])';
cv_GH1 = quantile(Stat_GH1_ML, [.99; .95; .90])';
cv_GH2 = quantile(Stat_GH2_ML, [.99; .95; .90])';
cv_GH3 = quantile(Stat_GH3_ML, [.99; .95; .90])';
cv_GH4 = quantile(Stat_GH4_ML, [.99; .95; .90])';

cv_KTk0 = quantile(StatKT_k0_ML, [.99; .95; .90])';
cv_KTk1 = quantile(StatKT_k1_ML, [.99; .95; .90])';
cv_KTk2 = quantile(StatKT_k2_ML, [.99; .95; .90])';
cv_KTk3 = quantile(StatKT_k3_ML, [.99; .95; .90])';
cv_KTk4 = quantile(StatKT_k4_ML, [.99; .95; .90])';

cv_KTGH0 = quantile(StatKT_GH0_ML, [.99; .95; .90])';
cv_KTGH1 = quantile(StatKT_GH1_ML, [.99; .95; .90])';
cv_KTGH2 = quantile(StatKT_GH2_ML, [.99; .95; .90])';
cv_KTGH3 = quantile(StatKT_GH3_ML, [.99; .95; .90])';
cv_KTGH4 = quantile(StatKT_GH4_ML, [.99; .95; .90])';

cv_HKk2 = quantile(Stat_HKk2_ML, [.99; .95; .90])';
cv_HKk3 = quantile(Stat_HKk3_ML, [.99; .95; .90])';
cv_HKs2 = quantile(Stat_HKs2_ML, [.99; .95; .90])';
cv_HKs3 = quantile(Stat_HKs3_ML, [.99; .95; .90])';
cv_HKGH2 = quantile(Stat_HKGH2_ML, [.99; .95; .90])';
cv_HKGH3 = quantile(Stat_HKGH3_ML, [.99; .95; .90])';

save crit.mat cv_k0 cv_k1 cv_k2 cv_k3 cv_k4 ...
    cv_s0 cv_s1 cv_s2 cv_s3 cv_s4 ...
    cv_GH0 cv_GH1 cv_GH2 cv_GH3 cv_GH4 ...
    cv_KTk0 cv_KTk1 cv_KTk2 cv_KTk3 cv_KTk4 ...
    cv_KTGH0 cv_KTGH1 cv_KTGH2 cv_KTGH3 cv_KTGH4 ...
    cv_HKk2 cv_HKk3 cv_HKs2 cv_HKs3 cv_HKGH2 cv_HKGH3

movefile('crit.mat',['IO/bootstrap/', title,'_critical_T',num2str(T),'.mat']);


%% BOOTSTRAP TEST

h = waitbar(0,'Computation of test statistic');
load(['IO/bootstrap/', title,'_critical_T',num2str(T),'.mat'])
load(['IO/estimate/', title,'_estimate_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat'])
load(['IO/test/', title,'_test_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat'])

R2_min = 0.01;
R2_max = 0.86;
R2_step = (R2_max-R2_min)/(S_grid-1);

R2 = 1/sqrt(1+4/q);

if (R2 < R2_min)
    id_R2 = 1;
    w_R2 = 1;
elseif (R2 >= R2_max)
    id_R2 = S_grid-1;
    w_R2 = 0;
else
    id_R2 = ceil((R2-R2_min)/R2_step);
    w_R2 = id_R2 - (R2-R2_min)/R2_step;
end

for s = 1:S
    
    q_ML = MLestimate(2,s)/MLestimate(3,s);
    R2_ML = 1/sqrt(1+4/q_ML);
    
    if (R2_ML <= R2_min)
        id_R2_ML = 1;
        w_R2_ML = 1;
    elseif (R2_ML >= R2_max)
        id_R2_ML = S_grid-1;
        w_R2_ML = 0;
    else
        id_R2_ML = ceil((R2_ML-R2_min)/R2_step);
        w_R2_ML = id_R2_ML - (R2_ML-R2_min)/R2_step;
    end
    
    position = id_R2:id_R2+1;
    position_ML = id_R2_ML:id_R2_ML+1;
    
    for index = [1,2,3]
        
        rej_k(1,index,s) = (Stat_k0(1,s) > [w_R2,1-w_R2]*cv_k0(position,index));
        rej_k(2,index,s) = (Stat_k1(1,s) > [w_R2,1-w_R2]*cv_k1(position,index));
        rej_k(3,index,s) = (Stat_k2(1,s) > [w_R2,1-w_R2]*cv_k2(position,index));
        rej_k(4,index,s) = (Stat_k3(1,s) > [w_R2,1-w_R2]*cv_k3(position,index));
        rej_k(5,index,s) = (Stat_k4(1,s) > [w_R2,1-w_R2]*cv_k4(position,index));
        
        rej_s(1,index,s) = (Stat_s0(1,s) > [w_R2,1-w_R2]*cv_s0(position,index));
        rej_s(2,index,s) = (Stat_s1(1,s) > [w_R2,1-w_R2]*cv_s1(position,index));
        rej_s(3,index,s) = (Stat_s2(1,s) > [w_R2,1-w_R2]*cv_s2(position,index));
        rej_s(4,index,s) = (Stat_s3(1,s) > [w_R2,1-w_R2]*cv_s3(position,index));
        rej_s(5,index,s) = (Stat_s4(1,s) > [w_R2,1-w_R2]*cv_s4(position,index));
        
        rej_GH(1,index,s) = (Stat_GH0(1,s) > [w_R2,1-w_R2]*cv_GH0(position,index));
        rej_GH(2,index,s) = (Stat_GH1(1,s) > [w_R2,1-w_R2]*cv_GH1(position,index));
        rej_GH(3,index,s) = (Stat_GH2(1,s) > [w_R2,1-w_R2]*cv_GH2(position,index));
        rej_GH(4,index,s) = (Stat_GH3(1,s) > [w_R2,1-w_R2]*cv_GH3(position,index));
        rej_GH(5,index,s) = (Stat_GH4(1,s) > [w_R2,1-w_R2]*cv_GH4(position,index));
        
        rej_KTk(1,index,s) = (StatKT_k0(1,s) > [w_R2,1-w_R2]*cv_KTk0(position,index));
        rej_KTk(2,index,s) = (StatKT_k1(1,s) > [w_R2,1-w_R2]*cv_KTk1(position,index));
        rej_KTk(3,index,s) = (StatKT_k2(1,s) > [w_R2,1-w_R2]*cv_KTk2(position,index));
        rej_KTk(4,index,s) = (StatKT_k3(1,s) > [w_R2,1-w_R2]*cv_KTk3(position,index));
        rej_KTk(5,index,s) = (StatKT_k4(1,s) > [w_R2,1-w_R2]*cv_KTk4(position,index));
        
        rej_KTGH(1,index,s) = (StatKT_GH0(1,s) > [w_R2,1-w_R2]*cv_KTGH0(position,index));
        rej_KTGH(2,index,s) = (StatKT_GH1(1,s) > [w_R2,1-w_R2]*cv_KTGH1(position,index));
        rej_KTGH(3,index,s) = (StatKT_GH2(1,s) > [w_R2,1-w_R2]*cv_KTGH2(position,index));
        rej_KTGH(4,index,s) = (StatKT_GH3(1,s) > [w_R2,1-w_R2]*cv_KTGH3(position,index));
        rej_KTGH(5,index,s) = (StatKT_GH4(1,s) > [w_R2,1-w_R2]*cv_KTGH4(position,index));
        
        rej_k_ML(1,index,s) = (Stat_k0_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_k0(position_ML,index));
        rej_k_ML(2,index,s) = (Stat_k1_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_k1(position_ML,index));
        rej_k_ML(3,index,s) = (Stat_k2_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_k2(position_ML,index));
        rej_k_ML(4,index,s) = (Stat_k3_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_k3(position_ML,index));
        rej_k_ML(5,index,s) = (Stat_k4_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_k4(position_ML,index));
        
        rej_s_ML(1,index,s) = (Stat_s0_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_s0(position_ML,index));
        rej_s_ML(2,index,s) = (Stat_s1_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_s1(position_ML,index));
        rej_s_ML(3,index,s) = (Stat_s2_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_s2(position_ML,index));
        rej_s_ML(4,index,s) = (Stat_s3_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_s3(position_ML,index));
        rej_s_ML(5,index,s) = (Stat_s4_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_s4(position_ML,index));
        
        rej_GH_ML(1,index,s) = (Stat_GH0_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_GH0(position_ML,index));
        rej_GH_ML(2,index,s) = (Stat_GH1_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_GH1(position_ML,index));
        rej_GH_ML(3,index,s) = (Stat_GH2_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_GH2(position_ML,index));
        rej_GH_ML(4,index,s) = (Stat_GH3_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_GH3(position_ML,index));
        rej_GH_ML(5,index,s) = (Stat_GH4_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_GH4(position_ML,index));
        
        rej_KTk_ML(1,index,s) = (StatKT_k0_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_KTk0(position_ML,index));
        rej_KTk_ML(2,index,s) = (StatKT_k1_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_KTk1(position_ML,index));
        rej_KTk_ML(3,index,s) = (StatKT_k2_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_KTk2(position_ML,index));
        rej_KTk_ML(4,index,s) = (StatKT_k3_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_KTk3(position_ML,index));
        rej_KTk_ML(5,index,s) = (StatKT_k4_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_KTk4(position_ML,index));
        
        rej_KTGH_ML(1,index,s) = (StatKT_GH0_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_KTGH0(position_ML,index));
        rej_KTGH_ML(2,index,s) = (StatKT_GH1_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_KTGH1(position_ML,index));
        rej_KTGH_ML(3,index,s) = (StatKT_GH2_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_KTGH2(position_ML,index));
        rej_KTGH_ML(4,index,s) = (StatKT_GH3_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_KTGH3(position_ML,index));
        rej_KTGH_ML(5,index,s) = (StatKT_GH4_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_KTGH4(position_ML,index));
        
        rej_HK(1,index,s) = (Stat_HKk2(1,s) > [w_R2,1-w_R2]*cv_HKk2(position,index));
        rej_HK(2,index,s) = (Stat_HKk3(1,s) > [w_R2,1-w_R2]*cv_HKk3(position,index));
        rej_HK(3,index,s) = (Stat_HKs2(1,s) > [w_R2,1-w_R2]*cv_HKs2(position,index));
        rej_HK(4,index,s) = (Stat_HKs3(1,s) > [w_R2,1-w_R2]*cv_HKs3(position,index));
        rej_HK(5,index,s) = (Stat_HKGH2(1,s) > [w_R2,1-w_R2]*cv_HKGH2(position,index));
        rej_HK(6,index,s) = (Stat_HKGH3(1,s) > [w_R2,1-w_R2]*cv_HKGH3(position,index));
        
        rej_HK_ML(1,index,s) = (Stat_HKk2_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_HKk2(position_ML,index));
        rej_HK_ML(2,index,s) = (Stat_HKk3_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_HKk3(position_ML,index));
        rej_HK_ML(3,index,s) = (Stat_HKs2_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_HKs2(position_ML,index));
        rej_HK_ML(4,index,s) = (Stat_HKs3_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_HKs3(position_ML,index));
        rej_HK_ML(5,index,s) = (Stat_HKGH2_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_HKGH2(position_ML,index));
        rej_HK_ML(6,index,s) = (Stat_HKGH3_ML(1,s) > [w_R2_ML,1-w_R2_ML]*cv_HKGH3(position_ML,index));
        
    end
    
    if (rem(s, safe) == 0 || s == S)        
        for index = [1,2,3]
            
            reject(1,index,1) = mean(rej_k(1,index,:),3);
            reject(2,index,1) = mean(rej_k(2,index,:),3);
            reject(3,index,1) = mean(rej_k(3,index,:),3);
            reject(4,index,1) = mean(rej_k(4,index,:),3);
            reject(5,index,1) = mean(rej_k(5,index,:),3);
            
            reject(1,index,2) = mean(rej_s(1,index,:),3);
            reject(2,index,2) = mean(rej_s(2,index,:),3);
            reject(3,index,2) = mean(rej_s(3,index,:),3);
            reject(4,index,2) = mean(rej_s(4,index,:),3);
            reject(5,index,2) = mean(rej_s(5,index,:),3);
            
            reject(1,index,3) = mean(rej_GH(1,index,:),3);
            reject(2,index,3) = mean(rej_GH(2,index,:),3);
            reject(3,index,3) = mean(rej_GH(3,index,:),3);
            reject(4,index,3) = mean(rej_GH(4,index,:),3);
            reject(5,index,3) = mean(rej_GH(5,index,:),3);
            
            reject_ML(1,index,1) = mean(rej_k_ML(1,index,:),3);
            reject_ML(2,index,1) = mean(rej_k_ML(2,index,:),3);
            reject_ML(3,index,1) = mean(rej_k_ML(3,index,:),3);
            reject_ML(4,index,1) = mean(rej_k_ML(4,index,:),3);
            reject_ML(5,index,1) = mean(rej_k_ML(5,index,:),3);
            
            reject_ML(1,index,2) = mean(rej_s_ML(1,index,:),3);
            reject_ML(2,index,2) = mean(rej_s_ML(2,index,:),3);
            reject_ML(3,index,2) = mean(rej_s_ML(3,index,:),3);
            reject_ML(4,index,2) = mean(rej_s_ML(4,index,:),3);
            reject_ML(5,index,2) = mean(rej_s_ML(5,index,:),3);
            
            reject_ML(1,index,3) = mean(rej_GH_ML(1,index,:),3);
            reject_ML(2,index,3) = mean(rej_GH_ML(2,index,:),3);
            reject_ML(3,index,3) = mean(rej_GH_ML(3,index,:),3);
            reject_ML(4,index,3) = mean(rej_GH_ML(4,index,:),3);
            reject_ML(5,index,3) = mean(rej_GH_ML(5,index,:),3);
            
            rejectKT(1,index,1) = mean(rej_KTk(1,index,:),3);
            rejectKT(2,index,1) = mean(rej_KTk(2,index,:),3);
            rejectKT(3,index,1) = mean(rej_KTk(3,index,:),3);
            rejectKT(4,index,1) = mean(rej_KTk(4,index,:),3);
            rejectKT(5,index,1) = mean(rej_KTk(5,index,:),3);
            
            rejectKT(1,index,2) = mean(rej_KTGH(1,index,:),3);
            rejectKT(2,index,2) = mean(rej_KTGH(2,index,:),3);
            rejectKT(3,index,2) = mean(rej_KTGH(3,index,:),3);
            rejectKT(4,index,2) = mean(rej_KTGH(4,index,:),3);
            rejectKT(5,index,2) = mean(rej_KTGH(5,index,:),3);
            
            rejectKT_ML(1,index,1) = mean(rej_KTk_ML(1,index,:),3);
            rejectKT_ML(2,index,1) = mean(rej_KTk_ML(2,index,:),3);
            rejectKT_ML(3,index,1) = mean(rej_KTk_ML(3,index,:),3);
            rejectKT_ML(4,index,1) = mean(rej_KTk_ML(4,index,:),3);
            rejectKT_ML(5,index,1) = mean(rej_KTk_ML(5,index,:),3);
            
            rejectKT_ML(1,index,2) = mean(rej_KTGH_ML(1,index,:),3);
            rejectKT_ML(2,index,2) = mean(rej_KTGH_ML(2,index,:),3);
            rejectKT_ML(3,index,2) = mean(rej_KTGH_ML(3,index,:),3);
            rejectKT_ML(4,index,2) = mean(rej_KTGH_ML(4,index,:),3);
            rejectKT_ML(5,index,2) = mean(rej_KTGH_ML(5,index,:),3);
            
            rejectHK(1,index,1) = mean(rej_HK(1,index,:),3);
            rejectHK(2,index,1) = mean(rej_HK(2,index,:),3);
            rejectHK(1,index,2) = mean(rej_HK(3,index,:),3);
            rejectHK(2,index,2) = mean(rej_HK(4,index,:),3);
            rejectHK(1,index,3) = mean(rej_HK(5,index,:),3);
            rejectHK(2,index,3) = mean(rej_HK(6,index,:),3);
            
            rejectHK_ML(1,index,1) = mean(rej_HK_ML(1,index,:),3);
            rejectHK_ML(2,index,1) = mean(rej_HK_ML(2,index,:),3);
            rejectHK_ML(1,index,2) = mean(rej_HK_ML(3,index,:),3);
            rejectHK_ML(2,index,2) = mean(rej_HK_ML(4,index,:),3);
            rejectHK_ML(1,index,3) = mean(rej_HK_ML(5,index,:),3);
            rejectHK_ML(2,index,3) = mean(rej_HK_ML(6,index,:),3);
            
        end
        save btest.mat s reject reject_ML rejectKT rejectKT_ML rejectHK rejectHK_ML
    end
    waitbar(s/S,h)
end
close(h)

movefile('btest.mat',['IO/bootstrap/', title,'_bootstrap_test_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat']);