clear
clc
close all
path([pwd, '/subprograms/'], path)


%% SETTINGS AND PREALLOCATION

% Simulation settings
S = 10000; % Number of Monte Carlo replicas
safe = ceil(0.05*S); % Number of replicas per saving at each stage
title = 'S622';

rng(261009)
options = optimoptions('fmincon','Algorithm','interior-point',...
    'GradObj','off','Hessian','off','Display','iter',...
    'MaxFunEvals',1000,'MaxIter',1000,'TolCon',0.0,'TolFun',10e-9);
maxerror = 1e-4; % Tolerance criterion for numerical convergence
maxiter = 1e2; % Maximum number of iterations for numerical convergence

% Model settings
T = 100; % Sample size
hypothesis = 0; % Distribution of innovations
NW_lags = ceil(T^(1/3));

c = 0;
q = 2;
parameters = [c;1;1/q];

nu = 8; % Degrees of freedom
beta = [-1,-1]; % Direction of asymmetry

N = 1;
M = 2;
K = 2;
d = size(parameters,1);

% Preallocation
Stat_k0 = zeros(1,S);                                                      % Case-0 statistic of kurtosis component at true parameter value
Stat_k1 = zeros(1,S);                                                      % Case-1 statistic of kurtosis component at true parameter value
Stat_k2 = zeros(1,S);                                                      % Case-2 statistic of kurtosis component at true parameter value
Stat_k3 = zeros(1,S);                                                      % Case-3 statistic of kurtosis component at true parameter value
Stat_k4 = zeros(1,S);                                                      % Case-4 statistic of kurtosis component at true parameter value

Stat_s0 = zeros(1,S);                                                      % Case-0 statistic of skewness component at true parameter value
Stat_s1 = zeros(1,S);                                                      % Case-1 statistic of skewness component at true parameter value
Stat_s2 = zeros(1,S);                                                      % Case-2 statistic of skewness component at true parameter value
Stat_s3 = zeros(1,S);                                                      % Case-3 statistic of skewness component at true parameter value
Stat_s4 = zeros(1,S);                                                      % Case-4 statistic of skewness component at true parameter value

Stat_GH0 = zeros(1,S);                                                     % Case-0 full statistic at true parameter value
Stat_GH1 = zeros(1,S);                                                     % Case-1 full statistic at true parameter value
Stat_GH2 = zeros(1,S);                                                     % Case-2 full statistic at true parameter value
Stat_GH3 = zeros(1,S);                                                     % Case-3 full statistic at true parameter value
Stat_GH4 = zeros(1,S);                                                     % Case-4 full statistic at true parameter value

StatKT_k0 = zeros(1,S);                                                    % Case-0 KT statistic of kurtosis component at true parameter value
StatKT_k1 = zeros(1,S);                                                    % Case-1 KT statistic of kurtosis component at true parameter value
StatKT_k2 = zeros(1,S);                                                    % Case-2 KT statistic of kurtosis component at true parameter value
StatKT_k3 = zeros(1,S);                                                    % Case-3 KT statistic of kurtosis component at true parameter value
StatKT_k4 = zeros(1,S);                                                    % Case-4 KT statistic of kurtosis component at true parameter value

StatKT_GH0 = zeros(1,S);                                                   % Case-0 KT full statistic at true parameter value
StatKT_GH1 = zeros(1,S);                                                   % Case-1 KT full statistic at true parameter value
StatKT_GH2 = zeros(1,S);                                                   % Case-2 KT full statistic at true parameter value
StatKT_GH3 = zeros(1,S);                                                   % Case-3 KT full statistic at true parameter value
StatKT_GH4 = zeros(1,S);                                                   % Case-4 KT full statistic at true parameter value

Stat_k0_ML = zeros(1,S);                                                   % Case-0 statistic of kurtosis component at ML estimate
Stat_k1_ML = zeros(1,S);                                                   % Case-1 statistic of kurtosis component at ML estimate
Stat_k2_ML = zeros(1,S);                                                   % Case-2 statistic of kurtosis component at ML estimate
Stat_k3_ML = zeros(1,S);                                                   % Case-3 statistic of kurtosis component at ML estimate
Stat_k4_ML = zeros(1,S);                                                   % Case-4 statistic of kurtosis component at ML estimate

Stat_s0_ML = zeros(1,S);                                                   % Case-0 statistic of skewness component at ML estimate
Stat_s1_ML = zeros(1,S);                                                   % Case-1 statistic of skewness component at ML estimate
Stat_s2_ML = zeros(1,S);                                                   % Case-2 statistic of skewness component at ML estimate
Stat_s3_ML = zeros(1,S);                                                   % Case-3 statistic of skewness component at ML estimate
Stat_s4_ML = zeros(1,S);                                                   % Case-4 statistic of skewness component at ML estimate

Stat_GH0_ML = zeros(1,S);                                                  % Case-0 full statistic at ML estimate
Stat_GH1_ML = zeros(1,S);                                                  % Case-1 full statistic at ML estimate
Stat_GH2_ML = zeros(1,S);                                                  % Case-2 full statistic at ML estimate
Stat_GH3_ML = zeros(1,S);                                                  % Case-3 full statistic at ML estimate
Stat_GH4_ML = zeros(1,S);                                                  % Case-4 full statistic at ML estimate

StatKT_k0_ML = zeros(1,S);                                                 % Case-0 KT statistic of kurtosis component at ML estimate
StatKT_k1_ML = zeros(1,S);                                                 % Case-1 KT statistic of kurtosis component at ML estimate
StatKT_k2_ML = zeros(1,S);                                                 % Case-2 KT statistic of kurtosis component at ML estimate
StatKT_k3_ML = zeros(1,S);                                                 % Case-3 KT statistic of kurtosis component at ML estimate
StatKT_k4_ML = zeros(1,S);                                                 % Case-4 KT statistic of kurtosis component at ML estimate

StatKT_GH0_ML = zeros(1,S);                                                % Case-0 full KT statistic at ML estimate
StatKT_GH1_ML = zeros(1,S);                                                % Case-1 full KT statistic at ML estimate
StatKT_GH2_ML = zeros(1,S);                                                % Case-2 full KT statistic at ML estimate
StatKT_GH3_ML = zeros(1,S);                                                % Case-3 full KT statistic at ML estimate
StatKT_GH4_ML = zeros(1,S);                                                % Case-4 full KT statistic at ML estimate

rejection = zeros(5,3,3);
rejection_ML = zeros(5,3,3);
rejectionKT = zeros(5,3,2);
rejectionKT_ML = zeros(5,3,2);


%% COMPUTATION OF HAC-BASED TEST

h = waitbar(0,'Computation of score');
load(['IO/estimate/', title,'_estimate_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat'])

for s = 1:S
    
    display(s,'Replica')
    
    Y = Data(:,:,s);
    
    kappa = 10e5;
    C = [parameters(1);...
        zeros(3,1)];
    F = [1,zeros(1,3);...
        zeros(3,4)];
    G = [diag([sqrt(parameters(2)),sqrt(parameters(3))]);...
        1,0;...
        0,1];
    H = [1,1,0,0];
    EX1 = zeros(4,1);
    VX1 = [diag([kappa*parameters(2),parameters(3)]),zeros(2,2);...
        zeros(2,2),eye(2)];
    
    C_ML = [MLestimate(1,s);...
        zeros(3,1)];
    G_ML = [diag([sqrt(MLestimate(2,s)),sqrt(MLestimate(3,s))]);...
        1,0;...
        0,1];
    VX1_ML = [diag([kappa*MLestimate(2,s),MLestimate(3,s)]),...
        zeros(2,2);...
        zeros(2,2),...
        eye(2)];
    
    [Xf,Pf,Xs,Ps] = KalmanFS(EX1,VX1,F,G,H,Y,C);
    
    e0 = zeros(N,T-2);
    P0 = zeros(N,N,T-2);
    for t = 2:T-1
        e0(:,t-1) = Y(:,t) - H*Xf(:,t);
        P0(:,:,t-1) = H*Pf(:,:,t)*H';
    end
    e1 = Xs(M+1:M+K,2:T-1);
    P1 = Ps(M+1:M+K,M+1:M+K,2:T-1);
    e2 = Xs(M+2:M+K,2:T-1);
    P2 = Ps(M+2:M+K,M+2:M+K,2:T-1);
    e3 = Xs(M+1,2:T-1);
    P3 = Ps(M+1,M+1,2:T-1);
    
    [Xf_ML,Pf_ML,Xs_ML,Ps_ML] = KalmanFS(EX1,VX1_ML,F,G_ML,H,Y,C_ML);
    
    e0_ML = zeros(N,T-2);
    P0_ML = zeros(N,N,T-2);
    for t = 2:T-1
        e0_ML(:,t-1) = Y(:,t) - H*Xf_ML(:,t);
        P0_ML(:,:,t-1) = H*Pf_ML(:,:,t)*H';
    end
    e1_ML = Xs_ML(M+1:M+K,2:T-1);
    P1_ML = Ps_ML(M+1:M+K,M+1:M+K,2:T-1);
    e2_ML = Xs_ML(M+2:M+K,2:T-1);
    P2_ML = Ps_ML(M+2:M+K,M+2:M+K,2:T-1);
    e3_ML = Xs_ML(M+1,2:T-1);
    P3_ML = Ps_ML(M+1,M+1,2:T-1);
    
    % Computation of test statistics
    % Our tests at true values
    sk0 = ScoreFK(e0,P0);
    sk1 = ScoreSK(e1,P1);
    sk2 = ScoreSK(e2,P2);
    sk3 = ScoreSK(e3,P3);
    sk4 = [sk2;sk3];
    
    ss0 = ScoreFS(e0,P0);
    ss1 = ScoreSS(e1,P1);
    ss2 = ScoreSS(e2,P2);
    ss3 = ScoreSS(e3,P3);
    ss4 = [ss2;ss3];
    
    Scr_k0 = mean(sk0,2);
    Scr_k1 = mean(sk1,2);
    Scr_k2 = mean(sk2,2);
    Scr_k3 = mean(sk3,2);
    
    Scr_s0 = mean(ss0,2);
    Scr_s1 = mean(ss1,2);
    Scr_s2 = mean(ss2,2);
    Scr_s3 = mean(ss3,2);
    
    Var_k0 = AutoCovariance(sk0',0);
    Var_k1 = AutoCovariance(sk1',0);
    Var_k2 = AutoCovariance(sk2',0);
    Var_k3 = AutoCovariance(sk3',0);
    Var_k4 = AutoCovariance(sk4',0);
    
    Var_s0 = AutoCovariance(ss0',0);
    Var_s1 = AutoCovariance(ss1',0);
    Var_s2 = AutoCovariance(ss2',0);
    Var_s3 = AutoCovariance(ss3',0);
    Var_s4 = AutoCovariance(ss4',0);
    
    for j = 1:NW_lags
        weight = 1 - j/(NW_lags+1);
        Var_k0 = Var_k0 + 2*weight*AutoCovariance(sk0',j);
        Var_k1 = Var_k1 + 2*weight*AutoCovariance(sk1',j);
        Var_k2 = Var_k2 + 2*weight*AutoCovariance(sk2',j);
        Var_k3 = Var_k3 + 2*weight*AutoCovariance(sk3',j);
        Var_k4 = Var_k4 + weight*AutoCovariance(sk4',j) + weight*AutoCovariance(sk4',j);
        
        Var_s0 = Var_s0 + 2*weight*AutoCovariance(ss0',j);
        Var_s1 = Var_s1 + weight*AutoCovariance(ss1',j) + weight*AutoCovariance(ss1',j)';
        Var_s2 = Var_s2 + 2*weight*AutoCovariance(ss2',j);
        Var_s3 = Var_s3 + 2*weight*AutoCovariance(ss3',j);
        Var_s4 = Var_s4 + weight*AutoCovariance(ss4',j) + weight*AutoCovariance(ss4',j)';
    end
    
    Stat_k0(:,s) = T*(Scr_k0.^2)/(Var_k0);
    Stat_k1(:,s) = T*(Scr_k1.^2)/(Var_k1);
    Stat_k2(:,s) = T*(Scr_k2.^2)/(Var_k2);
    Stat_k3(:,s) = T*(Scr_k3.^2)/(Var_k3);
    Scr_k4 = [Scr_k2;Scr_k3];
    Stat_k4(:,s) = T*Scr_k4'*((Var_k4)\Scr_k4);
    
    Stat_s0(:,s) = T*Scr_s0'*((Var_s0)\Scr_s0);
    Stat_s1(:,s) = T*Scr_s1'*((Var_s1)\Scr_s1);
    Stat_s2(:,s) = T*Scr_s2'*((Var_s2)\Scr_s2);
    Stat_s3(:,s) = T*Scr_s3'*((Var_s3)\Scr_s3);
    Scr_s4 = [Scr_s2;Scr_s3];
    Stat_s4(:,s) = T*Scr_s4'*((Var_s4)\Scr_s4);
    
    Stat_GH0(:,s) = Stat_k0(:,s) + Stat_s0(:,s);
    Stat_GH1(:,s) = Stat_k1(:,s) + Stat_s1(:,s);
    Stat_GH2(:,s) = Stat_k2(:,s) + Stat_s2(:,s);
    Stat_GH3(:,s) = Stat_k3(:,s) + Stat_s3(:,s);
    Stat_GH4(:,s) = Stat_k4(:,s) + Stat_s4(:,s);
    
    StatKT_k0(:,s) = Stat_k0(:,s) * (Scr_k0 > 0);
    StatKT_k1(:,s) = Stat_k1(:,s) * (Scr_k1 > 0);
    StatKT_k2(:,s) = Stat_k2(:,s) * (Scr_k2 > 0);
    StatKT_k3(:,s) = Stat_k3(:,s) * (Scr_k3 > 0);
    Scr_k4 = Scr_k4 .* (Scr_k4 > 0);
    StatKT_k4(:,s) = T*Scr_k4'*((Var_k4)\Scr_k4);
    
    StatKT_GH0(:,s) = StatKT_k0(:,s) + Stat_s0(:,s);
    StatKT_GH1(:,s) = StatKT_k1(:,s) + Stat_s1(:,s);
    StatKT_GH2(:,s) = StatKT_k2(:,s) + Stat_s2(:,s);
    StatKT_GH3(:,s) = StatKT_k3(:,s) + Stat_s3(:,s);
    StatKT_GH4(:,s) = StatKT_k4(:,s) + Stat_s4(:,s);
    
    % Our tests at ML estimates
    sk0_ML = ScoreFK(e0_ML,P0_ML);
    sk1_ML = ScoreSK(e1_ML,P1_ML);
    sk2_ML = ScoreSK(e2_ML,P2_ML);
    sk3_ML = ScoreSK(e3_ML,P3_ML);
    sk4_ML = [sk2_ML;sk3_ML];
    
    ss0_ML = ScoreFS(e0_ML,P0_ML);
    ss1_ML = ScoreSS(e1_ML,P1_ML);
    ss2_ML = ScoreSS(e2_ML,P2_ML);
    ss3_ML = ScoreSS(e3_ML,P3_ML);
    ss4_ML = [ss2_ML;ss3_ML];
    
    Scr_k0_ML = mean(sk0_ML,2);
    Scr_k1_ML = mean(sk1_ML,2);
    Scr_k2_ML = mean(sk2_ML,2);
    Scr_k3_ML = mean(sk3_ML,2);
    
    Scr_s0_ML = mean(ss0_ML,2);
    Scr_s1_ML = mean(ss1_ML,2);
    Scr_s2_ML = mean(ss2_ML,2);
    Scr_s3_ML = mean(ss3_ML,2);
    
    Var_k0_ML = AutoCovariance(sk0_ML',0);
    Var_k1_ML = AutoCovariance(sk1_ML',0);
    Var_k2_ML = AutoCovariance(sk2_ML',0);
    Var_k3_ML = AutoCovariance(sk3_ML',0);
    Var_k4_ML = AutoCovariance(sk4_ML',0);
    
    Var_s0_ML = AutoCovariance(ss0_ML',0);
    Var_s1_ML = AutoCovariance(ss1_ML',0);
    Var_s2_ML = AutoCovariance(ss2_ML',0);
    Var_s3_ML = AutoCovariance(ss3_ML',0);
    Var_s4_ML = AutoCovariance(ss4_ML',0);
    
    for j = 1:NW_lags
        weight = 1 - j/(NW_lags+1);
        Var_k0_ML = Var_k0_ML + 2*weight*AutoCovariance(sk0_ML',j);
        Var_k1_ML = Var_k1_ML + 2*weight*AutoCovariance(sk1_ML',j);
        Var_k2_ML = Var_k2_ML + 2*weight*AutoCovariance(sk2_ML',j);
        Var_k3_ML = Var_k3_ML + 2*weight*AutoCovariance(sk3_ML',j);
        Var_k4_ML = Var_k4_ML + weight*AutoCovariance(sk4_ML',j) + weight*AutoCovariance(sk4_ML',j)';
        
        Var_s0_ML = Var_s0_ML + 2*weight*AutoCovariance(ss0_ML',j);
        Var_s1_ML = Var_s1_ML + weight*AutoCovariance(ss1_ML',j) + weight*AutoCovariance(ss1_ML',j)';
        Var_s2_ML = Var_s2_ML + 2*weight*AutoCovariance(ss2_ML',j);
        Var_s3_ML = Var_s3_ML + 2*weight*AutoCovariance(ss3_ML',j);
        Var_s4_ML = Var_s4_ML + weight*AutoCovariance(ss4_ML',j) + weight*AutoCovariance(ss4_ML',j)';
    end
    
    Stat_k0_ML(:,s) = T*(Scr_k0_ML.^2)/(Var_k0_ML);
    Stat_k1_ML(:,s) = T*(Scr_k1_ML.^2)/(Var_k1_ML);
    Stat_k2_ML(:,s) = T*(Scr_k2_ML.^2)/(Var_k2_ML);
    Stat_k3_ML(:,s) = T*(Scr_k3_ML.^2)/(Var_k3_ML);
    Scr_k4_ML = [Scr_k2_ML;Scr_k3_ML];
    Stat_k4_ML(:,s) = T*Scr_k4_ML'*((Var_k4_ML)\Scr_k4_ML);
    
    Stat_s0_ML(:,s) = T*Scr_s0_ML'*((Var_s0_ML)\Scr_s0_ML);
    Stat_s1_ML(:,s) = T*Scr_s1_ML'*((Var_s1_ML)\Scr_s1_ML);
    Stat_s2_ML(:,s) = T*Scr_s2_ML'*((Var_s2_ML)\Scr_s2_ML);
    Stat_s3_ML(:,s) = T*Scr_s3_ML'*((Var_s3_ML)\Scr_s3_ML);
    Scr_s4_ML = [Scr_s2_ML;Scr_s3_ML];
    Stat_s4_ML(:,s) = T*Scr_s4_ML'*((Var_s4_ML)\Scr_s4_ML);
    
    Stat_GH0_ML(:,s) = Stat_k0_ML(:,s) + Stat_s0_ML(:,s);
    Stat_GH1_ML(:,s) = Stat_k1_ML(:,s) + Stat_s1_ML(:,s);
    Stat_GH2_ML(:,s) = Stat_k2_ML(:,s) + Stat_s2_ML(:,s);
    Stat_GH3_ML(:,s) = Stat_k3_ML(:,s) + Stat_s3_ML(:,s);
    Stat_GH4_ML(:,s) = Stat_k4_ML(:,s) + Stat_s4_ML(:,s);
    
    StatKT_k0_ML(:,s) = Stat_k0_ML(:,s) * (Scr_k0_ML > 0);
    StatKT_k1_ML(:,s) = Stat_k1_ML(:,s) * (Scr_k1_ML > 0);
    StatKT_k2_ML(:,s) = Stat_k2_ML(:,s) * (Scr_k2_ML > 0);
    StatKT_k3_ML(:,s) = Stat_k3_ML(:,s) * (Scr_k3_ML > 0);
    Scr_k4_ML = Scr_k4_ML .* (Scr_k4_ML > 0);
    StatKT_k4_ML(:,s) = T*Scr_k4_ML'*((Var_k4_ML)\Scr_k4_ML);
    
    StatKT_GH0_ML(:,s) = StatKT_k0_ML(:,s) + Stat_s0_ML(:,s);
    StatKT_GH1_ML(:,s) = StatKT_k1_ML(:,s) + Stat_s1_ML(:,s);
    StatKT_GH2_ML(:,s) = StatKT_k2_ML(:,s) + Stat_s2_ML(:,s);
    StatKT_GH3_ML(:,s) = StatKT_k3_ML(:,s) + Stat_s3_ML(:,s);
    StatKT_GH4_ML(:,s) = StatKT_k4_ML(:,s) + Stat_s4_ML(:,s);
    
    waitbar(s/S,h)
    
    % Saving results
    if (rem(s, safe) == 0 || s == S)
        
        for level = [1,5,10]
            if (level == 1)
                index = 1;
            elseif (level == 5)
                index = 2;
            elseif (level == 10)
                index = 3;
            end
            
            c1 = chi2inv(1-level/100,1);
            c2 = chi2inv(1-level/100,2);
            c3 = chi2inv(1-level/100,3);
            c4 = chi2inv(1-level/100,4);
            
            cKT1 = (level == 1)*5.41189 + (level == 5)*2.70554 ...
                + (level == 10)*1.64237;
            cKT2 = (level == 1)*8.27325 + (level == 5)*5.13838 ...
                + (level == 10)*3.80781;
            cKT3 = (level == 1)*10.5009 + (level == 5)*7.04506 ...
                + (level == 10)*5.52814;
            
            if (q < 1.5) % q = 0.5
                cKTcase4t = (level == 1)*7.33623 + (level == 5)*4.26121 ...
                    + (level == 10)*2.97309;
                cKTcase4GH = (level == 1)*11.7016 + (level == 5)*8.03853 ...
                    + (level == 10)*6.40778;
            else % q = 2.0
                cKTcase4t = (level == 1)*7.38755 + (level == 5)*4.29556 ...
                    + (level == 10)*2.99655;
                cKTcase4GH = (level == 1)*11.7294 + (level == 5)*8.02211 ...
                    + (level == 10)*6.42092;
            end
            
            rejection(1,index,1) = sum(Stat_k0 > c1)/S;
            rejection(2,index,1) = sum(Stat_k1 > c1)/S;
            rejection(3,index,1) = sum(Stat_k2 > c1)/S;
            rejection(4,index,1) = sum(Stat_k3 > c1)/S;
            rejection(5,index,1) = sum(Stat_k4 > c2)/S;
            
            rejection(1,index,2) = sum(Stat_s0 > c1)/S;
            rejection(2,index,2) = sum(Stat_s1 > c2)/S;
            rejection(3,index,2) = sum(Stat_s2 > c1)/S;
            rejection(4,index,2) = sum(Stat_s3 > c1)/S;
            rejection(5,index,2) = sum(Stat_s4 > c2)/S;
            
            rejection(1,index,3) = sum(Stat_GH0 > c2)/S;
            rejection(2,index,3) = sum(Stat_GH1 > c3)/S;
            rejection(3,index,3) = sum(Stat_GH2 > c2)/S;
            rejection(4,index,3) = sum(Stat_GH3 > c2)/S;
            rejection(5,index,3) = sum(Stat_GH4 > c4)/S;
            
            rejection_ML(1,index,1) = sum(Stat_k0_ML > c1)/S;
            rejection_ML(2,index,1) = sum(Stat_k1_ML > c1)/S;
            rejection_ML(3,index,1) = sum(Stat_k2_ML > c1)/S;
            rejection_ML(4,index,1) = sum(Stat_k3_ML > c1)/S;
            rejection_ML(5,index,1) = sum(Stat_k4_ML > c2)/S;
            
            rejection_ML(1,index,2) = sum(Stat_s0_ML > c1)/S;
            rejection_ML(2,index,2) = sum(Stat_s1_ML > c2)/S;
            rejection_ML(3,index,2) = sum(Stat_s2_ML > c1)/S;
            rejection_ML(4,index,2) = sum(Stat_s3_ML > c1)/S;
            rejection_ML(5,index,2) = sum(Stat_s4_ML > c2)/S;
            
            rejection_ML(1,index,3) = sum(Stat_GH0_ML > c2)/S;
            rejection_ML(2,index,3) = sum(Stat_GH1_ML > c3)/S;
            rejection_ML(3,index,3) = sum(Stat_GH2_ML > c2)/S;
            rejection_ML(4,index,3) = sum(Stat_GH3_ML > c2)/S;
            rejection_ML(5,index,3) = sum(Stat_GH4_ML > c4)/S;
            
            rejectionKT(1,index,1) = sum(StatKT_k0 > cKT1)/S;
            rejectionKT(2,index,1) = sum(StatKT_k1 > cKT1)/S;
            rejectionKT(3,index,1) = sum(StatKT_k2 > cKT1)/S;
            rejectionKT(4,index,1) = sum(StatKT_k3 > cKT1)/S;
            rejectionKT(5,index,1) = sum(StatKT_k4 > cKTcase4t)/S;
            
            rejectionKT(1,index,2) = sum(StatKT_GH0 > cKT2)/S;
            rejectionKT(2,index,2) = sum(StatKT_GH1 > cKT3)/S;
            rejectionKT(3,index,2) = sum(StatKT_GH2 > cKT2)/S;
            rejectionKT(4,index,2) = sum(StatKT_GH3 > cKT2)/S;
            rejectionKT(5,index,2) = sum(StatKT_GH4 > cKTcase4GH)/S;
            
            rejectionKT_ML(1,index,1) = sum(StatKT_k0_ML > cKT1)/S;
            rejectionKT_ML(2,index,1) = sum(StatKT_k1_ML > cKT1)/S;
            rejectionKT_ML(3,index,1) = sum(StatKT_k2_ML > cKT1)/S;
            rejectionKT_ML(4,index,1) = sum(StatKT_k3_ML > cKT1)/S;
            rejectionKT_ML(5,index,1) = sum(StatKT_k4_ML > cKTcase4t)/S;
            
            rejectionKT_ML(1,index,2) = sum(StatKT_GH0_ML > cKT2)/S;
            rejectionKT_ML(2,index,2) = sum(StatKT_GH1_ML > cKT3)/S;
            rejectionKT_ML(3,index,2) = sum(StatKT_GH2_ML > cKT2)/S;
            rejectionKT_ML(4,index,2) = sum(StatKT_GH3_ML > cKT2)/S;
            rejectionKT_ML(5,index,2) = sum(StatKT_GH4_ML > cKTcase4GH)/S;
        end
        
        save test.mat s rejection rejection_ML rejectionKT rejectionKT_ML ...
            Stat_k0 Stat_k1 Stat_k2 Stat_k3 Stat_k4 ...
            Stat_s0 Stat_s1 Stat_s2 Stat_s3 Stat_s4 ...
            Stat_GH0 Stat_GH1 Stat_GH2 Stat_GH3 Stat_GH4 ...
            StatKT_k0 StatKT_k1 StatKT_k2 StatKT_k3 StatKT_k4 ...
            StatKT_GH0 StatKT_GH1 StatKT_GH2 StatKT_GH3 StatKT_GH4 ...
            Stat_k0_ML Stat_k1_ML Stat_k2_ML Stat_k3_ML Stat_k4_ML ...
            Stat_s0_ML Stat_s1_ML Stat_s2_ML Stat_s3_ML Stat_s4_ML...
            Stat_GH0_ML Stat_GH1_ML Stat_GH2_ML Stat_GH3_ML Stat_GH4_ML ...
            StatKT_k0_ML StatKT_k1_ML StatKT_k2_ML StatKT_k3_ML StatKT_k4_ML ...
            StatKT_GH0_ML StatKT_GH1_ML StatKT_GH2_ML StatKT_GH3_ML StatKT_GH4_ML
    end
    
end
close(h)

movefile('test.mat',['IO/hac/', title,'_HAC_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat']);