clear
clc
close all
path([pwd, '/subprograms/'], path)


%% SETTINGS AND PREALLOCATION

% Simulation settings
S = 10000; % Number of Monte Carlo replicas
safe = ceil(0.5*S); % Number of replicas per saving at each stage
title = 'S623';

rng(261009)
options = optimoptions('fmincon','Algorithm','interior-point',...
    'GradObj','off','Hessian','off','Display','iter',...
    'MaxFunEvals',1000,'MaxIter',1000,'TolCon',0.0,'TolFun',10e-9);
maxerror = 1e-4; % Tolerance criterion for numerical convergence
maxiter = 1e2; % Maximum number of iterations for numerical convergence

% Model settings
T = 100; % Sample size
hypothesis = 0; % Distribution of innovations
BP = 200; % Burning period
NW_lags = ceil(T^(1/3));

N = 10;
M = N+1;
K = N+1;

gx = 2;
gv = 1;
parameters = [gx; gv];
nu = 8;                                                                    % Degrees of freedom
beta = -ones(1, K);                                                        % Direction of asymmetry
d = size(parameters,1);

% Preallocation
Data = zeros(N, T, S);                                                     % Simulated data processes
MLestimate = zeros(d, S);

Scr_k0 = zeros(1, S);
Scr_k1 = zeros(1, S);
Scr_k2 = zeros(1, S);
Scr_k3 = zeros(1, S, K);

MSE = zeros(N, N, S);
Scr_s0 = zeros(N, S);
Scr_s1 = zeros(K, S);
Scr_s2 = zeros(K-1, S);
Scr_s3 = zeros(1, S, K);

Stat_k0 = zeros(1, S);
Stat_k1 = zeros(1, S);
Stat_k2 = zeros(1, S);
Stat_k3 = zeros(1, S, K);
Stat_k4 = zeros(1, S);

Stat_s0 = zeros(1, S);
Stat_s1 = zeros(1, S);
Stat_s2 = zeros(1, S);
Stat_s3 = zeros(1, S, K);
Stat_s4 = zeros(1, S);

Stat_GH0 = zeros(1, S);
Stat_GH1 = zeros(1, S);
Stat_GH2 = zeros(1, S);
Stat_GH3 = zeros(1, S, K);
Stat_GH4 = zeros(1, S);

StatKT_k0 = zeros(1, S);
StatKT_k1 = zeros(1, S);
StatKT_k2 = zeros(1, S);
StatKT_k3 = zeros(1, S, K);
StatKT_k4 = zeros(1, S);

StatKT_GH0 = zeros(1, S);
StatKT_GH1 = zeros(1, S);
StatKT_GH2 = zeros(1, S);
StatKT_GH3 = zeros(1, S, K);
StatKT_GH4 = zeros(1, S);

Scr_k0_ML = zeros(1, S);
Scr_k1_ML = zeros(1, S);
Scr_k2_ML = zeros(1, S);
Scr_k3_ML = zeros(1, S, K);

MSE_ML = zeros(N, N, S);
Scr_s0_ML = zeros(N, S);
Scr_s1_ML = zeros(K, S);
Scr_s2_ML = zeros(K-1, S);
Scr_s3_ML = zeros(1, S, K);

Var_k1_ML = zeros(1, S);
Var_k2_ML = zeros(1, S);
Var_k3_ML = zeros(1, S, K);
Var_k4_ML = zeros(K, K, S);

Var_s1_ML = zeros(K, K, S);
Var_s2_ML = zeros(K-1, K-1, S);
Var_s3_ML = zeros(1, 1, S, K);
Var_s4_ML = zeros(K, K, S);

Stat_k0_ML = zeros(1, S);
Stat_k1_ML = zeros(1, S);
Stat_k2_ML = zeros(1, S);
Stat_k3_ML = zeros(1, S, K);
Stat_k4_ML = zeros(1, S);

Stat_s0_ML = zeros(1, S);
Stat_s1_ML = zeros(1, S);
Stat_s2_ML = zeros(1, S);
Stat_s3_ML = zeros(1, S, K);
Stat_s4_ML = zeros(1, S);

Stat_GH0_ML = zeros(1, S);
Stat_GH1_ML = zeros(1, S);
Stat_GH2_ML = zeros(1, S);
Stat_GH3_ML = zeros(1, S, K);
Stat_GH4_ML = zeros(1, S);

StatKT_k0_ML = zeros(1, S);
StatKT_k1_ML = zeros(1, S);
StatKT_k2_ML = zeros(1, S);
StatKT_k3_ML = zeros(1, S, K);
StatKT_k4_ML = zeros(1, S);

StatKT_GH0_ML = zeros(1, S);
StatKT_GH1_ML = zeros(1, S);
StatKT_GH2_ML = zeros(1, S);
StatKT_GH3_ML = zeros(1, S, K);
StatKT_GH4_ML = zeros(1, S);

HK_k3 = zeros(1, S, K);
HK_s3 = zeros(1, S, K);

Stat_HKk3 = zeros(1, S, K);
Stat_HKs3 = zeros(1, S, K);
Stat_HKGH3 = zeros(1, S, K);

HK_k3_ML = zeros(1, S, K);
HK_s3_ML = zeros(1, S, K);

Var_HKk3_ML = zeros(1, S, K);
Var_HKs3_ML = zeros(1, S, K);

Stat_HKk3_ML = zeros(1, S, K);
Stat_HKs3_ML = zeros(1, S, K);
Stat_HKGH3_ML = zeros(1, S, K);

HAC_k0 = zeros(1, S);
HAC_k1 = zeros(1, S);
HAC_k2 = zeros(1, S);
HAC_k3 = zeros(1, S, K);
HAC_k4 = zeros(K, K, S);

HAC_s0 = zeros(N, N, S);
HAC_s1 = zeros(K, K, S);
HAC_s2 = zeros(K-1, K-1, S);
HAC_s3 = zeros(1, 1, S, K);
HAC_s4 = zeros(K, K, S);

HAC_k0_ML = zeros(1, S);
HAC_k1_ML = zeros(1, S);
HAC_k2_ML = zeros(1, S);
HAC_k3_ML = zeros(1, S, K);
HAC_k4_ML = zeros(K, K, S);

HAC_s0_ML = zeros(N, N, S);
HAC_s1_ML = zeros(K, K, S);
HAC_s2_ML = zeros(K-1, K-1, S);
HAC_s3_ML = zeros(1, 1, S, K);
HAC_s4_ML = zeros(K, K, S);

HACStat_k0 = zeros(1, S);
HACStat_k1 = zeros(1, S);
HACStat_k2 = zeros(1, S);
HACStat_k3 = zeros(1, S, K);
HACStat_k4 = zeros(1, S);

HACStat_s0 = zeros(1, S);
HACStat_s1 = zeros(1, S);
HACStat_s2 = zeros(1, S);
HACStat_s3 = zeros(1, S, K);
HACStat_s4 = zeros(1, S);

HACStat_GH0 = zeros(1, S);
HACStat_GH1 = zeros(1, S);
HACStat_GH2 = zeros(1, S);
HACStat_GH3 = zeros(1, S, K);
HACStat_GH4 = zeros(1, S);

HACStatKT_k0 = zeros(1, S);
HACStatKT_k1 = zeros(1, S);
HACStatKT_k2 = zeros(1, S);
HACStatKT_k3 = zeros(1, S, K);
HACStatKT_k4 = zeros(1, S);

HACStatKT_GH0 = zeros(1, S);
HACStatKT_GH1 = zeros(1, S);
HACStatKT_GH2 = zeros(1, S);
HACStatKT_GH3 = zeros(1, S, K);
HACStatKT_GH4 = zeros(1, S);

HACStat_k0_ML = zeros(1, S);
HACStat_k1_ML = zeros(1, S);
HACStat_k2_ML = zeros(1, S);
HACStat_k3_ML = zeros(1, S, K);
HACStat_k4_ML = zeros(1, S);

HACStat_s0_ML = zeros(1, S);
HACStat_s1_ML = zeros(1, S);
HACStat_s2_ML = zeros(1, S);
HACStat_s3_ML = zeros(1, S, K);
HACStat_s4_ML = zeros(1, S);

HACStat_GH0_ML = zeros(1, S);
HACStat_GH1_ML = zeros(1, S);
HACStat_GH2_ML = zeros(1, S);
HACStat_GH3_ML = zeros(1, S, K);
HACStat_GH4_ML = zeros(1, S);

HACStatKT_k0_ML = zeros(1, S);
HACStatKT_k1_ML = zeros(1, S);
HACStatKT_k2_ML = zeros(1, S);
HACStatKT_k3_ML = zeros(1, S, K);
HACStatKT_k4_ML = zeros(1, S);

HACStatKT_GH0_ML = zeros(1, S);
HACStatKT_GH1_ML = zeros(1, S);
HACStatKT_GH2_ML = zeros(1, S);
HACStatKT_GH3_ML = zeros(1, S, K);
HACStatKT_GH4_ML = zeros(1, S);

rejection = zeros(K+4, 3, 3);
rejection_ML = zeros(K+4, 3, 3);
rejectionKT = zeros(K+4, 3, 2);
rejectionKT_ML = zeros(K+4, 3, 2);

HACrejection = zeros(K+4, 3, 3);
HACrejection_ML = zeros(K+4, 3, 3);
HACrejectionKT = zeros(K+4, 3, 2);
HACrejectionKT_ML = zeros(K+4, 3, 2);

rejectionHK = zeros(K, 3, 3);
rejectionHK_ML = zeros(K, 3, 3);


%% SIMULATION AND ESTIMATION

h = waitbar(0, 'Simulation and estimation');

for s = 1:S
    
    display(s, 'Replica')
    
    kappa = 1;
    C = zeros(K, 1);
    F = [[1, zeros(1, N)]; ...
        zeros(N, K)];
    G = [[sqrt(parameters(1)), zeros(1, N)]; ...
        [zeros(N, 1), sqrt(parameters(2))*eye(N)]];
    H = [ones(N, 1), eye(N)];
    
    EX1 = zeros(K, 1);
    VX1 = [[kappa, zeros(1, N)]; ...
        [zeros(N, 1), diag(parameters(2))*eye(N)]];
    
    X = C*ones(1, T);
    Y = zeros(N, T);
    
    X1 = mvnrnd(EX1, VX1)';
    
    for t = 1:BP
        X1 = C + F*X1 + G*randn(K, 1);
    end
    
    X(:, 1) = X1;
    Y(:, 1) = H*X1;
    
    if (hypothesis == 0)
        for t = 2:T
            X(:, t) = C + F*X(:, t-1) + G*randn(K, 1);
            Y(:, t) = H*X(:, t);
        end
    elseif (hypothesis == 1)
        for t = 2:T
            X(:, t) = C + F*X(:, t-1) + ...
                G*(sqrt((nu-2)/nu))*mvtrnd(eye(K), nu)';
            Y(:, t) = H*X(:, t);
        end
    elseif (hypothesis == 2)
        for t = 2:T
            X(:, t) = C + F*X(:, t-1) + ...
                G*[randn(1, 1); sqrt((nu-2)/nu)*mvtrnd(eye(N), nu)'];
            Y(:, t) = H*X(:, t);
        end
    elseif (hypothesis == 3)
        for t = 2:T
            X(:, t) = C + F*X(:, t-1) + ...
                G*[sqrt((nu-2)/nu)*mvtrnd(1, nu); randn(N, 1)];
            Y(:, t) = H*X(:, t);
        end
    elseif (hypothesis == 4)
        for t = 2:T
            X(:, t) = C + F*X(:,t-1) + G*sqrt((nu-2)/nu)*trnd(nu, K, 1);
            Y(:, t) = H*X(:, t);
        end
    elseif (hypothesis == 5)
        e = Asym_t_rnd([nu,beta], T-1, K, 3);
        for t = 2:T
            X(:, t) = C + F*X(:, t-1) + G*e(t-1, :)';
            Y(:, t) = H*X(:, t);
        end
    elseif (hypothesis == 6)
        e = Asym_t_rnd([nu, beta(2:end)], T-1, N, 3);
        for t = 2:T
            X(:, t) = C + F*X(:, t-1) + G*[randn(1, 1); e(t-1, :)'];
            Y(:, t) = H*X(:, t);
        end
    elseif (hypothesis == 7)
        e = Asym_t_rnd([nu, beta(1)], T-1, 1, 3);
        for t = 2:T
            X(:, t) = C + F*X(:, t-1) + G*[e(t-1, :); randn(N, 1)];
            Y(:,t) = H*X(:, t);
        end
    elseif (hypothesis == 8)
        e = zeros(T-1, K);
        for k = 1:K
            e(:, k) = Asym_t_rnd([nu, beta(k)], T-1, 1, 3);
        end
        for t = 2:T
            X(:,t) = C + F*X(:, t-1) + G*e(t-1, :)';
            Y(:,t) = H*X(:, t);
        end
    else
        disp('WARNING: hypothesis not valid.')
        close(h)
        return
    end
    
    Data(:, :, s) = Y;
    
    lower = [ 0.0001  0.0001]';
    upper = [10.0000 10.0000]';
    
    MLestimate(:, s) = fmincon('Multivariate_Likelihood', ...
        parameters, [], [], [], [], lower, upper, [], options, Y);
    
    waitbar(s/S, h)
    
    % Saving results
    if (rem(s, safe) == 0 || s == S)
        save estimate.mat s ...
            Data MLestimate
    end
    
end
close(h)

movefile('estimate.mat', ['IO/estimate/', title, '_estimate_T', num2str(T), ...
    '_H', num2str(hypothesis), '.mat']);


%% COMPUTATION OF SCORE

h = waitbar(0, 'Computation of score');
load(['IO/estimate/', title, '_estimate_T', num2str(T), ...
    '_H', num2str(hypothesis), '.mat'])

for s = 1:S
    
    display(s, 'Replica')
    
    Y = Data(:, :, s);
    
    kappa = 1e3;
    F = [[1, zeros(1, K)]; zeros(K, K+1)];
    G = [[sqrt(parameters(1)), zeros(1, N)]; ...
        [zeros(N, 1), sqrt(parameters(2))*eye(N)]; ...
        [sqrt(parameters(2)), zeros(1, N)]];
    H = [ones(N, 1), eye(N), zeros(N, 1)];
    
    EX1 = zeros(K+1, 1);
    VX1 = [[kappa*parameters(1), zeros(1, K)]; ...
        [zeros(N, 1), diag(parameters(2))*eye(N), zeros(N, 1)]; ...
        [zeros(1, K), parameters(2)]];
    
    G_ML = [[sqrt(MLestimate(1, s)), zeros(1, N)]; ...
        [zeros(N, 1), sqrt(MLestimate(2, s))*eye(N)]; ...
        [sqrt(MLestimate(2, s)), zeros(1, N)]];
    
    VX1_ML = [[kappa*MLestimate(1, s), zeros(1, K)]; ...
        [zeros(N, 1), diag(MLestimate(2, s))*eye(N), zeros(N, 1)]; ...
        [zeros(1, K), MLestimate(2, s)]];
    
    order = [K+1 2:K];
    
    [Xf, Pf, Xs, Ps] = KalmanFS(EX1, VX1, F, G, H, Y);
    
    Xs = Xs(order, :);
    Ps = Ps(order, order, :);
    
    e0 = zeros(N, T-2);
    P0 = zeros(N, N, T-2);
    for t = 2:T-1
        e0(:, t-1) = Y(:, t) - H*Xf(:, t);
        P0(:, :, t-1) = H*Pf(:, :, t)*H';
    end
    e1 = Xs(1:K, 2:T-1)/parameters(2);
    P1 = Ps(1:K, 1:K, 2:T-1)/parameters(2);
    e2 = Xs(2:K, 2:T-1)/sqrt(parameters(2));
    P2 = Ps(2:K, 2:K, 2:T-1)/parameters(2);
    e3 = zeros(1, T-2, K);
    P3 = zeros(1, 1, T-2, K);
    for k = 1:K
        e3(:, :, k) = Xs(k, 2:T-1)/sqrt(parameters(2));
        P3(:, :, :, k) = Ps(k, k, 2:T-1)/parameters(2);
    end
    
    [Xf_ML, Pf_ML, Xs_ML, Ps_ML] = KalmanFS(EX1, VX1_ML, F, G_ML, H, Y);
    
    Xs_ML = Xs_ML(order, :);
    Ps_ML = Ps_ML(order, order, :);
    
    e0_ML = zeros(N, T-2);
    P0_ML = zeros(N, N, T-2);
    for t = 2:T-1
        e0_ML(:, t-1) = Y(:, t) - H*Xf_ML(:, t);
        P0_ML(:, :, t-1) = H*Pf_ML(:, :, t)*H';
    end
    e1_ML = Xs_ML(1:K, 2:T-1)/sqrt(MLestimate(2, s));
    P1_ML = Ps_ML(1:K, 1:K, 2:T-1)/MLestimate(2, s);
    e2_ML = Xs_ML(2:K, 2:T-1)/sqrt(MLestimate(2, s));
    P2_ML = Ps_ML(2:K, 2:K,2:T-1)/MLestimate(2, s);
    e3_ML = zeros(1, T-2, K);
    P3_ML = zeros(1, 1, T-2, K);
    for k = 1:K
        e3_ML(:, :, k) = Xs_ML(k, 2:T-1)/sqrt(MLestimate(2, s));
        P3_ML(:, :, :, k) = Ps_ML(k, k, 2:T-1)/MLestimate(2, s);
    end
    
    % Computation of score
    sk0 = ScoreFK(e0, P0);
    sk1 = ScoreSK(e1, P1);
    sk2 = ScoreSK(e2, P2);
    sk3 = zeros(1, T-2, K);
    sk4 = zeros(K, T-2);
    for k = 1:K
        sk3(:, :, k) = ScoreSK(e3(:, :, k), P3(:, :, :, k));
        sk4(k, :) = sk3(:, :, k);
    end
    
    ss0 = ScoreFS(e0, P0);
    ss1 = ScoreSS(e1, P1);
    ss2 = ScoreSS(e2, P2);
    ss3 = zeros(1, T-2, K);
    ss4 = zeros(K, T-2);
    for k = 1:K
        ss3(:, :, k) = ScoreSS(e3(:, :, k), P3(:, :, :, k));
        ss4(k, :) = ss3(:, :, k);
    end
    
    Scr_k0(:, s) = mean(sk0, 2);
    Scr_k1(:, s) = mean(sk1, 2);
    Scr_k2(:, s) = mean(sk2, 2);
    for k = 1:K
        Scr_k3(:, s, k) = mean(sk3(:, :, k), 2);
    end
    
    MSE(:, :, s) = mean(P0, 3);
    
    Scr_s0(:, s) = mean(ss0, 2);
    Scr_s1(:, s) = mean(ss1, 2);
    Scr_s2(:, s) = mean(ss2, 2);
    for k = 1:K
        Scr_s3(:, s, k) = mean(ss3(:, :, k), 2);
    end
    
    sk0_ML = ScoreFK(e0_ML, P0_ML);
    sk1_ML = ScoreSK(e1_ML, P1_ML);
    sk2_ML = ScoreSK(e2_ML, P2_ML);
    sk3_ML = zeros(1, T-2, K);
    sk4_ML = zeros(K, T-2);
    for k = 1:K
        sk3_ML(:, :, k) = ScoreSK(e3_ML(:, :, k), P3_ML(:, :, :, k));
        sk4_ML(k, :) = sk3_ML(:, :, k);
    end
    
    ss0_ML = ScoreFS(e0_ML, P0_ML);
    ss1_ML = ScoreSS(e1_ML, P1_ML);
    ss2_ML = ScoreSS(e2_ML, P2_ML);
    ss3_ML = zeros(1, T-2, K);
    ss4_ML = zeros(K, T-2);
    for k = 1:K
        ss3_ML(:, :, k) = ScoreSS(e3_ML(:, :, k), P3_ML(:, :, :, k));
        ss4_ML(k, :) = ss3_ML(:, :, k);
    end
    
    Scr_k0_ML(:, s) = mean(sk0_ML, 2);
    Scr_k1_ML(:, s) = mean(sk1_ML, 2);
    Scr_k2_ML(:, s) = mean(sk2_ML, 2);
    for k = 1:K
        Scr_k3_ML(:, s, k) = mean(sk3_ML(:, :, k), 2);
    end
    
    MSE_ML(:, :, s) = mean(P0_ML, 3);
    
    Scr_s0_ML(:, s) = mean(ss0_ML, 2);
    Scr_s1_ML(:, s) = mean(ss1_ML, 2);
    Scr_s2_ML(:, s) = mean(ss2_ML, 2);
    for k = 1:K
        Scr_s3_ML(:, s, k) = mean(ss3_ML(:, :, k), 2);
    end
    
    for k = 1:K
        HK_k3(:, s, k) = StatHKK(e3(:, :, k));
        HK_s3(:, s, k) = StatHKS(e3(:, :, k));
        HK_k3_ML(:, s, k) = StatHKK(e3_ML(:, :, k));
        HK_s3_ML(:, s, k) = StatHKS(e3_ML(:, :, k));
    end
    
    HAC_k0(:, s) = AutoCovariance(sk0', 0);
    HAC_k1(:, s) = AutoCovariance(sk1', 0);
    HAC_k2(:, s) = AutoCovariance(sk2', 0);
    for k = 1:K
        HAC_k3(:, s, k) = AutoCovariance(sk3(:, :, k)', 0);
    end
    HAC_k4(:, :, s) = AutoCovariance(sk4', 0);
    
    HAC_s0(:, :, s) = AutoCovariance(ss0', 0);
    HAC_s1(:, :, s) = AutoCovariance(ss1', 0);
    HAC_s2(:, :, s) = AutoCovariance(ss2', 0);
    for k = 1:K
        HAC_s3(:, :, s, k) = AutoCovariance(ss3(:, :, k)', 0);
    end
    HAC_s4(:, :, s) = AutoCovariance(ss4', 0);
    
    HAC_k0_ML(:, s) = AutoCovariance(sk0_ML', 0);
    HAC_k1_ML(:, s) = AutoCovariance(sk1_ML', 0);
    HAC_k2_ML(:, s) = AutoCovariance(sk2_ML', 0);
    for k = 1:K
        HAC_k3_ML(:, s, k) = AutoCovariance(sk3_ML(:, :, k)', 0);
    end
    HAC_k4_ML(:, :, s) = AutoCovariance(sk4_ML', 0);
    
    HAC_s0_ML(:, :, s) = AutoCovariance(ss0_ML', 0);
    HAC_s1_ML(:, :, s) = AutoCovariance(ss1_ML', 0);
    HAC_s2_ML(:, :, s) = AutoCovariance(ss2_ML', 0);
    for k = 1:K
        HAC_s3_ML(:, :, s, k) = AutoCovariance(ss3_ML(:, :, k)', 0);
    end
    HAC_s4_ML(:, :, s) = AutoCovariance(ss4_ML', 0);
    
    for l = 1:NW_lags
        weight = 1 - l/(NW_lags+1);
        HAC_k0(:, s) = HAC_k0(:, s) + 2*weight*AutoCovariance(sk0', l);
        HAC_k1(:, s) = HAC_k1(:, s) + 2*weight*AutoCovariance(sk1', l);
        HAC_k2(:, s) = HAC_k2(:, s) + 2*weight*AutoCovariance(sk2', l);
        for k = 1:K
            HAC_k3(:, s, k) = HAC_k3(:, s, k) + 2*weight*AutoCovariance(sk3(:, :, k)', l);
        end
        HAC_k4(:, :, s) = HAC_k4(:, :, s) + weight*AutoCovariance(sk4', l) + weight*AutoCovariance(sk4', l)';
        
        HAC_s0(:, :, s) = HAC_s0(:, :, s) + weight*AutoCovariance(ss0', l) + weight*AutoCovariance(ss0', l)';
        HAC_s1(:, :, s) = HAC_s1(:, :, s) + weight*AutoCovariance(ss1', l) + weight*AutoCovariance(ss1', l)';
        HAC_s2(:, :, s) = HAC_s2(:, :, s) + weight*AutoCovariance(ss2', l) + weight*AutoCovariance(ss2', l)';
        for k = 1:K
            HAC_s3(:, :, s, k) = HAC_s3(:, :, s, k) + 2*weight*AutoCovariance(ss3(:, :, k)', l);
        end
        HAC_s4(:, :, s) = HAC_s4(:, :, s) + weight*AutoCovariance(ss4', l) + weight*AutoCovariance(ss4', l)';
        
        HAC_k0_ML(:, s) = HAC_k0_ML(:, s) + 2*weight*AutoCovariance(sk0_ML', l);
        HAC_k1_ML(:, s) = HAC_k1_ML(:, s) + 2*weight*AutoCovariance(sk1_ML', l);
        HAC_k2_ML(:, s) = HAC_k2_ML(:, s) + 2*weight*AutoCovariance(sk2_ML', l);
        for k = 1:K
            HAC_k3_ML(:, s, k) = HAC_k3_ML(:, s, k) + 2*weight*AutoCovariance(sk3_ML(:, :, k)', l);
        end
        HAC_k4_ML(:, :, s) = HAC_k4_ML(:, :, s) + weight*AutoCovariance(sk4_ML', l) + weight*AutoCovariance(sk4_ML', l)';
        
        HAC_s0_ML(:, :, s) = HAC_s0_ML(:, :, s) + weight*AutoCovariance(ss0_ML', l) + weight*AutoCovariance(ss0_ML', l)';
        HAC_s1_ML(:, :, s) = HAC_s1_ML(:, :, s) + weight*AutoCovariance(ss1_ML', l) + weight*AutoCovariance(ss1_ML', l)';
        HAC_s2_ML(:, :, s) = HAC_s2_ML(:, :, s) + weight*AutoCovariance(ss2_ML', l) + weight*AutoCovariance(ss2_ML', l)';
        for k = 1:K
            HAC_s3_ML(:, :, s, k) = HAC_s3_ML(:, :, s, k) + 2*weight*AutoCovariance(ss3_ML(:, :, k)', l);
        end
        HAC_s4_ML(:, :, s) = HAC_s4_ML(:, :, s) + weight*AutoCovariance(ss4_ML', l) + weight*AutoCovariance(ss4_ML', l)';
    end
    
    waitbar(s/S, h)
    
    % Saving results
    if (rem(s, safe) == 0 || s == S)
        save score.mat s ...
            Scr_k0 Scr_k1 Scr_k2 Scr_k3 ...
            MSE Scr_s0 Scr_s1 Scr_s2 Scr_s3 ...
            Scr_k0_ML Scr_k1_ML Scr_k2_ML Scr_k3_ML ...
            MSE_ML Scr_s0_ML Scr_s1_ML Scr_s2_ML Scr_s3_ML ...
            HK_k3 HK_s3 ...
            HK_k3_ML HK_s3_ML ...
            HAC_k0 HAC_k1 HAC_k2 HAC_k3 HAC_k4 ...
            HAC_s0 HAC_s1 HAC_s2 HAC_s3 HAC_s4 ...
            HAC_k0_ML HAC_k1_ML HAC_k2_ML HAC_k3_ML HAC_k4_ML ...
            HAC_s0_ML HAC_s1_ML HAC_s2_ML HAC_s3_ML HAC_s4_ML
    end
    
end
close(h)

movefile('score.mat', ['IO/score/', title, '_score_T', num2str(T),...
    '_H', num2str(hypothesis), '.mat']);


%% COMPUTATION OF ASYMPTOTIC VARIANCE

h = waitbar(0, 'Computation of asymptotic variance');
load(['IO/estimate/', title, '_estimate_T', num2str(T), '_H', num2str(hypothesis), '.mat'])

[PHI, THETA] = Multivariate_WKfilter(parameters(1), parameters(2));
THETA = THETA(1:(end-1), :);
auxGAMMA0 = AutocovarianceWK(PHI, THETA, eye(4), 0);
auxGAMMA0 = [[auxGAMMA0(1, 1), auxGAMMA0(1, 2)*ones(1, N)]; ...
    [auxGAMMA0(2, 1)*ones(N, 1), ...
    auxGAMMA0(2, 2)*eye(N) + auxGAMMA0(2, 3)*(ones(N, N) - eye(N))]];

r = K;
[~, aux_bs, ~] = ScoreSS(zeros(K, 1), eye(K) - auxGAMMA0);
[~, aux_bk, ~] = ScoreSK(zeros(K, 1), eye(K) - auxGAMMA0);

B_e1_1 = aux_bs(1:r, :);
B_e2_1 = aux_bk(2:1+r^2);
B_e3_1 = aux_bs(r+1:end, :);
B_e4_1 = aux_bk(2+r^2:end);

r = K-1;
E2 = [zeros(K-1, 1), eye(K-1)];
[~, aux_bs, ~] = ScoreSS(zeros(K-1, 1), eye(K-1) - auxGAMMA0(2:K, 2:K));
[~, aux_bk, ~] = ScoreSK(zeros(K-1, 1), eye(K-1) - auxGAMMA0(2:K, 2:K));

B_e1_2 = E2'*aux_bs(1:r, :);
B_e2_2 = kron(E2',E2')*aux_bk(2:1+r^2);
B_e3_2 = kron(E2',E2')*aux_bs(r+1:end, :);
B_e4_2 = kron(E2',E2')*aux_bk(2+r^2:end);

r = 1;
E3 = zeros(1, K, K);
B_e1_3 = zeros(K, 1, K);
B_e2_3 = zeros(K^2, 1, K);
B_e3_3 = zeros(K^2, 1, K);
B_e4_3 = zeros(K^2, 1, K);
for k = 1:K
    E3(1, k, k) = 1;
    [~, aux_bs, ~] = ScoreSS(0, 1 - auxGAMMA0(k, k));
    [~, aux_bk, ~] = ScoreSK(0, 1 - auxGAMMA0(k, k));
    
    B_e1_3(:, :, k) = E3(:, :, k)'*aux_bs(1:r, :);
    B_e2_3(:, :, k) = kron(E3(:, :, k)', E3(:, :, k)')*aux_bk(2:1+r^2);
    B_e3_3(:, :, k) = kron(E3(:, :, k)', E3(:, :, k)')*aux_bs(r+1:end, :);
    B_e4_3(:, :, k) = kron(E3(:, :, k)', E3(:, :, k)')*aux_bk(2+r^2:end);
end

B_e1_4 = reshape(B_e1_3, [K, K]);
B_e2_4 = reshape(B_e2_3, [K^2, K]);
B_e3_4 = reshape(B_e3_3, [K^2, K]);
B_e4_4 = reshape(B_e4_3, [K^2, K]);

[CMe1e1, CMe2e2, CMe3e3, CMe4e4] = AutocovarianceMM(auxGAMMA0, auxGAMMA0);

AGFe1e1 = CMe1e1;
AGFe2e2 = CMe2e2;
AGFe3e3 = CMe3e3;
AGFe4e4 = CMe4e4;

kappa_k3 = zeros(1, K);
kappa_s3 = zeros(1, K);
for k = 1:K
    kappa_k3(:, k) = auxGAMMA0(k, k)^4;
    kappa_s3(:, k) = auxGAMMA0(k, k)^3;
end

j = 0;
error = 1;

while ((j < maxiter) && (error >= maxerror))
    
    auxGAMMAj = AutocovarianceWK(PHI, THETA, eye(4), j+1);
    auxGAMMAj = [[auxGAMMAj(1, 1), auxGAMMAj(1, 2)*ones(1, N)]; ...
        [auxGAMMAj(2, 1)*ones(N, 1), ...
        auxGAMMAj(2, 2)*eye(N) + auxGAMMAj(2, 3)*(ones(N, N) - eye(N))]];
    [CMe1e1, CMe2e2, CMe3e3, CMe4e4] = AutocovarianceMM(auxGAMMA0, auxGAMMAj);
    [CMe1e1m, CMe2e2m, CMe3e3m, CMe4e4m] = AutocovarianceMM(auxGAMMA0, auxGAMMAj');
    
    AGFe1e1 = AGFe1e1 + CMe1e1 + CMe1e1m;
    AGFe2e2 = AGFe2e2 + CMe2e2 + CMe2e2m;
    AGFe3e3 = AGFe3e3 + CMe3e3 + CMe3e3m;
    AGFe4e4 = AGFe4e4 + CMe4e4 + CMe4e4m;
    
    kappa_k3(:, k) = kappa_k3(:, k) + 2*auxGAMMAj(k, k)^4;
    kappa_s3(:, k) = kappa_s3(:, k) + 2*auxGAMMAj(k, k)^3;
    
    j = j+1;
    
    de1e1 = max(max(abs(CMe1e1 + CMe1e1m)));
    de2e2 = max(max(abs(CMe2e2 + CMe2e2m)));
    de3e3 = max(max(abs(CMe3e3 + CMe3e3m)));
    de4e4 = max(max(abs(CMe4e4 + CMe4e4m)));
    error = max([de1e1, de2e2, de3e3, de4e4]);
    
    display([j, error], 'Iteration')
end

Var_k1 = (B_e4_1')*AGFe4e4*(B_e4_1) - (B_e2_1')*AGFe2e2*(B_e2_1);
Var_k2 = (B_e4_2')*AGFe4e4*(B_e4_2) - (B_e2_2')*AGFe2e2*(B_e2_2);
Var_k3 = zeros(1, K);
for k = 1:K
    Var_k3(:, k) = (B_e4_3(:, :, k)')*AGFe4e4*(B_e4_3(:, :, k)) - ...
        (B_e2_3(:, :, k)')*AGFe2e2*(B_e2_3(:, :, k));
end
Var_k4 = (B_e4_4')*AGFe4e4*(B_e4_4) - (B_e2_4')*AGFe2e2*(B_e2_4);

Var_s1 = (B_e3_1')*AGFe3e3*(B_e3_1) - (B_e1_1')*AGFe1e1*(B_e1_1);
Var_s2 = (B_e3_2')*AGFe3e3*(B_e3_2) - (B_e1_2')*AGFe1e1*(B_e1_2);
Var_s3 = zeros(1, 1, K);
for k = 1:K
    Var_s3(:, :, k) = (B_e3_3(:, :, k)')*AGFe3e3*(B_e3_3(:, :, k)) - ...
        (B_e1_3(:, :, k)')*AGFe1e1*(B_e1_3(:, :, k));
end
Var_s4 = (B_e3_4')*AGFe3e3*(B_e3_4) - (B_e1_4')*AGFe1e1*(B_e1_4);

Var_HKk3 = zeros(1, K);
Var_HKs3 = zeros(1, K);
for k = 1:K
    Var_HKk3(:, k) = kappa_k3(:, k)/auxGAMMA0(k,k)^4;
    Var_HKs3(:, k) = kappa_s3(:, k)/auxGAMMA0(k,k)^3;
end

% Computation of asymptotic variance
for s = 1:S
    
    display(s,'Replica')
    t1 = cputime;
    
    [PHI, THETA] = Multivariate_WKfilter(MLestimate(1, s), MLestimate(2, s));
    THETA = THETA(1:(end-1), :);
    auxGAMMA0 = AutocovarianceWK(PHI, THETA, eye(4), 0);
    auxGAMMA0 = [[auxGAMMA0(1, 1), auxGAMMA0(1, 2)*ones(1, N)]; ...
        [auxGAMMA0(2, 1)*ones(N, 1), ...
        auxGAMMA0(2, 2)*eye(N) + auxGAMMA0(2, 3)*(ones(N, N) - eye(N))]];
    
    r = K;
    [~, aux_bs, ~] = ScoreSS(zeros(K, 1), eye(K) - auxGAMMA0);
    [~, aux_bk, ~] = ScoreSK(zeros(K, 1), eye(K) - auxGAMMA0);
    
    B_e1_1 = aux_bs(1:r, :);
    B_e2_1 = aux_bk(2:1+r^2);
    B_e3_1 = aux_bs(r+1:end, :);
    B_e4_1 = aux_bk(2+r^2:end);
    
    r = K-1;
    [~, aux_bs, ~] = ScoreSS(zeros(K-1, 1), eye(K-1) - auxGAMMA0(2:K, 2:K));
    [~, aux_bk, ~] = ScoreSK(zeros(K-1, 1), eye(K-1) - auxGAMMA0(2:K, 2:K));
    
    B_e1_2 = E2'*aux_bs(1:r, :);
    B_e2_2 = kron(E2',E2')*aux_bk(2:1+r^2);
    B_e3_2 = kron(E2',E2')*aux_bs(r+1:end, :);
    B_e4_2 = kron(E2',E2')*aux_bk(2+r^2:end);
    
    r = 1;
    B_e1_3 = zeros(K, 1, K);
    B_e2_3 = zeros(K^2, 1, K);
    B_e3_3 = zeros(K^2, 1, K);
    B_e4_3 = zeros(K^2, 1, K);
    for k = 1:K
        [~, aux_bs, ~] = ScoreSS(0, 1 - auxGAMMA0(k, k));
        [~, aux_bk, ~] = ScoreSK(0, 1 - auxGAMMA0(k, k));
        
        B_e1_3(:, :, k) = E3(:, :, k)'*aux_bs(1:r, :);
        B_e2_3(:, :, k) = kron(E3(:, :, k)', E3(:, :, k)')*aux_bk(2:1+r^2);
        B_e3_3(:, :, k) = kron(E3(:, :, k)', E3(:, :, k)')*aux_bs(r+1:end, :);
        B_e4_3(:, :, k) = kron(E3(:, :, k)', E3(:, :, k)')*aux_bk(2+r^2:end);
    end
    
    B_e1_4 = reshape(B_e1_3, [K, K]);
    B_e2_4 = reshape(B_e2_3, [K^2, K]);
    B_e3_4 = reshape(B_e3_3, [K^2, K]);
    B_e4_4 = reshape(B_e4_3, [K^2, K]);
    
    [CMe1e1, CMe2e2, CMe3e3, CMe4e4] = AutocovarianceMM(auxGAMMA0, auxGAMMA0);
    
    AGFe1e1 = CMe1e1;
    AGFe2e2 = CMe2e2;
    AGFe3e3 = CMe3e3;
    AGFe4e4 = CMe4e4;
    
    kappa_k3 = zeros(1, K);
    kappa_s3 = zeros(1, K);
    for k = 1:K
        kappa_k3(:, k) = auxGAMMA0(k, k)^4;
        kappa_s3(:, k) = auxGAMMA0(k, k)^3;
    end
    
    j = 0;
    error = 1;
    
    while ((j < maxiter) && (error >= maxerror))
        
        auxGAMMAj = AutocovarianceWK(PHI, THETA, eye(4), j+1);
        auxGAMMAj = [[auxGAMMAj(1, 1), auxGAMMAj(1, 2)*ones(1, N)]; ...
            [auxGAMMAj(2, 1)*ones(N, 1), ...
            auxGAMMAj(2, 2)*eye(N) + auxGAMMAj(2, 3)*(ones(N, N) - eye(N))]];
        [CMe1e1, CMe2e2, CMe3e3, CMe4e4] = AutocovarianceMM(auxGAMMA0, auxGAMMAj);
        [CMe1e1m, CMe2e2m, CMe3e3m, CMe4e4m] = AutocovarianceMM(auxGAMMA0, auxGAMMAj');
        
        AGFe1e1 = AGFe1e1 + CMe1e1 + CMe1e1m;
        AGFe2e2 = AGFe2e2 + CMe2e2 + CMe2e2m;
        AGFe3e3 = AGFe3e3 + CMe3e3 + CMe3e3m;
        AGFe4e4 = AGFe4e4 + CMe4e4 + CMe4e4m;
        
        kappa_k3(:, k) = kappa_k3(:, k) + 2*auxGAMMAj(k, k)^4;
        kappa_s3(:, k) = kappa_s3(:, k) + 2*auxGAMMAj(k, k)^3;
        
        j = j+1;
        
        de1e1 = max(max(abs(CMe1e1 + CMe1e1m)));
        de2e2 = max(max(abs(CMe2e2 + CMe2e2m)));
        de3e3 = max(max(abs(CMe3e3 + CMe3e3m)));
        de4e4 = max(max(abs(CMe4e4 + CMe4e4m)));
        error = max([de1e1, de2e2, de3e3, de4e4]);
        
        display([j, error], 'Iteration')
    end
    
    Var_k1_ML(:, s) = (B_e4_1')*AGFe4e4*(B_e4_1) - (B_e2_1')*AGFe2e2*(B_e2_1);
    Var_k2_ML(:, s) = (B_e4_2')*AGFe4e4*(B_e4_2) - (B_e2_2')*AGFe2e2*(B_e2_2);
    for k = 1:K
        Var_k3_ML(:, s, k) = (B_e4_3(:, :, k)')*AGFe4e4*(B_e4_3(:, :, k)) - ...
            (B_e2_3(:, :, k)')*AGFe2e2*(B_e2_3(:, :, k));
    end
    Var_k4_ML(:, :, s) = (B_e4_4')*AGFe4e4*(B_e4_4) - (B_e2_4')*AGFe2e2*(B_e2_4);
    
    Var_s1_ML(:, :, s) = (B_e3_1')*AGFe3e3*(B_e3_1) - (B_e1_1')*AGFe1e1*(B_e1_1);
    Var_s2_ML(:, :, s) = (B_e3_2')*AGFe3e3*(B_e3_2) - (B_e1_2')*AGFe1e1*(B_e1_2);
    for k = 1:K
        Var_s3_ML(:, :, s, k) = (B_e3_3(:, :, k)')*AGFe3e3*(B_e3_3(:, :, k)) - ...
            (B_e1_3(:, :, k)')*AGFe1e1*(B_e1_3(:, :, k));
    end
    Var_s4_ML(:, :, s) = (B_e3_4')*AGFe3e3*(B_e3_4) - (B_e1_4')*AGFe1e1*(B_e1_4);
    
    for k = 1:K
        Var_HKk3_ML(:, s, k) = kappa_k3(:, k)/auxGAMMA0(k,k)^4;
        Var_HKs3_ML(:, s, k) = kappa_s3(:, k)/auxGAMMA0(k,k)^3;
    end
    
    waitbar(s/S, h)
    
    % Saving results
    if (rem(s, safe) == 0 || s == S)
        save stderr.mat s ...
            Var_k1 Var_k2 Var_k3 Var_k4...
            Var_s1 Var_s2 Var_s3 Var_s4 ...
            Var_k1_ML Var_k2_ML Var_k3_ML Var_k4_ML ...
            Var_s1_ML Var_s2_ML Var_s3_ML Var_s4_ML ...
            Var_HKk3 Var_HKs3 ...
            Var_HKk3_ML Var_HKs3_ML
    end
end
close(h)

movefile('stderr.mat',['IO/stderr/', title,'_stderr_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat']);


%% COMPUTATION OF TEST STATISTICS

h = waitbar(0,'Computation of test statistic');
load(['IO/score/', title,'_score_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat'])
load(['IO/stderr/', title,'_stderr_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat'])

for s = 1:S
    
    display(s,'Replica')
    
    % True parameter values
    Stat_k0(:, s) = T*(Scr_k0(:, s).^2)/(N*(N+2)/2);
    Stat_k1(:, s) = T*(Scr_k1(:, s).^2)/(Var_k1);
    Stat_k2(:, s) = T*(Scr_k2(:, s).^2)/(Var_k2);
    Scr_k4 = zeros(K, 1);
    for k = 1:K
        Stat_k3(:, s, k) = T*(Scr_k3(:, s, k).^2)/(Var_k3(:, k));
        Scr_k4(k, 1) = Scr_k3(:, s, k);
    end
    Stat_k4(:, s) = T*Scr_k4'*((Var_k4)\Scr_k4);
    
    Stat_s0(:, s) = T*Scr_s0(:, s)'*pinv(MSE(:, :, s))*Scr_s0(:, s)/(2*(N+2));
    Stat_s1(:, s) = T*Scr_s1(:, s)'*((Var_s1)\Scr_s1(:, s));
    Stat_s2(:, s) = T*Scr_s2(:, s)'*((Var_s2)\Scr_s2(:, s));
    Scr_s4 = zeros(K, 1);
    for k = 1:K
        Stat_s3(:, s, k) = T*Scr_s3(:, s, k)'*((Var_s3(:, :, k))\Scr_s3(:, s, k));
        Scr_s4(k, 1) = Scr_s3(:, s, k);
    end
    Stat_s4(:, s) = T*Scr_s4'*((Var_s4)\Scr_s4);
    
    Stat_GH0(:, s) = Stat_k0(:, s) + Stat_s0(:, s);
    Stat_GH1(:, s) = Stat_k1(:, s) + Stat_s1(:, s);
    Stat_GH2(:, s) = Stat_k2(:, s) + Stat_s2(:, s);
    for k = 1:K
        Stat_GH3(:, s, k) = Stat_k3(:, s, k) + Stat_s3(:, s, k);
    end
    Stat_GH4(:, s) = Stat_k4(:, s) + Stat_s4(:, s);
    
    % True parameter values - HAC
    HACStat_k0(:, s) = T*(Scr_k0(:, s).^2)/HAC_k0(:, s);
    HACStat_k1(:, s) = T*(Scr_k1(:, s).^2)/HAC_k1(:, s);
    HACStat_k2(:, s) = T*(Scr_k2(:, s).^2)/HAC_k2(:, s);
    for k = 1:K
        HACStat_k3(:, s, k) = T*(Scr_k3(:, s, k).^2)/(HAC_k3(:, s, k));
    end
    HACStat_k4(:, s) = T*Scr_k4'*((HAC_k4(:, :, s))\Scr_k4);
    
    HACStat_s0(:, s) = T*Scr_s0(:, s)'*((HAC_s0(:, :, s))\Scr_s0(:, s));
    HACStat_s1(:, s) = T*Scr_s1(:, s)'*((HAC_s1(:, :, s))\Scr_s1(:, s));
    HACStat_s2(:, s) = T*Scr_s2(:, s)'*((HAC_s2(:, :, s))\Scr_s2(:, s));
    for k = 1:K
        HACStat_s3(:, s, k) = T*Scr_s3(:, s, k)'*((HAC_s3(:, :, s, k))\Scr_s3(:, s, k));
    end
    HACStat_s4(:, s) = T*Scr_s4'*((HAC_s4(:, :, s))\Scr_s4);
    
    HACStat_GH0(:, s) = HACStat_k0(:, s) + HACStat_s0(:, s);
    HACStat_GH1(:, s) = HACStat_k1(:, s) + HACStat_s1(:, s);
    HACStat_GH2(:, s) = HACStat_k2(:, s) + HACStat_s2(:, s);
    for k = 1:K
        HACStat_GH3(:, s, k) = HACStat_k3(:, s, k) + HACStat_s3(:, s, k);
    end
    HACStat_GH4(:, s) = HACStat_k4(:, s) + HACStat_s4(:, s);
    
    % True parameter values - KT
    StatKT_k0(:, s) = Stat_k0(:, s) * (Scr_k0(:, s) > 0);
    StatKT_k1(:, s) = Stat_k1(:, s) * (Scr_k1(:, s) > 0);
    StatKT_k2(:, s) = Stat_k2(:, s) * (Scr_k2(:, s) > 0);
    for k = 1:K
        StatKT_k3(:, s, k) = Stat_k3(:, s, k) * (Scr_k3(:, s, k) > 0);
    end
    Scr_k4 = Scr_k4 .* (Scr_k4 > 0);
    StatKT_k4(:, s) = T*Scr_k4'*((Var_k4)\Scr_k4);
    
    StatKT_GH0(:, s) = StatKT_k0(:, s) + Stat_s0(:, s);
    StatKT_GH1(:, s) = StatKT_k1(:, s) + Stat_s1(:, s);
    StatKT_GH2(:, s) = StatKT_k2(:, s) + Stat_s2(:, s);
    for k = 1:K
        StatKT_GH3(:, s, k) = StatKT_k3(:, s, k) + Stat_s3(:, s, k);
    end
    StatKT_GH4(:, s) = StatKT_k4(:, s) + Stat_s4(:, s);
    
    % True parameter values - HAC - KT
    HACStatKT_k0(:, s) = HACStat_k0(:, s) * (Scr_k0(:, s) > 0);
    HACStatKT_k1(:, s) = HACStat_k1(:, s) * (Scr_k1(:, s) > 0);
    HACStatKT_k2(:, s) = HACStat_k2(:, s) * (Scr_k2(:, s) > 0);
    for k = 1:K
        HACStatKT_k3(:, s, k) = HACStat_k3(:, s, k) * (Scr_k3(:, s, k) > 0);
    end
    HACStatKT_k4(:, s) = T*Scr_k4'*((HAC_k4(:, :, s))\Scr_k4);
    
    HACStatKT_GH0(:, s) = HACStatKT_k0(:, s) + HACStat_s0(:, s);
    HACStatKT_GH1(:, s) = HACStatKT_k1(:, s) + HACStat_s1(:, s);
    HACStatKT_GH2(:, s) = HACStatKT_k2(:, s) + HACStat_s2(:, s);
    for k = 1:K
        HACStatKT_GH3(:, s, k) = HACStatKT_k3(:, s, k) + HACStat_s3(:, s, k);
    end
    HACStatKT_GH4(:, s) = HACStatKT_k4(:, s) + HACStat_s4(:, s);
    
    % True parameter values - HK
    for k = 1:K
        Stat_HKk3(:, s, k) = HK_k3(:, s, k)/(Var_HKk3(:, k));
        Stat_HKs3(:, s, k) = HK_s3(:, s, k)/(Var_HKs3(:, k));
        Stat_HKGH3(:, s, k) = Stat_HKk3(:, s, k) + Stat_HKs3(:, s, k);
    end
    
    % ML estimates
    Stat_k0_ML(:, s) = T*(Scr_k0_ML(:, s).^2)/(N*(N+2)/2);
    Stat_k1_ML(:, s) = T*(Scr_k1_ML(:, s).^2)/(Var_k1_ML(:, s));
    Stat_k2_ML(:, s) = T*(Scr_k2_ML(:, s).^2)/(Var_k2_ML(:, s));
    Scr_k4_ML = zeros(K, 1);
    for k = 1:K
        Stat_k3_ML(:, s, k) = T*(Scr_k3_ML(:, s, k).^2)/(Var_k3_ML(:, s, k));
        Scr_k4_ML(k, 1) = Scr_k3_ML(:, s, k);
    end
    Stat_k4_ML(:, s) = T*Scr_k4_ML'*((Var_k4_ML(:, :, s))\Scr_k4_ML);
    
    Stat_s0_ML(:, s) = T*Scr_s0_ML(:, s)'*pinv(MSE_ML(:, :, s))...
        *Scr_s0_ML(:, s)/(2*(N+2));
    Stat_s1_ML(:, s) = T*Scr_s1_ML(:, s)'*((Var_s1_ML(:, :, s))\Scr_s1_ML(:, s));
    Stat_s2_ML(:, s) = T*Scr_s2_ML(:, s)'*((Var_s2_ML(:, :, s))\Scr_s2_ML(:, s));
    Scr_s4_ML = zeros(K, 1);
    for k = 1:K
        Stat_s3_ML(:, s, k) = T*Scr_s3_ML(:, s, k)'*...
            ((Var_s3_ML(:, :, s, k))\Scr_s3_ML(:, s, k));
        Scr_s4_ML(k, 1) = Scr_s3_ML(:, s, k);
    end
    Stat_s4_ML(:, s) = T*Scr_s4_ML'*((Var_s4_ML(:, :, s))\Scr_s4_ML);
    
    Stat_GH0_ML(:, s) = Stat_k0_ML(:, s) + Stat_s0_ML(:, s);
    Stat_GH1_ML(:, s) = Stat_k1_ML(:, s) + Stat_s1_ML(:, s);
    Stat_GH2_ML(:, s) = Stat_k2_ML(:, s) + Stat_s2_ML(:, s);
    for k = 1:K
        Stat_GH3_ML(:, s, k) = Stat_k3_ML(:, s, k) + Stat_s3_ML(:, s, k);
    end
    Stat_GH4_ML(:, s) = Stat_k4_ML(:, s) + Stat_s4_ML(:, s);
    
    % ML estimates - HAC
    HACStat_k0_ML(:, s) = T*(Scr_k0_ML(:, s).^2)/HAC_k0_ML(:, s);
    HACStat_k1_ML(:, s) = T*(Scr_k1_ML(:, s).^2)/HAC_k1_ML(:, s);
    HACStat_k2_ML(:, s) = T*(Scr_k2_ML(:, s).^2)/HAC_k2_ML(:, s);
    for k = 1:K
        HACStat_k3_ML(:, s, k) = T*(Scr_k3_ML(:, s, k).^2)/(HAC_k3_ML(:, s, k));
    end
    HACStat_k4_ML(:, s) = T*Scr_k4_ML'*((HAC_k4_ML(:, :, s))\Scr_k4_ML);
    
    HACStat_s0_ML(:, s) = T*Scr_s0_ML(:, s)'*((HAC_s0_ML(:, :, s))\Scr_s0_ML(:, s));
    HACStat_s1_ML(:, s) = T*Scr_s1_ML(:, s)'*((HAC_s1_ML(:, :, s))\Scr_s1_ML(:, s));
    HACStat_s2_ML(:, s) = T*Scr_s2_ML(:, s)'*((HAC_s2_ML(:, :, s))\Scr_s2_ML(:, s));
    for k = 1:K
        HACStat_s3_ML(:, s, k) = T*Scr_s3_ML(:, s, k)'*((HAC_s3_ML(:, :, s, k))\Scr_s3_ML(:, s, k));
    end
    HACStat_s4_ML(:, s) = T*Scr_s4_ML'*((HAC_s4_ML(:, :, s))\Scr_s4_ML);
    
    HACStat_GH0_ML(:, s) = HACStat_k0_ML(:, s) + HACStat_s0_ML(:, s);
    HACStat_GH1_ML(:, s) = HACStat_k1_ML(:, s) + HACStat_s1_ML(:, s);
    HACStat_GH2_ML(:, s) = HACStat_k2_ML(:, s) + HACStat_s2_ML(:, s);
    for k = 1:K
        HACStat_GH3_ML(:, s, k) = HACStat_k3_ML(:, s, k) + HACStat_s3_ML(:, s, k);
    end
    HACStat_GH4_ML(:, s) = HACStat_k4_ML(:, s) + HACStat_s4_ML(:, s);
    
    % ML estimates - KT
    StatKT_k0_ML(:, s) = Stat_k0_ML(:, s) * (Scr_k0_ML(:, s) > 0);
    StatKT_k1_ML(:, s) = Stat_k1_ML(:, s) * (Scr_k1_ML(:, s) > 0);
    StatKT_k2_ML(:, s) = Stat_k2_ML(:, s) * (Scr_k2_ML(:, s) > 0);
    for k = 1:K
        StatKT_k3_ML(:, s, k) = Stat_k3_ML(:, s, k) * (Scr_k3_ML(:, s, k) > 0);
    end
    Scr_k4_ML = Scr_k4_ML .* (Scr_k4_ML > 0);
    StatKT_k4_ML(:, s) = T*Scr_k4_ML'*((Var_k4_ML(:, :, s))\Scr_k4_ML);
    
    StatKT_GH0_ML(:, s) = StatKT_k0_ML(:, s) + Stat_s0_ML(:, s);
    StatKT_GH1_ML(:, s) = StatKT_k1_ML(:, s) + Stat_s1_ML(:, s);
    StatKT_GH2_ML(:, s) = StatKT_k2_ML(:, s) + Stat_s2_ML(:, s);
    for k = 1:K
        StatKT_GH3_ML(:, s, k) = StatKT_k3_ML(:, s, k) + Stat_s3_ML(:, s, k);
    end
    StatKT_GH4_ML(:, s) = StatKT_k4_ML(:, s) + Stat_s4_ML(:, s);
    
    % ML estimates - HAC - KT
    HACStatKT_k0_ML(:, s) = HACStat_k0_ML(:, s) * (Scr_k0_ML(:, s) > 0);
    HACStatKT_k1_ML(:, s) = HACStat_k1_ML(:, s) * (Scr_k1_ML(:, s) > 0);
    HACStatKT_k2_ML(:, s) = HACStat_k2_ML(:, s) * (Scr_k2_ML(:, s) > 0);
    for k = 1:K
        HACStatKT_k3_ML(:, s, k) = HACStat_k3_ML(:, s, k) * (Scr_k3_ML(:, s, k) > 0);
    end
    HACStatKT_k4_ML(:, s) = T*Scr_k4_ML'*((HAC_k4_ML(:, :, s))\Scr_k4_ML);
    
    HACStatKT_GH0_ML(:, s) = HACStatKT_k0_ML(:, s) + HACStat_s0_ML(:, s);
    HACStatKT_GH1_ML(:, s) = HACStatKT_k1_ML(:, s) + HACStat_s1_ML(:, s);
    HACStatKT_GH2_ML(:, s) = HACStatKT_k2_ML(:, s) + HACStat_s2_ML(:, s);
    for k = 1:K
        HACStatKT_GH3_ML(:, s, k) = HACStatKT_k3_ML(:, s, k) + HACStat_s3_ML(:, s, k);
    end
    HACStatKT_GH4_ML(:, s) = HACStatKT_k4_ML(:, s) + HACStat_s4_ML(:, s);
    
    % ML estimates - HK
    for k = 1:K
        Stat_HKk3_ML(:, s, k) = HK_k3_ML(:, s, k)/(Var_HKk3_ML(:, s, k));
        Stat_HKs3_ML(:, s, k) = HK_s3_ML(:, s, k)/(Var_HKs3_ML(:, s, k));
        Stat_HKGH3_ML(:, s, k) = Stat_HKk3_ML(:, s, k) + Stat_HKs3_ML(:, s, k);
    end
    
    waitbar(s/S,h)
    
    % Saving results
    if (rem(s, safe) == 0 || s == S)
        
        for level = [1, 5, 10]
            if (level == 1)
                index = 1;
            elseif (level == 5)
                index = 2;
            elseif (level == 10)
                index = 3;
            end
            
            c1 = chi2inv(1-level/100, 1);
            c2 = chi2inv(1-level/100, 2);
            c10 = chi2inv(1-level/100, 10);
            c11 = chi2inv(1-level/100, 11);
            c12 = chi2inv(1-level/100, 12);
            c22 = chi2inv(1-level/100, 22);
            
            cKT1 = (level == 1)*5.41189 + (level == 5)*2.70554 ...
                + (level == 10)*1.64237;
            cKT2 = (level == 1)*8.27325 + (level == 5)*5.13838 ...
                + (level == 10)*3.80781;
            cKT11 = (level == 1)*24.0489 + (level == 5)*19.0445 ...
                + (level == 10)*16.6703;
            cKT12 = (level == 1)*25.5483 + (level == 5)*20.4012 ...
                + (level == 10)*17.9494;
            cKTcase4t = (level == 1)*17.0996 + (level == 5)*12.5906 ...
                + (level == 10)*10.4939;
            cKTcase4GH = (level == 1)*33.3194 + (level == 5)*27.3636 ...
                + (level == 10)*24.4702;
            
            % True parameter values
            rejection(1, index, 1) = mean(Stat_k0 > c1);
            rejection(2, index, 1) = mean(Stat_k1 > c1);
            rejection(3, index, 1) = mean(Stat_k2 > c1);
            for k = 1:K
                rejection(3+k, index, 1) = mean(Stat_k3(:, :, k) > c1);
            end
            rejection(K+4, index, 1) = mean(Stat_k4 > c11);
            
            rejection(1, index, 2) = mean(Stat_s0 > c10);
            rejection(2, index, 2) = mean(Stat_s1 > c11);
            rejection(3, index, 2) = mean(Stat_s2 > c10);
            for k = 1:K
                rejection(3+k, index, 2) = mean(Stat_s3(:, :, k) > c1);
            end
            rejection(K+4, index, 2) = mean(Stat_s4 > c11);
            
            rejection(1, index, 3) = mean(Stat_GH0 > c11);
            rejection(2, index, 3) = mean(Stat_GH1 > c12);
            rejection(3, index, 3) = mean(Stat_GH2 > c11);
            for k = 1:K
                rejection(3+k, index, 3) = mean(Stat_GH3(:, :, k) > c2);
            end
            rejection(K+4, index, 3) = mean(Stat_GH4 > c22);
            
            % True parameter values - HAC
            HACrejection(1, index, 1) = mean(HACStat_k0 > c1);
            HACrejection(2, index, 1) = mean(HACStat_k1 > c1);
            HACrejection(3, index, 1) = mean(HACStat_k2 > c1);
            for k = 1:K
                HACrejection(3+k, index, 1) = mean(HACStat_k3(:, :, k) > c1);
            end
            HACrejection(K+4, index, 1) = mean(HACStat_k4 > c11);
            
            HACrejection(1, index, 2) = mean(HACStat_s0 > c10);
            HACrejection(2, index, 2) = mean(HACStat_s1 > c11);
            HACrejection(3, index, 2) = mean(HACStat_s2 > c10);
            for k = 1:K
                HACrejection(3+k, index, 2) = mean(HACStat_s3(:, :, k) > c1);
            end
            HACrejection(K+4, index, 2) = mean(HACStat_s4 > c11);
            
            HACrejection(1, index, 3) = mean(HACStat_GH0 > c11);
            HACrejection(2, index, 3) = mean(HACStat_GH1 > c12);
            HACrejection(3, index, 3) = mean(HACStat_GH2 > c11);
            for k = 1:K
                HACrejection(3+k, index, 3) = mean(HACStat_GH3(:, :, k) > c2);
            end
            HACrejection(K+4, index, 3) = mean(HACStat_GH4 > c22);
            
            % True parameter values - KT
            rejectionKT(1, index, 1) = mean(StatKT_k0 > cKT1);
            rejectionKT(2, index, 1) = mean(StatKT_k1 > cKT1);
            rejectionKT(3, index, 1) = mean(StatKT_k2 > cKT1);
            for k = 1:K
                rejectionKT(3+k, index, 1) = mean(StatKT_k3(:, :, k) > cKT1);
            end
            rejectionKT(K+4, index, 1) = mean(StatKT_k4 > cKTcase4t);
            
            rejectionKT(1, index, 2) = mean(StatKT_GH0 > cKT11);
            rejectionKT(2, index, 2) = mean(StatKT_GH1 > cKT12);
            rejectionKT(3, index, 2) = mean(StatKT_GH2 > cKT11);
            for k = 1:K
                rejectionKT(3+k, index, 2) = mean(StatKT_GH3(:, :, k) > cKT2);
            end
            rejectionKT(K+4, index, 2) = mean(StatKT_GH4 > cKTcase4GH);
            
            % True parameter values - HAC - KT
            HACrejectionKT(1, index, 1) = mean(HACStatKT_k0 > cKT1);
            HACrejectionKT(2, index, 1) = mean(HACStatKT_k1 > cKT1);
            HACrejectionKT(3, index, 1) = mean(HACStatKT_k2 > cKT1);
            for k = 1:K
                HACrejectionKT(3+k, index, 1) = mean(HACStatKT_k3(:, :, k) > cKT1);
            end
            HACrejectionKT(K+4, index, 1) = mean(HACStatKT_k4 > cKTcase4t);
            
            HACrejectionKT(1, index, 2) = mean(HACStatKT_GH0 > cKT11);
            HACrejectionKT(2, index, 2) = mean(HACStatKT_GH1 > cKT12);
            HACrejectionKT(3, index, 2) = mean(HACStatKT_GH2 > cKT11);
            for k = 1:K
                HACrejectionKT(3+k, index, 2) = mean(HACStatKT_GH3(:, :, k) > cKT2);
            end
            HACrejectionKT(K+4, index, 2) = mean(HACStatKT_GH4 > cKTcase4GH);
            
            % True parameter values - HK
            for k = 1:K
                rejectionHK(k, index, 1) = mean(Stat_HKk3(:, :, k) > c1);
                rejectionHK(k, index, 2) = mean(Stat_HKs3(:, :, k) > c1);
                rejectionHK(k, index, 3) = mean(Stat_HKGH3(:, :, k) > c2);
            end
            
            % ML estimates
            rejection_ML(1, index, 1) = mean(Stat_k0_ML > c1);
            rejection_ML(2, index, 1) = mean(Stat_k1_ML > c1);
            rejection_ML(3, index, 1) = mean(Stat_k2_ML > c1);
            for k = 1:K
                rejection_ML(3+k, index, 1) = mean(Stat_k3_ML(:, :, k) > c1);
            end
            rejection_ML(K+4, index, 1) = mean(Stat_k4_ML > c11);
            
            rejection_ML(1, index, 2) = mean(Stat_s0_ML > c10);
            rejection_ML(2, index, 2) = mean(Stat_s1_ML > c11);
            rejection_ML(3, index, 2) = mean(Stat_s2_ML > c10);
            for k = 1:K
                rejection_ML(3+k, index, 2) = mean(Stat_s3_ML(:, :, k) > c1);
            end
            rejection_ML(K+4, index, 2) = mean(Stat_s4_ML > c11);
            
            rejection_ML(1, index, 3) = mean(Stat_GH0_ML > c11);
            rejection_ML(2, index, 3) = mean(Stat_GH1_ML > c12);
            rejection_ML(3, index, 3) = mean(Stat_GH2_ML > c11);
            for k = 1:K
                rejection_ML(3+k, index, 3) = mean(Stat_GH3_ML(:, :, k) > c2);
            end
            rejection_ML(K+4, index, 3) = mean(Stat_GH4_ML > c22);
            
            % ML estimates - HAC
            HACrejection_ML(1, index, 1) = mean(HACStat_k0_ML > c1);
            HACrejection_ML(2, index, 1) = mean(HACStat_k1_ML > c1);
            HACrejection_ML(3, index, 1) = mean(HACStat_k2_ML > c1);
            for k = 1:K
                HACrejection_ML(3+k, index, 1) = mean(HACStat_k3_ML(:, :, k) > c1);
            end
            HACrejection_ML(K+4, index, 1) = mean(HACStat_k4_ML > c11);
            
            HACrejection_ML(1, index, 2) = mean(HACStat_s0_ML > c10);
            HACrejection_ML(2, index, 2) = mean(HACStat_s1_ML > c11);
            HACrejection_ML(3, index, 2) = mean(HACStat_s2_ML > c10);
            for k = 1:K
                HACrejection_ML(3+k, index, 2) = mean(HACStat_s3_ML(:, :, k) > c1);
            end
            HACrejection_ML(K+4, index, 2) = mean(HACStat_s4_ML > c11);
            
            HACrejection_ML(1, index, 3) = mean(HACStat_GH0_ML > c11);
            HACrejection_ML(2, index, 3) = mean(HACStat_GH1_ML > c12);
            HACrejection_ML(3, index, 3) = mean(HACStat_GH2_ML > c11);
            for k = 1:K
                HACrejection_ML(3+k, index, 3) = mean(HACStat_GH3_ML(:, :, k) > c2);
            end
            HACrejection_ML(K+4, index, 3) = mean(HACStat_GH4_ML > c22);
            
            % ML estimates - KT
            rejectionKT_ML(1, index, 1) = mean(StatKT_k0_ML > cKT1);
            rejectionKT_ML(2, index, 1) = mean(StatKT_k1_ML > cKT1);
            rejectionKT_ML(3, index, 1) = mean(StatKT_k2_ML > cKT1);
            for k = 1:K
                rejectionKT_ML(3+k, index, 1) = mean(StatKT_k3_ML(:, :, k) > cKT1);
            end
            rejectionKT_ML(K+4, index, 1) = mean(StatKT_k4_ML > cKTcase4t);
            
            rejectionKT_ML(1, index, 2) = mean(StatKT_GH0_ML > cKT11);
            rejectionKT_ML(2, index, 2) = mean(StatKT_GH1_ML > cKT12);
            rejectionKT_ML(3, index, 2) = mean(StatKT_GH2_ML > cKT11);
            for k = 1:K
                rejectionKT_ML(3+k, index, 2) = mean(StatKT_GH3_ML(:, :, k) > cKT2);
            end
            rejectionKT_ML(K+4, index, 2) = mean(StatKT_GH4_ML > cKTcase4GH);
            
            % ML estimates - HAC - KT
            HACrejectionKT_ML(1, index, 1) = mean(HACStatKT_k0_ML > cKT1);
            HACrejectionKT_ML(2, index, 1) = mean(HACStatKT_k1_ML > cKT1);
            HACrejectionKT_ML(3, index, 1) = mean(HACStatKT_k2_ML > cKT1);
            for k = 1:K
                HACrejectionKT_ML(3+k, index, 1) = mean(HACStatKT_k3_ML(:, :, k) > cKT1);
            end
            HACrejectionKT_ML(K+4, index, 1) = mean(HACStatKT_k4_ML > cKTcase4t);
            
            HACrejectionKT_ML(1, index, 2) = mean(HACStatKT_GH0_ML > cKT11);
            HACrejectionKT_ML(2, index, 2) = mean(HACStatKT_GH1_ML > cKT12);
            HACrejectionKT_ML(3, index, 2) = mean(HACStatKT_GH2_ML > cKT11);
            for k = 1:K
                HACrejectionKT_ML(3+k, index, 2) = mean(HACStatKT_GH3_ML(:, :, k) > cKT2);
            end
            HACrejectionKT_ML(K+4, index, 2) = mean(HACStatKT_GH4_ML > cKTcase4GH);
            
            % MLestimates - HK
            for k = 1:K
                rejectionHK_ML(k, index, 1) = mean(Stat_HKk3_ML(:, :, k) > c1);
                rejectionHK_ML(k, index, 2) = mean(Stat_HKs3_ML(:, :, k) > c1);
                rejectionHK_ML(k, index, 3) = mean(Stat_HKGH3_ML(:, :, k) > c2);
            end
            
        end
        
        save test.mat s ...
            rejection rejection_ML rejectionKT rejectionKT_ML ...
            Stat_k0 Stat_k1 Stat_k2 Stat_k3 Stat_k4 ...
            Stat_s0 Stat_s1 Stat_s2 Stat_s3 Stat_s4 ...
            Stat_GH0 Stat_GH1 Stat_GH2 Stat_GH3 Stat_GH4 ...
            StatKT_k0 StatKT_k1 StatKT_k2 StatKT_k3 StatKT_k4 ...
            StatKT_GH0 StatKT_GH1 StatKT_GH2 StatKT_GH3 StatKT_GH4 ...
            Stat_k0_ML Stat_k1_ML Stat_k2_ML Stat_k3_ML Stat_k4_ML ...
            Stat_s0_ML Stat_s1_ML Stat_s2_ML Stat_s3_ML Stat_s4_ML...
            Stat_GH0_ML Stat_GH1_ML Stat_GH2_ML Stat_GH3_ML Stat_GH4_ML ...
            StatKT_k0_ML StatKT_k1_ML StatKT_k2_ML StatKT_k3_ML StatKT_k4_ML ...
            StatKT_GH0_ML StatKT_GH1_ML StatKT_GH2_ML StatKT_GH3_ML StatKT_GH4_ML ...
            HACrejection HACrejection_ML HACrejectionKT HACrejectionKT_ML ...
            HACStat_k0 HACStat_k1 HACStat_k2 HACStat_k3 HACStat_k4 ...
            HACStat_s0 HACStat_s1 HACStat_s2 HACStat_s3 HACStat_s4 ...
            HACStat_GH0 HACStat_GH1 HACStat_GH2 HACStat_GH3 HACStat_GH4 ...
            HACStatKT_k0 HACStatKT_k1 HACStatKT_k2 HACStatKT_k3 HACStatKT_k4 ...
            HACStatKT_GH0 HACStatKT_GH1 HACStatKT_GH2 HACStatKT_GH3 HACStatKT_GH4 ...
            HACStat_k0_ML HACStat_k1_ML HACStat_k2_ML HACStat_k3_ML HACStat_k4_ML ...
            HACStat_s0_ML HACStat_s1_ML HACStat_s2_ML HACStat_s3_ML HACStat_s4_ML...
            HACStat_GH0_ML HACStat_GH1_ML HACStat_GH2_ML HACStat_GH3_ML HACStat_GH4_ML ...
            HACStatKT_k0_ML HACStatKT_k1_ML HACStatKT_k2_ML HACStatKT_k3_ML HACStatKT_k4_ML ...
            HACStatKT_GH0_ML HACStatKT_GH1_ML HACStatKT_GH2_ML HACStatKT_GH3_ML HACStatKT_GH4_ML ...
            rejectionHK rejectionHK_ML ...
            Stat_HKk3 Stat_HKs3 Stat_HKGH3 Stat_HKk3_ML Stat_HKs3_ML Stat_HKGH3_ML
    end
end
close(h)

movefile('test.mat',['IO/test/', title,'_test_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat']);