clear
clc
close all
path([pwd, '/subprograms/'], path)


%% SETTINGS AND PREALLOCATION

% Simulation settings
S = 10000; % Number of Monte Carlo replicas
safe = ceil(0.05*S); % Number of replicas per saving at each stage
title = 'S623';

S_bootstrap = 10000; % Number of bootstrap samples in each iteration
S1_grid = 7;
S2_grid = 4;
gx_grid = linspace(1.35, 2.85, S1_grid);
gv_grid = linspace(0.85, 1.15, S2_grid);

options = optimoptions('fmincon','Algorithm','interior-point',...
    'GradObj','off','Hessian','off','Display','off',...
    'MaxFunEvals',1000,'MaxIter',1000,'TolCon',0.0,'TolFun',10e-9);
maxerror = 1e-4; % Tolerance criterion for numerical convergence
maxiter = 1e2; % Maximum number of iterations for numerical convergence

% Model settings
T = 100; % Sample size
hypothesis = 0; % Distribution of innovations
N = 10;
M = N+1;
K = N+1;

% Preallocation
Stat_k0_ML = zeros(S1_grid, S2_grid, S_bootstrap);
Stat_k1_ML = zeros(S1_grid, S2_grid, S_bootstrap);
Stat_k2_ML = zeros(S1_grid, S2_grid, S_bootstrap);
Stat_k3_ML = zeros(S1_grid, S2_grid, S_bootstrap, K);
Stat_k4_ML = zeros(S1_grid, S2_grid, S_bootstrap);

Stat_s0_ML = zeros(S1_grid, S2_grid, S_bootstrap);
Stat_s1_ML = zeros(S1_grid, S2_grid, S_bootstrap);
Stat_s2_ML = zeros(S1_grid, S2_grid, S_bootstrap);
Stat_s3_ML = zeros(S1_grid, S2_grid, S_bootstrap, K);
Stat_s4_ML = zeros(S1_grid, S2_grid, S_bootstrap);

Stat_GH0_ML = zeros(S1_grid, S2_grid, S_bootstrap);
Stat_GH1_ML = zeros(S1_grid, S2_grid, S_bootstrap);
Stat_GH2_ML = zeros(S1_grid, S2_grid, S_bootstrap);
Stat_GH3_ML = zeros(S1_grid, S2_grid, S_bootstrap, K);
Stat_GH4_ML = zeros(S1_grid, S2_grid, S_bootstrap);

StatKT_k0_ML = zeros(S1_grid, S2_grid, S_bootstrap);
StatKT_k1_ML = zeros(S1_grid, S2_grid, S_bootstrap);
StatKT_k2_ML = zeros(S1_grid, S2_grid, S_bootstrap);
StatKT_k3_ML = zeros(S1_grid, S2_grid, S_bootstrap, K);
StatKT_k4_ML = zeros(S1_grid, S2_grid, S_bootstrap);

StatKT_GH0_ML = zeros(S1_grid, S2_grid, S_bootstrap);
StatKT_GH1_ML = zeros(S1_grid, S2_grid, S_bootstrap);
StatKT_GH2_ML = zeros(S1_grid, S2_grid, S_bootstrap);
StatKT_GH3_ML = zeros(S1_grid, S2_grid, S_bootstrap, K);
StatKT_GH4_ML = zeros(S1_grid, S2_grid, S_bootstrap);

Stat_HKk3_ML = zeros(S1_grid, S2_grid, S_bootstrap, K);
Stat_HKs3_ML = zeros(S1_grid, S2_grid, S_bootstrap, K);
Stat_HKGH3_ML = zeros(S1_grid, S2_grid, S_bootstrap, K);

innovations = normrnd(0, 1, K, T, S_bootstrap);

Stat_k0_cv = zeros(S1_grid, S2_grid, 3);
Stat_k1_cv = zeros(S1_grid, S2_grid, 3);
Stat_k2_cv = zeros(S1_grid, S2_grid, 3);
Stat_k3_cv = zeros(S1_grid, S2_grid, 3, K);
Stat_k4_cv = zeros(S1_grid, S2_grid, 3);

Stat_s0_cv = zeros(S1_grid, S2_grid, 3);
Stat_s1_cv = zeros(S1_grid, S2_grid, 3);
Stat_s2_cv = zeros(S1_grid, S2_grid, 3);
Stat_s3_cv = zeros(S1_grid, S2_grid, 3, K);
Stat_s4_cv = zeros(S1_grid, S2_grid, 3);

Stat_GH0_cv = zeros(S1_grid, S2_grid, 3);
Stat_GH1_cv = zeros(S1_grid, S2_grid, 3);
Stat_GH2_cv = zeros(S1_grid, S2_grid, 3);
Stat_GH3_cv = zeros(S1_grid, S2_grid, 3, K);
Stat_GH4_cv = zeros(S1_grid, S2_grid, 3);

StatKT_k0_cv = zeros(S1_grid, S2_grid, 3);
StatKT_k1_cv = zeros(S1_grid, S2_grid, 3);
StatKT_k2_cv = zeros(S1_grid, S2_grid, 3);
StatKT_k3_cv = zeros(S1_grid, S2_grid, 3, K);
StatKT_k4_cv = zeros(S1_grid, S2_grid, 3);

StatKT_GH0_cv = zeros(S1_grid, S2_grid, 3);
StatKT_GH1_cv = zeros(S1_grid, S2_grid, 3);
StatKT_GH2_cv = zeros(S1_grid, S2_grid, 3);
StatKT_GH3_cv = zeros(S1_grid, S2_grid, 3, K);
StatKT_GH4_cv = zeros(S1_grid, S2_grid, 3);

Stat_HKk3_cv = zeros(S1_grid, S2_grid, 3, K);
Stat_HKs3_cv = zeros(S1_grid, S2_grid, 3, K);
Stat_HKGH3_cv = zeros(S1_grid, S2_grid, 3, K);

rejection = zeros(K+4, 3, 3);
rejection_ML = zeros(K+4, 3, 3);
rejectionKT = zeros(K+4, 3, 2);
rejectionKT_ML = zeros(K+4, 3, 2);

rejectionHK = zeros(K, 3, 3);
rejectionHK_ML = zeros(K, 3, 3);


%% BOOTSTRAP CALCULATIONS

for s1_grid = 1:S1_grid
    for s2_grid = 1:S2_grid
        
        % Simulation settings
        rng('default')
        rng(280990)
        
        % Model settings
        parameters = [gx_grid(s1_grid); gv_grid(s2_grid)];
        kappa = 1;
        
        % Bootstrap calculations
        h = waitbar(0, ['BOOTSTRAP. gx: ', num2str(s1_grid), ' out of ', ...
            num2str(S1_grid), ' - gv: ', num2str(s2_grid), ...
            ' out of ', num2str(S2_grid)]);
        
        for s = 1:S_bootstrap
            
            display(s,'Replica')
            
            % Simulation and estimation
            C = zeros(K, 1);
            F = [[1, zeros(1, N)]; ...
                zeros(N, K)];
            G = [[sqrt(parameters(1)), zeros(1, N)]; ...
                [zeros(N, 1), sqrt(parameters(2))*eye(N)]];
            H = [ones(N, 1), eye(N)];
            
            EX1 = zeros(K, 1);
            VX1 = [[kappa, zeros(1, N)]; ...
                [zeros(N, 1), diag(parameters(2))*eye(N)]];
            
            X = zeros(M, T);
            Y = zeros(N, T);
            
            X1 = mvnrnd(EX1, VX1)';
            X(:, 1) = X1;
            Y(:, 1) = H*X1;
            
            for t = 2:T
                X(:, t) = C + F*X(:, t-1) + G*innovations(:, t, s);
                Y(:, t) = H*X(:, t);
            end
            
            lower = [ 0.0001  0.0001]';
            upper = [10.0000 10.0000]';
            
            MLestimate = fmincon('Multivariate_Likelihood', ...
                parameters, [], [], [], [], lower, upper, [], options, Y);
            
            % Computation of score
            kappa = 1e3;
            F = [[1, zeros(1, K)]; zeros(K, K+1)];
            H = [ones(N, 1), eye(N), zeros(N, 1)];
            
            G_ML = [[sqrt(MLestimate(1)), zeros(1, N)]; ...
                [zeros(N, 1), sqrt(MLestimate(2))*eye(N)]; ...
                [sqrt(MLestimate(2)), zeros(1, N)]];
            
            EX1 = zeros(K+1, 1);
            VX1_ML = [[kappa*MLestimate(1), zeros(1, K)]; ...
                [zeros(N, 1), diag(MLestimate(2))*eye(N), zeros(N, 1)]; ...
                [zeros(1, K), MLestimate(2)]];
            
            order = [K+1 2:K];
            
            [Xf_ML, Pf_ML, Xs_ML, Ps_ML] = KalmanFS(EX1, VX1_ML, F, G_ML, H, Y);
            
            Xs_ML = Xs_ML(order, :);
            Ps_ML = Ps_ML(order, order, :);
            
            e0_ML = zeros(N, T-2);
            P0_ML = zeros(N, N, T-2);
            for t = 2:T-1
                e0_ML(:, t-1) = Y(:, t) - H*Xf_ML(:, t);
                P0_ML(:, :, t-1) = H*Pf_ML(:, :, t)*H';
            end
            e1_ML = Xs_ML(1:K, 2:T-1)/sqrt(MLestimate(2));
            P1_ML = Ps_ML(1:K, 1:K, 2:T-1)/MLestimate(2);
            e2_ML = Xs_ML(2:K, 2:T-1)/sqrt(MLestimate(2));
            P2_ML = Ps_ML(2:K, 2:K,2:T-1)/MLestimate(2);
            e3_ML = zeros(1, T-2, K);
            P3_ML = zeros(1, 1, T-2, K);
            for k = 1:K
                e3_ML(:, :, k) = Xs_ML(k, 2:T-1)/sqrt(MLestimate(2));
                P3_ML(:, :, :, k) = Ps_ML(k, k, 2:T-1)/MLestimate(2);
            end
            
            sk0_ML = ScoreFK(e0_ML, P0_ML);
            sk1_ML = ScoreSK(e1_ML, P1_ML);
            sk2_ML = ScoreSK(e2_ML, P2_ML);
            sk3_ML = zeros(1, T-2, K);
            for k = 1:K
                sk3_ML(:, :, k) = ScoreSK(e3_ML(:, :, k), P3_ML(:, :, :, k));
            end
            
            ss0_ML = ScoreFS(e0_ML, P0_ML);
            ss1_ML = ScoreSS(e1_ML, P1_ML);
            ss2_ML = ScoreSS(e2_ML, P2_ML);
            ss3_ML = zeros(1, T-2, K);
            for k = 1:K
                ss3_ML(:, :, k) = ScoreSS(e3_ML(:, :, k), P3_ML(:, :, :, k));
            end
            
            Scr_k0_ML = mean(sk0_ML, 2);
            Scr_k1_ML = mean(sk1_ML, 2);
            Scr_k2_ML = mean(sk2_ML, 2);
            Scr_k3_ML = zeros(1, K);
            for k = 1:K
                Scr_k3_ML(:, k) = mean(sk3_ML(:, :, k), 2);
            end
            
            MSE_ML = mean(P0_ML, 3);
            
            Scr_s0_ML = mean(ss0_ML, 2);
            Scr_s1_ML = mean(ss1_ML, 2);
            Scr_s2_ML = mean(ss2_ML, 2);
            Scr_s3_ML = zeros(1, K);
            for k = 1:K
                Scr_s3_ML(:, k) = mean(ss3_ML(:, :, k), 2);
            end
            
            HK_k3_ML = zeros(1, K);
            HK_s3_ML = zeros(1, K);
            for k = 1:K
                HK_k3_ML(:, k) = StatHKK(e3_ML(:, :, k));
                HK_s3_ML(:, k) = StatHKS(e3_ML(:, :, k));
            end
            
            % Asymptotic variance
            [PHI, THETA] = Multivariate_WKfilter(MLestimate(1), MLestimate(2));
            THETA = THETA(1:(end-1), :);
            auxGAMMA0 = AutocovarianceWK(PHI, THETA, eye(4), 0);
            auxGAMMA0 = [[auxGAMMA0(1, 1), auxGAMMA0(1, 2)*ones(1, N)]; ...
                [auxGAMMA0(2, 1)*ones(N, 1), ...
                auxGAMMA0(2, 2)*eye(N) + auxGAMMA0(2, 3)*(ones(N, N) - eye(N))]];
            
            r = K;
            [~, aux_bs, ~] = ScoreSS(zeros(K, 1), eye(K) - auxGAMMA0);
            [~, aux_bk, ~] = ScoreSK(zeros(K, 1), eye(K) - auxGAMMA0);
            
            B_e1_1 = aux_bs(1:r, :);
            B_e2_1 = aux_bk(2:1+r^2);
            B_e3_1 = aux_bs(r+1:end, :);
            B_e4_1 = aux_bk(2+r^2:end);
            
            r = K-1;
            E2 = [zeros(K-1, 1), eye(K-1)];
            [~, aux_bs, ~] = ScoreSS(zeros(K-1, 1), eye(K-1) - auxGAMMA0(2:K, 2:K));
            [~, aux_bk, ~] = ScoreSK(zeros(K-1, 1), eye(K-1) - auxGAMMA0(2:K, 2:K));
            
            B_e1_2 = E2'*aux_bs(1:r, :);
            B_e2_2 = kron(E2',E2')*aux_bk(2:1+r^2);
            B_e3_2 = kron(E2',E2')*aux_bs(r+1:end, :);
            B_e4_2 = kron(E2',E2')*aux_bk(2+r^2:end);
            
            r = 1;
            E3 = zeros(1, K, K);
            B_e1_3 = zeros(K, 1, K);
            B_e2_3 = zeros(K^2, 1, K);
            B_e3_3 = zeros(K^2, 1, K);
            B_e4_3 = zeros(K^2, 1, K);
            for k = 1:K
                E3(1, k, k) = 1;
                [~, aux_bs, ~] = ScoreSS(0, 1 - auxGAMMA0(k, k));
                [~, aux_bk, ~] = ScoreSK(0, 1 - auxGAMMA0(k, k));
                
                B_e1_3(:, :, k) = E3(:, :, k)'*aux_bs(1:r, :);
                B_e2_3(:, :, k) = kron(E3(:, :, k)', E3(:, :, k)')*aux_bk(2:1+r^2);
                B_e3_3(:, :, k) = kron(E3(:, :, k)', E3(:, :, k)')*aux_bs(r+1:end, :);
                B_e4_3(:, :, k) = kron(E3(:, :, k)', E3(:, :, k)')*aux_bk(2+r^2:end);
            end
            
            B_e1_4 = reshape(B_e1_3, [K, K]);
            B_e2_4 = reshape(B_e2_3, [K^2, K]);
            B_e3_4 = reshape(B_e3_3, [K^2, K]);
            B_e4_4 = reshape(B_e4_3, [K^2, K]);
            
            [CMe1e1, CMe2e2, CMe3e3, CMe4e4] = AutocovarianceMM(auxGAMMA0, auxGAMMA0);
            
            AGFe1e1 = CMe1e1;
            AGFe2e2 = CMe2e2;
            AGFe3e3 = CMe3e3;
            AGFe4e4 = CMe4e4;
            
            kappa_k3 = zeros(1, K);
            kappa_s3 = zeros(1, K);
            for k = 1:K
                kappa_k3(:, k) = auxGAMMA0(k, k)^4;
                kappa_s3(:, k) = auxGAMMA0(k, k)^3;
            end
            
            j = 0;
            error = 1;
            
            while ((j < maxiter) && (error >= maxerror))
                
                auxGAMMAj = AutocovarianceWK(PHI, THETA, eye(4), j+1);
                auxGAMMAj = [[auxGAMMAj(1, 1), auxGAMMAj(1, 2)*ones(1, N)]; ...
                    [auxGAMMAj(2, 1)*ones(N, 1), ...
                    auxGAMMAj(2, 2)*eye(N) + auxGAMMAj(2, 3)*(ones(N, N) - eye(N))]];
                [CMe1e1, CMe2e2, CMe3e3, CMe4e4] = AutocovarianceMM(auxGAMMA0, auxGAMMAj);
                [CMe1e1m, CMe2e2m, CMe3e3m, CMe4e4m] = AutocovarianceMM(auxGAMMA0, auxGAMMAj');
                
                AGFe1e1 = AGFe1e1 + CMe1e1 + CMe1e1m;
                AGFe2e2 = AGFe2e2 + CMe2e2 + CMe2e2m;
                AGFe3e3 = AGFe3e3 + CMe3e3 + CMe3e3m;
                AGFe4e4 = AGFe4e4 + CMe4e4 + CMe4e4m;
                
                kappa_k3(:, k) = kappa_k3(:, k) + 2*auxGAMMAj(k, k)^4;
                kappa_s3(:, k) = kappa_s3(:, k) + 2*auxGAMMAj(k, k)^3;
                
                j = j+1;
                
                de1e1 = max(max(abs(CMe1e1 + CMe1e1m)));
                de2e2 = max(max(abs(CMe2e2 + CMe2e2m)));
                de3e3 = max(max(abs(CMe3e3 + CMe3e3m)));
                de4e4 = max(max(abs(CMe4e4 + CMe4e4m)));
                error = max([de1e1, de2e2, de3e3, de4e4]);
                
            end
            
            Var_k1_ML = (B_e4_1')*AGFe4e4*(B_e4_1) - (B_e2_1')*AGFe2e2*(B_e2_1);
            Var_k2_ML = (B_e4_2')*AGFe4e4*(B_e4_2) - (B_e2_2')*AGFe2e2*(B_e2_2);
            Var_k3_ML = zeros(1, K);
            for k = 1:K
                Var_k3_ML(:, k) = (B_e4_3(:, :, k)')*AGFe4e4*(B_e4_3(:, :, k)) - ...
                    (B_e2_3(:, :, k)')*AGFe2e2*(B_e2_3(:, :, k));
            end
            Var_k4_ML = (B_e4_4')*AGFe4e4*(B_e4_4) - (B_e2_4')*AGFe2e2*(B_e2_4);
            
            Var_s1_ML = (B_e3_1')*AGFe3e3*(B_e3_1) - (B_e1_1')*AGFe1e1*(B_e1_1);
            Var_s2_ML = (B_e3_2')*AGFe3e3*(B_e3_2) - (B_e1_2')*AGFe1e1*(B_e1_2);
            Var_s3_ML = zeros(1, K);
            for k = 1:K
                Var_s3_ML(:, k) = (B_e3_3(:, :, k)')*AGFe3e3*(B_e3_3(:, :, k)) - ...
                    (B_e1_3(:, :, k)')*AGFe1e1*(B_e1_3(:, :, k));
            end
            Var_s4_ML = (B_e3_4')*AGFe3e3*(B_e3_4) - (B_e1_4')*AGFe1e1*(B_e1_4);
            
            Var_HKk3_ML = zeros(1, K);
            Var_HKs3_ML = zeros(1, K);
            for k = 1:K
                Var_HKk3_ML(:, k) = kappa_k3(:, k)/auxGAMMA0(k,k)^4;
                Var_HKs3_ML(:, k) = kappa_s3(:, k)/auxGAMMA0(k,k)^3;
            end
            
            % Test statistic
            Stat_k0_ML(s1_grid, s2_grid, s) = T*(Scr_k0_ML.^2)/(N*(N+2)/2);
            Stat_k1_ML(s1_grid, s2_grid, s) = T*(Scr_k1_ML.^2)/(Var_k1_ML);
            Stat_k2_ML(s1_grid, s2_grid, s) = T*(Scr_k2_ML.^2)/(Var_k2_ML);
            Scr_k4_ML = zeros(K, 1);
            for k = 1:K
                Stat_k3_ML(s1_grid, s2_grid, s, k) = T*(Scr_k3_ML(:, k).^2)/(Var_k3_ML(:, k));
                Scr_k4_ML(k, 1) = Scr_k3_ML(:, k);
            end
            Stat_k4_ML(s1_grid, s2_grid, s) = T*Scr_k4_ML'*((Var_k4_ML)\Scr_k4_ML);
            
            Stat_s0_ML(s1_grid, s2_grid, s) = T*Scr_s0_ML'*pinv(MSE_ML)...
                *Scr_s0_ML/(2*(N+2));
            Stat_s1_ML(s1_grid, s2_grid, s) = T*Scr_s1_ML'*((Var_s1_ML)\Scr_s1_ML);
            Stat_s2_ML(s1_grid, s2_grid, s) = T*Scr_s2_ML'*((Var_s2_ML)\Scr_s2_ML);
            Scr_s4_ML = zeros(K, 1);
            for k = 1:K
                Stat_s3_ML(s1_grid, s2_grid, s, k) = T*Scr_s3_ML(:, k)'*...
                    ((Var_s3_ML(:, k))\Scr_s3_ML(:, k));
                Scr_s4_ML(k, 1) = Scr_s3_ML(:, k);
            end
            Stat_s4_ML(s1_grid, s2_grid, s) = T*Scr_s4_ML'*((Var_s4_ML)\Scr_s4_ML);
            
            Stat_GH0_ML(s1_grid, s2_grid, s) = Stat_k0_ML(s1_grid, s2_grid, s) + Stat_s0_ML(s1_grid, s2_grid, s);
            Stat_GH1_ML(s1_grid, s2_grid, s) = Stat_k1_ML(s1_grid, s2_grid, s) + Stat_s1_ML(s1_grid, s2_grid, s);
            Stat_GH2_ML(s1_grid, s2_grid, s) = Stat_k2_ML(s1_grid, s2_grid, s) + Stat_s2_ML(s1_grid, s2_grid, s);
            for k = 1:K
                Stat_GH3_ML(s1_grid, s2_grid, s, k) = Stat_k3_ML(s1_grid, s2_grid, s, k) + Stat_s3_ML(s1_grid, s2_grid, s, k);
            end
            Stat_GH4_ML(s1_grid, s2_grid, s) = Stat_k4_ML(s1_grid, s2_grid, s) + Stat_s4_ML(s1_grid, s2_grid, s);
            
            StatKT_k0_ML(s1_grid, s2_grid, s) = Stat_k0_ML(s1_grid, s2_grid, s) * (Scr_k0_ML > 0);
            StatKT_k1_ML(s1_grid, s2_grid, s) = Stat_k1_ML(s1_grid, s2_grid, s) * (Scr_k1_ML > 0);
            StatKT_k2_ML(s1_grid, s2_grid, s) = Stat_k2_ML(s1_grid, s2_grid, s) * (Scr_k2_ML > 0);
            for k = 1:K
                StatKT_k3_ML(s1_grid, s2_grid, s, k) = Stat_k3_ML(s1_grid, s2_grid, s, k) * (Scr_k3_ML(:, k) > 0);
            end
            Scr_k4_ML = Scr_k4_ML .* (Scr_k4_ML > 0);
            StatKT_k4_ML(s1_grid, s2_grid, s) = T*Scr_k4_ML'*((Var_k4_ML)\Scr_k4_ML);
            
            StatKT_GH0_ML(s1_grid, s2_grid, s) = StatKT_k0_ML(s1_grid, s2_grid, s) + Stat_s0_ML(s1_grid, s2_grid, s);
            StatKT_GH1_ML(s1_grid, s2_grid, s) = StatKT_k1_ML(s1_grid, s2_grid, s) + Stat_s1_ML(s1_grid, s2_grid, s);
            StatKT_GH2_ML(s1_grid, s2_grid, s) = StatKT_k2_ML(s1_grid, s2_grid, s) + Stat_s2_ML(s1_grid, s2_grid, s);
            for k = 1:K
                StatKT_GH3_ML(s1_grid, s2_grid, s, k) = StatKT_k3_ML(s1_grid, s2_grid, s, k) + Stat_s3_ML(s1_grid, s2_grid, s, k);
            end
            StatKT_GH4_ML(s1_grid, s2_grid, s) = StatKT_k4_ML(s1_grid, s2_grid, s) + Stat_s4_ML(s1_grid, s2_grid, s);
            
            for k = 1:K
                Stat_HKk3_ML(s1_grid, s2_grid, s, k) = HK_k3_ML(:, k)/(Var_HKk3_ML(:, k));
                Stat_HKs3_ML(s1_grid, s2_grid, s, k) = HK_s3_ML(:, k)/(Var_HKs3_ML(:, k));
                Stat_HKGH3_ML(s1_grid, s2_grid, s, k) = Stat_HKk3_ML(s1_grid, s2_grid, s, k) + Stat_HKs3_ML(s1_grid, s2_grid, s, k);
            end
            
            waitbar(s/S_bootstrap, h)
            
        end
        close(h)
        
        save bootstrap.mat gx_grid gv_grid ...
            Stat_k0_ML Stat_k1_ML Stat_k2_ML Stat_k3_ML Stat_k4_ML ...
            Stat_s0_ML Stat_s1_ML Stat_s2_ML Stat_s3_ML Stat_s4_ML...
            Stat_GH0_ML Stat_GH1_ML Stat_GH2_ML Stat_GH3_ML Stat_GH4_ML ...
            StatKT_k0_ML StatKT_k1_ML StatKT_k2_ML StatKT_k3_ML StatKT_k4_ML ...
            StatKT_GH0_ML StatKT_GH1_ML StatKT_GH2_ML StatKT_GH3_ML StatKT_GH4_ML ...
            Stat_HKk3_ML Stat_HKs3_ML Stat_HKGH3_ML
        
    end
end

movefile('bootstrap.mat',['IO/bootstrap/', title,'_bootstrap_T',num2str(T),'.mat']);


%% BOOTSTRAP CRITICAL VALUES

load(['IO/bootstrap/', title,'_bootstrap_T',num2str(T),'.mat'])

for s1_grid = 1:S1_grid
    for s2_grid = 1:S2_grid
        
        for a = 1:3
            
            level = (a == 1)*0.99 + (a == 2)*0.95 + (a == 3)*0.90;
            
            % Test statistic
            Stat_k0_cv(s1_grid, s2_grid, a) = quantile(Stat_k0_ML(s1_grid, s2_grid, :), level);
            Stat_k1_cv(s1_grid, s2_grid, a) = quantile(Stat_k1_ML(s1_grid, s2_grid, :), level);
            Stat_k2_cv(s1_grid, s2_grid, a) = quantile(Stat_k2_ML(s1_grid, s2_grid, :), level);
            for k = 1:K
                Stat_k3_cv(s1_grid, s2_grid, a, k) = quantile(Stat_k3_ML(s1_grid, s2_grid, :, k), level);
            end
            Stat_k4_cv(s1_grid, s2_grid, a) = quantile(Stat_k4_ML(s1_grid, s2_grid, :), level);
            
            Stat_s0_cv(s1_grid, s2_grid, a) = quantile(Stat_s0_ML(s1_grid, s2_grid, :), level);
            Stat_s1_cv(s1_grid, s2_grid, a) = quantile(Stat_s1_ML(s1_grid, s2_grid, :), level);
            Stat_s2_cv(s1_grid, s2_grid, a) = quantile(Stat_s2_ML(s1_grid, s2_grid, :), level);
            for k = 1:K
                Stat_s3_cv(s1_grid, s2_grid, a, k) = quantile(Stat_s3_ML(s1_grid, s2_grid, :, k), level);
            end
            Stat_s4_cv(s1_grid, s2_grid, a) = quantile(Stat_s4_ML(s1_grid, s2_grid, :), level);
            
            Stat_GH0_cv(s1_grid, s2_grid, a) = quantile(Stat_GH0_ML(s1_grid, s2_grid, :), level);
            Stat_GH1_cv(s1_grid, s2_grid, a) = quantile(Stat_GH1_ML(s1_grid, s2_grid, :), level);
            Stat_GH2_cv(s1_grid, s2_grid, a) = quantile(Stat_GH2_ML(s1_grid, s2_grid, :), level);
            for k = 1:K
                Stat_GH3_cv(s1_grid, s2_grid, a, k) = quantile(Stat_GH3_ML(s1_grid, s2_grid, :, k), level);
            end
            Stat_GH4_cv(s1_grid, s2_grid, a) = quantile(Stat_GH4_ML(s1_grid, s2_grid, :), level);
            
            StatKT_k0_cv(s1_grid, s2_grid, a) = quantile(StatKT_k0_ML(s1_grid, s2_grid, :), level);
            StatKT_k1_cv(s1_grid, s2_grid, a) = quantile(StatKT_k1_ML(s1_grid, s2_grid, :), level);
            StatKT_k2_cv(s1_grid, s2_grid, a) = quantile(StatKT_k2_ML(s1_grid, s2_grid, :), level);
            for k = 1:K
                StatKT_k3_cv(s1_grid, s2_grid, a, k) = quantile(StatKT_k3_ML(s1_grid, s2_grid, :, k), level);
            end
            StatKT_k4_cv(s1_grid, s2_grid, a) = quantile(StatKT_k4_ML(s1_grid, s2_grid, :), level);
            
            StatKT_GH0_cv(s1_grid, s2_grid, a) = quantile(StatKT_GH0_ML(s1_grid, s2_grid, :), level);
            StatKT_GH1_cv(s1_grid, s2_grid, a) = quantile(StatKT_GH1_ML(s1_grid, s2_grid, :), level);
            StatKT_GH2_cv(s1_grid, s2_grid, a) = quantile(StatKT_GH2_ML(s1_grid, s2_grid, :), level);
            for k = 1:K
                StatKT_GH3_cv(s1_grid, s2_grid, a, k) = quantile(StatKT_GH3_ML(s1_grid, s2_grid, :, k), level);
            end
            StatKT_GH4_cv(s1_grid, s2_grid, a) = quantile(StatKT_GH4_ML(s1_grid, s2_grid, :), level);
            
            for k = 1:K
                Stat_HKk3_cv(s1_grid, s2_grid, a, k) = quantile(Stat_HKk3_ML(s1_grid, s2_grid, :, k), level);
                Stat_HKs3_cv(s1_grid, s2_grid, a, k) = quantile(Stat_HKs3_ML(s1_grid, s2_grid, :, k), level);
                Stat_HKGH3_cv(s1_grid, s2_grid, a, k) = quantile(Stat_HKGH3_ML(s1_grid, s2_grid, :, k), level);
            end
            
        end
        
        save crit.mat gx_grid gv_grid ...
            Stat_k0_cv Stat_k1_cv Stat_k2_cv Stat_k3_cv Stat_k4_cv ...
            Stat_s0_cv Stat_s1_cv Stat_s2_cv Stat_s3_cv Stat_s4_cv...
            Stat_GH0_cv Stat_GH1_cv Stat_GH2_cv Stat_GH3_cv Stat_GH4_cv ...
            StatKT_k0_cv StatKT_k1_cv StatKT_k2_cv StatKT_k3_cv StatKT_k4_cv ...
            StatKT_GH0_cv StatKT_GH1_cv StatKT_GH2_cv StatKT_GH3_cv StatKT_GH4_cv ...
            Stat_HKk3_cv Stat_HKs3_cv Stat_HKGH3_cv
    end
end

movefile('crit.mat',['IO/bootstrap/', title,'_critical_T',num2str(T),'.mat']);


%% BOOTSTRAP TEST

h = waitbar(0,'Computation of test statistic');
load(['IO/bootstrap/', title,'_critical_T',num2str(T),'.mat'])
load(['IO/estimate/', title,'_estimate_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat'])
load(['IO/test/', title,'_test_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat'])

if (parameters(1) < gx_grid(1))
    ix = 1;
    wx = 1;
elseif (parameters(1) >= gx_grid(S1_grid))
    ix = S1_grid-1;
    wx = 0;
else
    ix = sum((parameters(1) >= gx_grid));
    wx = (parameters(1) - gx_grid(ix))/(gx_grid(ix+1) - gx_grid(ix));
end

if (parameters(2) < gv_grid(1))
    iv = 1;
    wv = 1;
elseif (parameters(2) >= gv_grid(S2_grid))
    iv = S2_grid-1;
    wv = 0;
else
    iv = sum((parameters(2) >= gv_grid));
    wv = (parameters(2) - gv_grid(iv))/(gv_grid(iv+1) - gv_grid(iv));
end

for s = 1:S
    
    for a = 1:3
        
        if (MLestimate(1, s) < gx_grid(1))
            ix_ML = 1;
            wx_ML = 1;
        elseif (MLestimate(1, s) >= gx_grid(S1_grid))
            ix_ML = S1_grid-1;
            wx_ML = 0;
        else
            ix_ML = sum((MLestimate(1, s) >= gx_grid));
            wx_ML = (MLestimate(1, s) - gx_grid(ix_ML))/(gx_grid(ix_ML+1) - gx_grid(ix_ML));
        end
        
        if (MLestimate(2, s) < gv_grid(1))
            iv_ML = 1;
            wv_ML = 1;
        elseif (MLestimate(2, s) >= gv_grid(S2_grid))
            iv_ML = S2_grid-1;
            wv_ML = 0;
        else
            iv_ML = sum((MLestimate(2, s) >= gv_grid));
            wv_ML = (MLestimate(2, s) - gv_grid(iv_ML))/(gv_grid(iv_ML+1) - gv_grid(iv_ML));
        end
        
        % True parameter values
        rejection(1, a, 1) = mean(Stat_k0 > wx*wv*Stat_k0_cv(ix, iv, a) + (1-wx)*wv*Stat_k0_cv(ix+1, iv, a) + wx*(1-wv)*Stat_k0_cv(ix, iv+1, a) + (1-wx)*(1-wv)*Stat_k0_cv(ix+1, iv+1, a));
        rejection(2, a, 1) = mean(Stat_k1 > wx*wv*Stat_k1_cv(ix, iv, a) + (1-wx)*wv*Stat_k1_cv(ix+1, iv, a) + wx*(1-wv)*Stat_k1_cv(ix, iv+1, a) + (1-wx)*(1-wv)*Stat_k1_cv(ix+1, iv+1, a));
        rejection(3, a, 1) = mean(Stat_k2 > wx*wv*Stat_k2_cv(ix, iv, a) + (1-wx)*wv*Stat_k2_cv(ix+1, iv, a) + wx*(1-wv)*Stat_k2_cv(ix, iv+1, a) + (1-wx)*(1-wv)*Stat_k2_cv(ix+1, iv+1, a));
        for k = 1:K
            rejection(3+k, a, 1) = mean(Stat_k3(:, :, k) > wx*wv*Stat_k3_cv(ix, iv, a, k) + (1-wx)*wv*Stat_k3_cv(ix+1, iv, a, k) + wx*(1-wv)*Stat_k3_cv(ix, iv+1, a, k) + (1-wx)*(1-wv)*Stat_k3_cv(ix+1, iv+1, a, k));
        end
        rejection(K+4, a, 1) = mean(Stat_k4 > wx*wv*Stat_k4_cv(ix, iv, a) + (1-wx)*wv*Stat_k4_cv(ix+1, iv, a) + wx*(1-wv)*Stat_k4_cv(ix, iv+1, a) + (1-wx)*(1-wv)*Stat_k4_cv(ix+1, iv+1, a));
        
        rejection(1, a, 2) = mean(Stat_s0 > wx*wv*Stat_s0_cv(ix, iv, a) + (1-wx)*wv*Stat_s0_cv(ix+1, iv, a) + wx*(1-wv)*Stat_s0_cv(ix, iv+1, a) + (1-wx)*(1-wv)*Stat_s0_cv(ix+1, iv+1, a));
        rejection(2, a, 2) = mean(Stat_s1 > wx*wv*Stat_s1_cv(ix, iv, a) + (1-wx)*wv*Stat_s1_cv(ix+1, iv, a) + wx*(1-wv)*Stat_s1_cv(ix, iv+1, a) + (1-wx)*(1-wv)*Stat_s1_cv(ix+1, iv+1, a));
        rejection(3, a, 2) = mean(Stat_s2 > wx*wv*Stat_s2_cv(ix, iv, a) + (1-wx)*wv*Stat_s2_cv(ix+1, iv, a) + wx*(1-wv)*Stat_s2_cv(ix, iv+1, a) + (1-wx)*(1-wv)*Stat_s2_cv(ix+1, iv+1, a));
        for k = 1:K
            rejection(3+k, a, 2) = mean(Stat_s3(:, :, k) > wx*wv*Stat_s3_cv(ix, iv, a, k) + (1-wx)*wv*Stat_s3_cv(ix+1, iv, a, k) + wx*(1-wv)*Stat_s3_cv(ix, iv+1, a, k) + (1-wx)*(1-wv)*Stat_s3_cv(ix+1, iv+1, a, k));
        end
        rejection(K+4, a, 2) = mean(Stat_s4 > wx*wv*Stat_s4_cv(ix, iv, a) + (1-wx)*wv*Stat_s4_cv(ix+1, iv, a) + wx*(1-wv)*Stat_s4_cv(ix, iv+1, a) + (1-wx)*(1-wv)*Stat_s4_cv(ix+1, iv+1, a));
        
        rejection(1, a, 3) = mean(Stat_GH0 > wx*wv*Stat_GH0_cv(ix, iv, a) + (1-wx)*wv*Stat_GH0_cv(ix+1, iv, a) + wx*(1-wv)*Stat_GH0_cv(ix, iv+1, a) + (1-wx)*(1-wv)*Stat_GH0_cv(ix+1, iv+1, a));
        rejection(2, a, 3) = mean(Stat_GH1 > wx*wv*Stat_GH1_cv(ix, iv, a) + (1-wx)*wv*Stat_GH1_cv(ix+1, iv, a) + wx*(1-wv)*Stat_GH1_cv(ix, iv+1, a) + (1-wx)*(1-wv)*Stat_GH1_cv(ix+1, iv+1, a));
        rejection(3, a, 3) = mean(Stat_GH2 > wx*wv*Stat_GH2_cv(ix, iv, a) + (1-wx)*wv*Stat_GH2_cv(ix+1, iv, a) + wx*(1-wv)*Stat_GH2_cv(ix, iv+1, a) + (1-wx)*(1-wv)*Stat_GH2_cv(ix+1, iv+1, a));
        for k = 1:K
            rejection(3+k, a, 3) = mean(Stat_GH3(:, :, k) > wx*wv*Stat_GH3_cv(ix, iv, a, k) + (1-wx)*wv*Stat_GH3_cv(ix+1, iv, a, k) + wx*(1-wv)*Stat_GH3_cv(ix, iv+1, a, k) + (1-wx)*(1-wv)*Stat_GH3_cv(ix+1, iv+1, a, k));
        end
        rejection(K+4, a, 3) = mean(Stat_GH4 > wx*wv*Stat_GH4_cv(ix, iv, a) + (1-wx)*wv*Stat_GH4_cv(ix+1, iv, a) + wx*(1-wv)*Stat_GH4_cv(ix, iv+1, a) + (1-wx)*(1-wv)*Stat_GH4_cv(ix+1, iv+1, a));
        
        % True parameter values - KT
        rejectionKT(1, a, 1) = mean(StatKT_k0 > wx*wv*StatKT_k0_cv(ix, iv, a) + (1-wx)*wv*StatKT_k0_cv(ix+1, iv, a) + wx*(1-wv)*StatKT_k0_cv(ix, iv+1, a) + (1-wx)*(1-wv)*StatKT_k0_cv(ix+1, iv+1, a));
        rejectionKT(2, a, 1) = mean(StatKT_k1 > wx*wv*StatKT_k1_cv(ix, iv, a) + (1-wx)*wv*StatKT_k1_cv(ix+1, iv, a) + wx*(1-wv)*StatKT_k1_cv(ix, iv+1, a) + (1-wx)*(1-wv)*StatKT_k1_cv(ix+1, iv+1, a));
        rejectionKT(3, a, 1) = mean(StatKT_k2 > wx*wv*StatKT_k2_cv(ix, iv, a) + (1-wx)*wv*StatKT_k2_cv(ix+1, iv, a) + wx*(1-wv)*StatKT_k2_cv(ix, iv+1, a) + (1-wx)*(1-wv)*StatKT_k2_cv(ix+1, iv+1, a));
        for k = 1:K
            rejectionKT(3+k, a, 1) = mean(StatKT_k3(:, :, k) > wx*wv*StatKT_k3_cv(ix, iv, a, k) + (1-wx)*wv*StatKT_k3_cv(ix+1, iv, a, k) + wx*(1-wv)*StatKT_k3_cv(ix, iv+1, a, k) + (1-wx)*(1-wv)*StatKT_k3_cv(ix+1, iv+1, a, k));
        end
        rejectionKT(K+4, a, 1) = mean(StatKT_k4 > wx*wv*StatKT_k4_cv(ix, iv, a) + (1-wx)*wv*StatKT_k4_cv(ix+1, iv, a) + wx*(1-wv)*StatKT_k4_cv(ix, iv+1, a) + (1-wx)*(1-wv)*StatKT_k4_cv(ix+1, iv+1, a));
        
        rejectionKT(1, a, 2) = mean(StatKT_GH0 > wx*wv*StatKT_GH0_cv(ix, iv, a) + (1-wx)*wv*StatKT_GH0_cv(ix+1, iv, a) + wx*(1-wv)*StatKT_GH0_cv(ix, iv+1, a) + (1-wx)*(1-wv)*StatKT_GH0_cv(ix+1, iv+1, a));
        rejectionKT(2, a, 2) = mean(StatKT_GH1 > wx*wv*StatKT_GH1_cv(ix, iv, a) + (1-wx)*wv*StatKT_GH1_cv(ix+1, iv, a) + wx*(1-wv)*StatKT_GH1_cv(ix, iv+1, a) + (1-wx)*(1-wv)*StatKT_GH1_cv(ix+1, iv+1, a));
        rejectionKT(3, a, 2) = mean(StatKT_GH2 > wx*wv*StatKT_GH2_cv(ix, iv, a) + (1-wx)*wv*StatKT_GH2_cv(ix+1, iv, a) + wx*(1-wv)*StatKT_GH2_cv(ix, iv+1, a) + (1-wx)*(1-wv)*StatKT_GH2_cv(ix+1, iv+1, a));
        for k = 1:K
            rejectionKT(3+k, a, 2) = mean(StatKT_GH3(:, :, k) > wx*wv*StatKT_GH3_cv(ix, iv, a, k) + (1-wx)*wv*StatKT_GH3_cv(ix+1, iv, a, k) + wx*(1-wv)*StatKT_GH3_cv(ix, iv+1, a, k) + (1-wx)*(1-wv)*StatKT_GH3_cv(ix+1, iv+1, a, k));
        end
        rejectionKT(K+4, a, 2) = mean(StatKT_GH4 > wx*wv*StatKT_GH4_cv(ix, iv, a) + (1-wx)*wv*StatKT_GH4_cv(ix+1, iv, a) + wx*(1-wv)*StatKT_GH4_cv(ix, iv+1, a) + (1-wx)*(1-wv)*StatKT_GH4_cv(ix+1, iv+1, a));
        
        % True parameter values - HK
        for k = 1:K
            rejectionHK(k, a, 1) = mean(Stat_HKk3(:, :, k) > wx*wv*Stat_HKk3_cv(ix, iv, a) + (1-wx)*wv*Stat_HKk3_cv(ix+1, iv, a, k) + wx*(1-wv)*Stat_HKk3_cv(ix, iv+1, a, k) + (1-wx)*(1-wv)*Stat_HKk3_cv(ix+1, iv+1, a, k));
            rejectionHK(k, a, 2) = mean(Stat_HKs3(:, :, k) > wx*wv*Stat_HKs3_cv(ix, iv, a) + (1-wx)*wv*Stat_HKs3_cv(ix+1, iv, a, k) + wx*(1-wv)*Stat_HKs3_cv(ix, iv+1, a, k) + (1-wx)*(1-wv)*Stat_HKs3_cv(ix+1, iv+1, a, k));
            rejectionHK(k, a, 3) = mean(Stat_HKGH3(:, :, k) > wx*wv*Stat_HKGH3_cv(ix, iv, a) + (1-wx)*wv*Stat_HKGH3_cv(ix+1, iv, a, k) + wx*(1-wv)*Stat_HKGH3_cv(ix, iv+1, a, k) + (1-wx)*(1-wv)*Stat_HKGH3_cv(ix+1, iv+1, a, k));
        end
        
        % ML estimates
        rejection_ML(1, a, 1) = mean(Stat_k0_ML > wx_ML*wv_ML*Stat_k0_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*Stat_k0_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*Stat_k0_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*Stat_k0_cv(ix_ML+1, iv_ML+1, a));
        rejection_ML(2, a, 1) = mean(Stat_k1_ML > wx_ML*wv_ML*Stat_k1_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*Stat_k1_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*Stat_k1_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*Stat_k1_cv(ix_ML+1, iv_ML+1, a));
        rejection_ML(3, a, 1) = mean(Stat_k2_ML > wx_ML*wv_ML*Stat_k2_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*Stat_k2_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*Stat_k2_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*Stat_k2_cv(ix_ML+1, iv_ML+1, a));
        for k = 1:K
            rejection_ML(3+k, a, 1) = mean(Stat_k3_ML(:, :, k) > wx_ML*wv_ML*Stat_k3_cv(ix_ML, iv_ML, a, k) + (1-wx_ML)*wv_ML*Stat_k3_cv(ix_ML+1, iv_ML, a, k) + wx_ML*(1-wv_ML)*Stat_k3_cv(ix_ML, iv_ML+1, a, k) + (1-wx_ML)*(1-wv_ML)*Stat_k3_cv(ix_ML+1, iv_ML+1, a, k));
        end
        rejection_ML(K+4, a, 1) = mean(Stat_k4_ML > wx_ML*wv_ML*Stat_k4_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*Stat_k4_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*Stat_k4_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*Stat_k4_cv(ix_ML+1, iv_ML+1, a));
        
        rejection_ML(1, a, 2) = mean(Stat_s0_ML > wx_ML*wv_ML*Stat_s0_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*Stat_s0_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*Stat_s0_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*Stat_s0_cv(ix_ML+1, iv_ML+1, a));
        rejection_ML(2, a, 2) = mean(Stat_s1_ML > wx_ML*wv_ML*Stat_s1_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*Stat_s1_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*Stat_s1_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*Stat_s1_cv(ix_ML+1, iv_ML+1, a));
        rejection_ML(3, a, 2) = mean(Stat_s2_ML > wx_ML*wv_ML*Stat_s2_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*Stat_s2_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*Stat_s2_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*Stat_s2_cv(ix_ML+1, iv_ML+1, a));
        for k = 1:K
            rejection_ML(3+k, a, 2) = mean(Stat_s3_ML(:, :, k) > wx_ML*wv_ML*Stat_s3_cv(ix_ML, iv_ML, a, k) + (1-wx_ML)*wv_ML*Stat_s3_cv(ix_ML+1, iv_ML, a, k) + wx_ML*(1-wv_ML)*Stat_s3_cv(ix_ML, iv_ML+1, a, k) + (1-wx_ML)*(1-wv_ML)*Stat_s3_cv(ix_ML+1, iv_ML+1, a, k));
        end
        rejection_ML(K+4, a, 2) = mean(Stat_s4_ML > wx_ML*wv_ML*Stat_s4_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*Stat_s4_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*Stat_s4_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*Stat_s4_cv(ix_ML+1, iv_ML+1, a));
        
        rejection_ML(1, a, 3) = mean(Stat_GH0_ML > wx_ML*wv_ML*Stat_GH0_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*Stat_GH0_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*Stat_GH0_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*Stat_GH0_cv(ix_ML+1, iv_ML+1, a));
        rejection_ML(2, a, 3) = mean(Stat_GH1_ML > wx_ML*wv_ML*Stat_GH1_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*Stat_GH1_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*Stat_GH1_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*Stat_GH1_cv(ix_ML+1, iv_ML+1, a));
        rejection_ML(3, a, 3) = mean(Stat_GH2_ML > wx_ML*wv_ML*Stat_GH2_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*Stat_GH2_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*Stat_GH2_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*Stat_GH2_cv(ix_ML+1, iv_ML+1, a));
        for k = 1:K
            rejection_ML(3+k, a, 3) = mean(Stat_GH3_ML(:, :, k) > wx_ML*wv_ML*Stat_GH3_cv(ix_ML, iv_ML, a, k) + (1-wx_ML)*wv_ML*Stat_GH3_cv(ix_ML+1, iv_ML, a, k) + wx_ML*(1-wv_ML)*Stat_GH3_cv(ix_ML, iv_ML+1, a, k) + (1-wx_ML)*(1-wv_ML)*Stat_GH3_cv(ix_ML+1, iv_ML+1, a, k));
        end
        rejection_ML(K+4, a, 3) = mean(Stat_GH4_ML > wx_ML*wv_ML*Stat_GH4_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*Stat_GH4_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*Stat_GH4_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*Stat_GH4_cv(ix_ML+1, iv_ML+1, a));
        
        % ML estimates - KT
        rejectionKT_ML(1, a, 1) = mean(StatKT_k0_ML > wx_ML*wv_ML*StatKT_k0_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*StatKT_k0_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*StatKT_k0_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*StatKT_k0_cv(ix_ML+1, iv_ML+1, a));
        rejectionKT_ML(2, a, 1) = mean(StatKT_k1_ML > wx_ML*wv_ML*StatKT_k1_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*StatKT_k1_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*StatKT_k1_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*StatKT_k1_cv(ix_ML+1, iv_ML+1, a));
        rejectionKT_ML(3, a, 1) = mean(StatKT_k2_ML > wx_ML*wv_ML*StatKT_k2_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*StatKT_k2_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*StatKT_k2_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*StatKT_k2_cv(ix_ML+1, iv_ML+1, a));
        for k = 1:K
            rejectionKT_ML(3+k, a, 1) = mean(StatKT_k3_ML(:, :, k) > wx_ML*wv_ML*StatKT_k3_cv(ix_ML, iv_ML, a, k) + (1-wx_ML)*wv_ML*StatKT_k3_cv(ix_ML+1, iv_ML, a, k) + wx_ML*(1-wv_ML)*StatKT_k3_cv(ix_ML, iv_ML+1, a, k) + (1-wx_ML)*(1-wv_ML)*StatKT_k3_cv(ix_ML+1, iv_ML+1, a, k));
        end
        rejectionKT_ML(K+4, a, 1) = mean(StatKT_k4_ML > wx_ML*wv_ML*StatKT_k4_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*StatKT_k4_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*StatKT_k4_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*StatKT_k4_cv(ix_ML+1, iv_ML+1, a));
        
        rejectionKT_ML(1, a, 2) = mean(StatKT_GH0_ML > wx_ML*wv_ML*StatKT_GH0_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*StatKT_GH0_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*StatKT_GH0_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*StatKT_GH0_cv(ix_ML+1, iv_ML+1, a));
        rejectionKT_ML(2, a, 2) = mean(StatKT_GH1_ML > wx_ML*wv_ML*StatKT_GH1_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*StatKT_GH1_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*StatKT_GH1_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*StatKT_GH1_cv(ix_ML+1, iv_ML+1, a));
        rejectionKT_ML(3, a, 2) = mean(StatKT_GH2_ML > wx_ML*wv_ML*StatKT_GH2_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*StatKT_GH2_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*StatKT_GH2_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*StatKT_GH2_cv(ix_ML+1, iv_ML+1, a));
        for k = 1:K
            rejectionKT_ML(3+k, a, 2) = mean(StatKT_GH3_ML(:, :, k) > wx_ML*wv_ML*StatKT_GH3_cv(ix_ML, iv_ML, a, k) + (1-wx_ML)*wv_ML*StatKT_GH3_cv(ix_ML+1, iv_ML, a, k) + wx_ML*(1-wv_ML)*StatKT_GH3_cv(ix_ML, iv_ML+1, a, k) + (1-wx_ML)*(1-wv_ML)*StatKT_GH3_cv(ix_ML+1, iv_ML+1, a, k));
        end
        rejectionKT_ML(K+4, a, 2) = mean(StatKT_GH4_ML > wx_ML*wv_ML*StatKT_GH4_cv(ix_ML, iv_ML, a) + (1-wx_ML)*wv_ML*StatKT_GH4_cv(ix_ML+1, iv_ML, a) + wx_ML*(1-wv_ML)*StatKT_GH4_cv(ix_ML, iv_ML+1, a) + (1-wx_ML)*(1-wv_ML)*StatKT_GH4_cv(ix_ML+1, iv_ML+1, a));
        
        % MLestimates - HK
        for k = 1:K
            rejectionHK_ML(k, a, 1) = mean(Stat_HKk3_ML(:, :, k) > wx_ML*wv_ML*Stat_HKk3_cv(ix_ML, iv_ML, a, k) + (1-wx_ML)*wv_ML*Stat_HKk3_cv(ix_ML+1, iv_ML, a, k) + wx_ML*(1-wv_ML)*Stat_HKk3_cv(ix_ML, iv_ML+1, a, k) + (1-wx_ML)*(1-wv_ML)*Stat_HKk3_cv(ix_ML+1, iv_ML+1, a, k));
            rejectionHK_ML(k, a, 2) = mean(Stat_HKs3_ML(:, :, k) > wx_ML*wv_ML*Stat_HKs3_cv(ix_ML, iv_ML, a, k) + (1-wx_ML)*wv_ML*Stat_HKs3_cv(ix_ML+1, iv_ML, a, k) + wx_ML*(1-wv_ML)*Stat_HKs3_cv(ix_ML, iv_ML+1, a, k) + (1-wx_ML)*(1-wv_ML)*Stat_HKs3_cv(ix_ML+1, iv_ML+1, a, k));
            rejectionHK_ML(k, a, 3) = mean(Stat_HKGH3_ML(:, :, k) > wx_ML*wv_ML*Stat_HKGH3_cv(ix_ML, iv_ML, a, k) + (1-wx_ML)*wv_ML*Stat_HKGH3_cv(ix_ML+1, iv_ML, a, k) + wx_ML*(1-wv_ML)*Stat_HKGH3_cv(ix_ML, iv_ML+1, a, k) + (1-wx_ML)*(1-wv_ML)*Stat_HKGH3_cv(ix_ML+1, iv_ML+1, a, k));
        end
        
    end
    
    waitbar(s/S, h)
    
    % Saving results
    if (rem(s, safe) == 0 || s == S)
        save btest.mat s ...
            rejection rejection_ML rejectionKT rejectionKT_ML ...
            rejectionHK rejectionHK_ML
    end

end
close(h)

movefile('btest.mat',['IO/bootstrap/', title,'_bootstrap_test_T',num2str(T),...
    '_H',num2str(hypothesis),'.mat']);