function s = ScoreFK(e,V)
% DESCRIPTION
% This function computes the kurtosis component of the filtered score with
% respect to the shape parameters.
%
% INPUT
% e = [e_(1|0) ... e_(T|T-1)] is (N,T) and contains filtered errors, all
% non-Gaussian under the alternative, e_(t|t-1) = LSP[e_t|y_1 ... y_(t-1)],
% V = [V_(1|0) ... V_(T|T-1)] is (N,N,T) and contains the filter
% mean squared errors, V_(t|t-1) = E[(e_t-e_(t|t-1))(e_t-e_(t|t-1))'].
%
% OUTPUT
% s = [s_(1|0) ... s_(T|T-1)] is (1,T) and contains filtered contributions
% to the kurtosis component of the score with respect to shape parameters.
%
% INFORMATION
% Authors: Almuzara, Amengual, Sentana.
% Date: January 2016.


%% 01. Settings and pre-allocation

[N,T] = size(e);

c0 = N*(N+2)/4;
c1 = -(N+2)/2;
c2 = 1/4;

s = zeros(1,T);


%% 02. Computation of the filtered contributions to the kurtosis-score

for t = 1:T
    aux_sigma = e(:,t)'*pinv(V(:,:,t))*e(:,t);
    s(1,t) = c0 + c1*aux_sigma + c2*aux_sigma^2;
end