function [w_grid_1, L_grid_1] = ARMS_hull(w_grid_0, L_grid_0)
% This function computes the parameters necessary for linear interpolation
% of the hull function appearing in the adaptive rejection Metropolis
% sampling method of Gilks, Best & Tan (1995, JRSS)
%
% INPUTS:
% - w_grid_0 is the original grid for the argument
% - L_grid_0 is the original grid for the log-density at each argument
%
% OUTPUT:
% - w_grid_1 is the new grid of arguments
% - L_grid_1 is the new grid of log-densities
%
% AUTHOR: Tincho Almuzara, DATE: 22-Dec-2017

n_grid = max(size(w_grid_0));
w_grid_0 = reshape(w_grid_0, [n_grid, 1]);
L_grid_0 = reshape(L_grid_0, [n_grid, 1]);

[w_grid_0, order] = sort(w_grid_0);
L_grid_0 = L_grid_0(order);

d = (L_grid_0(2:end)-L_grid_0(1:(end-1))) ./ (w_grid_0(2:end)-w_grid_0(1:(end-1)));
test_d = (d(2:(end-1)) < d(1:(end-2))) .* (d(3:(end)) < d(2:(end-1)));
points_d = (L_grid_0(3:(end-1)) - L_grid_0(1:(end-3))...
            - (d(3:(end)) .* w_grid_0(3:(end-1))) ...
            + (d(1:(end-2)) .* w_grid_0(1:(end-3)))) ...
            ./ (d(1:(end-2)) - d(3:(end)));
L_points_d = L_grid_0(1:(end-3)) + d(1:(end-2)) .* (points_d - w_grid_0(1:(end-3)));       

[w_grid_1, order] = sort([w_grid_0; points_d(test_d==1)]);
L_grid_1 = [L_grid_0; L_points_d(test_d==1)];
L_grid_1 = L_grid_1(order);



