function [epsilon_0, zeta_0] = Asym_t_rnd(param, K, T)
% Inputs: [param,N,T,dist]
%       param are the parameters that characterize the distribution
%       N is the cross sectional dimension of the model (scalar)
%       T is the times/series dimension of the model (scalar)
%
% Output: epsilon, a T x N vector of unit variance
% Author: Julio Crego
% I modified the SqRR matrix and replaced the r argument by N.
% This function needs to be tested for N >= 3.

tolerance_b = 1e-3;

nu_sh = param(1);   
b_sh = reshape(param(2:K+1), [K, 1]);
if (b_sh'*b_sh < tolerance_b) 
    a_sh = 1;
    b_sh = zeros(K, 1);
    Upsilon_sh = (nu_sh-2)*eye(K);
else
    a_sh = (-(nu_sh-4) + sqrt((nu_sh-4)^2+8*(nu_sh-4)*(b_sh'*b_sh)))/(4*(b_sh'*b_sh));
    Upsilon_sh = (nu_sh-2)*(eye(K) + (a_sh-1)/(b_sh'*b_sh)*(b_sh*b_sh'));
end
Upsilon_sqrt_sh = chol(Upsilon_sh, 'lower');

Zt = normrnd(0, 1, K, T); 
zeta_0 = gamrnd(nu_sh/2, 2, 1, T);
epsilon_0 = (-a_sh*b_sh)*ones(1, T) + (Upsilon_sh*b_sh)./zeta_0 + ...
             Upsilon_sqrt_sh*Zt./sqrt(zeta_0);