function C_draw = C_sampler(theta_mv_0, xi_0, theta_sh_0, zeta_0)

% Default settings
K = 3; T = size(xi_0, 2);
tolerance_b = 1e-3;

% Parameters
F_mv = [1+theta_mv_0(3), -theta_mv_0(3), 0, 0; ...
        1, 0, 0, 0; ...
        0, 0, theta_mv_0(4), 0; ...
        0, 0, 0, theta_mv_0(5)];
G_mv = [sqrt(theta_mv_0(6)), 0, 0; ...
        0, 0, 0; ...
        0, sqrt(theta_mv_0(7)), 0; ...
        0, 0, sqrt(theta_mv_0(8))];        

nu_sh = theta_sh_0(1);
b_sh = theta_sh_0(2:4);
if (b_sh'*b_sh < tolerance_b) 
    a_sh = 1;
    b_sh = zeros(K, 1);
    Upsilon_sh = (nu_sh-2)*eye(K);
else
    a_sh = (-(nu_sh-4) + sqrt((nu_sh-4)^2+8*(nu_sh-4)*(b_sh'*b_sh)))/(4*(b_sh'*b_sh));
    Upsilon_sh = (nu_sh-2)*(eye(K) + (a_sh-1)/(b_sh'*b_sh)*(b_sh*b_sh'));
end

% Prior hyperparameters
C_prior = [0.75; ...
           0.00];
Varinv_C_prior = [1/2.5, 0.0; ...
                  0.0, 1/25.0];

% Posterior draw
DD_C = [1/(1+F_mv(1, 2)), 0, 0, 0; ...
        0, 0, 1/(1-F_mv(3, 3)), -1/(1-F_mv(4, 4))];
YY_C = (DD_C*(xi_0(:, 2:end) - F_mv*xi_0(:, 1:(end-1)) ...
              - G_mv*(Upsilon_sh*b_sh./zeta_0 - a_sh*b_sh*ones(1, T-1)))) ...
        .* sqrt(zeta_0);  
XX_C = sqrt(zeta_0);

C_hat = (YY_C*(XX_C'))/(XX_C*(XX_C'));
Varinv_C_hat = (XX_C*(XX_C'))*pinv(DD_C*G_mv*Upsilon_sh*(G_mv')*DD_C');

Varinv_C_posterior = Varinv_C_prior + Varinv_C_hat; 
C_posterior = pinv(Varinv_C_posterior)*(Varinv_C_prior*C_prior + Varinv_C_hat*C_hat);
Std_C_posterior = pinv(chol(Varinv_C_posterior, 'lower'));
C_draw = C_posterior + Std_C_posterior*normrnd(0, 1, K-1, 1);