function F_draw = F_sampler(theta_mv_0, xi_0, theta_sh_0, zeta_0)

% Default settings
K = 3; T = size(xi_0, 2);
tolerance_b = 1e-3; barrier_rho = 0.99;

% Parameters
G_mv = [sqrt(theta_mv_0(6)), 0, 0; ...
        0, 0, 0; ...
        0, sqrt(theta_mv_0(7)), 0; ...
        0, 0, sqrt(theta_mv_0(8))];        

nu_sh = theta_sh_0(1);
b_sh = reshape(theta_sh_0(2:4), [3, 1]);
if (b_sh'*b_sh < tolerance_b) 
    a_sh = 1;
    b_sh = zeros(K, 1);
    Upsilon_sh = (nu_sh-2)*eye(K);
else
    a_sh = (-(nu_sh-4) + sqrt((nu_sh-4)^2+8*(nu_sh-4)*(b_sh'*b_sh)))/(4*(b_sh'*b_sh));
    Upsilon_sh = (nu_sh-2)*(eye(K) + (a_sh-1)/(b_sh'*b_sh)*(b_sh*b_sh'));
end
Upsilon_sqrt_sh = chol(Upsilon_sh, 'lower');

% Prior hyperparameters
F_prior = [0.5; ...
           0.0; ...
           0.0];
Varinv_F_prior = [0.0, 0.0, 0.0; ...
                  0.0, 0.0, 0.0; ...
                  0.0, 0.0, 0.0];       
       
% Posterior draw
DD_F = [1, -1, 0, 0; ...
        0, 0, 1, 0; ...
        0, 0, 0, 1];
WW_F = pinv(DD_F*G_mv*Upsilon_sqrt_sh);    
YY_F = WW_F * ((DD_F*(xi_0(:, 2:end) - G_mv*(Upsilon_sh*b_sh./zeta_0 - a_sh*b_sh*ones(1, T-1))) ...
                      - [theta_mv_0(1); theta_mv_0(2)/2; -theta_mv_0(2)/2]*ones(1, T-1)) ...
              .* sqrt(zeta_0));
XX_F = WW_F * ((DD_F*xi_0(:, 1:(end-1)) - [theta_mv_0(1); theta_mv_0(2)/2; -theta_mv_0(2)/2]*ones(1, T-1)) ...
              .* sqrt(zeta_0));
                    
F_hat = [(YY_F(1, :)*(XX_F(1, :)'))/(XX_F(1, :)*(XX_F(1, :)')); ...
         (YY_F(2, :)*(XX_F(2, :)'))/(XX_F(2, :)*(XX_F(2, :)')); ...
         (YY_F(3, :)*(XX_F(3, :)'))/(XX_F(3, :)*(XX_F(3, :)'))];
Varinv_F_hat = diag([(XX_F(1, :)*(XX_F(1, :)')); ...
                     (XX_F(2, :)*(XX_F(2, :)')); ...
                     (XX_F(3, :)*(XX_F(3, :)'))]);   
              
Varinv_F_posterior = Varinv_F_prior + Varinv_F_hat; 
F_posterior = pinv(Varinv_F_posterior)*(Varinv_F_prior*F_prior + Varinv_F_hat*F_hat);
Std_F_posterior = pinv(chol(Varinv_F_posterior, 'lower'));
F_draw = max(min(F_posterior + Std_F_posterior*normrnd(0, 1, K, 1), barrier_rho), -barrier_rho);