function [G_draw, rejection] = G_sampler(theta_mv_0, xi_0, theta_sh_0, zeta_0, method)

% Default settings
K = 3; T = size(xi_0, 2);
grid_min = 0.02; grid_max = 0.98; tolerance_b = 1e-3;

% Parameters
C_mv = [(1-theta_mv_0(3))*theta_mv_0(1); ...
        0; ...
        (1-theta_mv_0(4))*theta_mv_0(2)/2; ...
        -(1-theta_mv_0(5))*theta_mv_0(2)/2];
F_mv = [1+theta_mv_0(3), -theta_mv_0(3), 0, 0; ...
        1, 0, 0, 0; ...
        0, 0, theta_mv_0(4), 0; ...
        0, 0, 0, theta_mv_0(5)];

nu_sh = theta_sh_0(1);
b_sh = theta_sh_0(2:4);

% Prior hyperparameters
G_degree = [5; ...
            5; ...
            5];
G_scale = [5*0.5; ...
           5*0.1; ...
           5*0.1];       

% Posterior draw
DD_G = [1, -1, 0, 0; ...
        0, 0, 1, 0; ...
        0, 0, 0, 1];
eta_0 = ((DD_G*(xi_0(:, 2:end) - C_mv - F_mv*xi_0(:, 1:(T-1)))) ...
         .* sqrt(zeta_0));     
G_hat = (eta_0*(eta_0'));

if (b_sh'*b_sh < tolerance_b)
    if (strcmp(method, 'ARMS'))
        rejection = zeros(3, 2);
    else
        rejection = zeros(3, 3);
    end
    G_draw = ((1/(nu_sh-2))*diag(G_hat)+G_scale) ...
             ./ [gamrnd((T+G_degree(1)-1)/2-1, 2, 1, 1); ...
                 gamrnd((T+G_degree(2)-1)/2-1, 2, 1, 1); ...
                 gamrnd((T+G_degree(3)-1)/2-1, 2, 1, 1)];
else
    if (strcmp(method, 'ARMS'))
    % Extra settings
    S_ARMS = 10; N_ARS = 15; N_ARS_add = 4*S_ARMS; 

    % Posterior draws    
    rejection = zeros(3, 2);
    for j = 1:3
        log_posterior = log_posterior_mv(theta_mv_0, theta_sh_0, eta_0, zeta_0, G_degree, G_scale);

        % The grid is constructed under the transformation gamma |-> exp(-3gamma)
        % (log-posterior includes a term for flat prior)
        theta_mv_aux = theta_mv_0;
        gamma_grid = zeros(N_ARS+N_ARS_add, 1);
        L_gamma_grid = zeros(N_ARS+N_ARS_add, 1);
        gamma_grid(1:N_ARS) = linspace(grid_min, grid_max, N_ARS)';    
        for n_ARS = 1:N_ARS
            theta_mv_aux(5+j) = -(1/3)*log(gamma_grid(n_ARS));
            L_gamma_grid(n_ARS) = log_posterior_mv(theta_mv_aux, theta_sh_0, eta_0, zeta_0, G_degree, G_scale);
        end
        [gamma_ARS, h_gamma_ARS] = ARMS_pwexp_draw(gamma_grid(1:N_ARS), L_gamma_grid(1:N_ARS));
        theta_mv_aux(5+j) = -(1/3)*log(gamma_ARS);
        log_posterior_aux = log_posterior_mv(theta_mv_aux, theta_sh_0, eta_0, zeta_0, G_degree, G_scale);
    
        for s_ARMS = 1:S_ARMS

            while (log(rand(1, 1)) > log_posterior_aux-h_gamma_ARS) && (rejection(j, 1) < N_ARS_add)
                rejection(j, 1) = rejection(j, 1)+1;            
                N_ARS_aux = N_ARS+rejection(j, 1);
                gamma_grid(N_ARS_aux) = gamma_ARS;
                L_gamma_grid(N_ARS_aux) = log_posterior_aux;
                [gamma_ARS, h_gamma_ARS] = ARMS_pwexp_draw(gamma_grid(1:N_ARS_aux), L_gamma_grid(1:N_ARS_aux));
                theta_mv_aux(5+j) = -(1/3)*log(gamma_ARS);
                log_posterior_aux = log_posterior_mv(theta_mv_aux, theta_sh_0, eta_0, zeta_0, G_degree, G_scale);
            end
    
            if (log(rand(1, 1)) < log_posterior_aux-log_posterior)
                theta_mv_0 = theta_mv_aux;
                log_posterior = log_posterior_aux;
            else
                rejection(j, 2) = rejection(j, 2)+1;
            end  
        end
    end
    rejection(:, 1) = rejection(:, 1)/S_ARMS; rejection(:, 2) = rejection(:, 2)/S_ARMS;
    G_draw = theta_mv_0(6:8);
    
    else
        % Extra settings
        w_SLICE = 1/50; m_SLICE = 10; 
        
        % Posterio draws
        rejection = zeros(3, 3);
        log_posterior_0 = log_posterior_mv(theta_mv_0, theta_sh_0, eta_0, zeta_0, G_degree, G_scale);
        for j = 1:3
            e_0 = -log(rand(1, 1));

            m_LL = floor(rand(1, 1)*m_SLICE);
            m_RR = m_SLICE-1-m_LL;
        
            % The steps are defined under the transformation gamma |-> exp(-3gamma)
            gamma_0 = exp(-3*theta_mv_0(5+j));
            LL = max(gamma_0-rand(1, 1)*w_SLICE, grid_min);
            RR = min(LL+w_SLICE, grid_max);
        
            theta_mv_LL = theta_mv_0;
            theta_mv_RR = theta_mv_0;
            theta_mv_LL(5+j) = -(1/3)*log(LL);
            theta_mv_RR(5+j) = -(1/3)*log(RR);
            L_gamma_LL = log_posterior_mv(theta_mv_LL, theta_sh_0, eta_0, zeta_0, G_degree, G_scale);
            L_gamma_RR = log_posterior_mv(theta_mv_RR, theta_sh_0, eta_0, zeta_0, G_degree, G_scale);

            while (L_gamma_LL > log_posterior_0-e_0) && (m_LL > 0)
                rejection(j, 1) = rejection(j, 1)+1;
                m_LL = m_LL-1;
                LL = max(LL-w_SLICE, grid_min);
                theta_mv_LL(5+j) = -(1/3)*log(LL);
                L_gamma_LL = log_posterior_mv(theta_mv_LL, theta_sh_0, eta_0, zeta_0, G_degree, G_scale);
            end    
    
            while (L_gamma_RR > log_posterior_0-e_0) && (m_RR > 0)
                rejection(j, 2) = rejection(j, 2)+1;
                m_RR = m_RR-1;
                RR = min(RR+w_SLICE, grid_max);
                theta_mv_RR(5+j) = -(1/3)*log(RR);
                L_gamma_RR = log_posterior_mv(theta_mv_RR, theta_sh_0, eta_0, zeta_0, G_degree, G_scale);
            end
        
            gamma_1 = LL+rand(1, 1)*(RR-LL);
            theta_mv_0(5+j) = -(1/3)*log(gamma_1);
            log_posterior_1 = log_posterior_mv(theta_mv_0, theta_sh_0, eta_0, zeta_0, G_degree, G_scale);
    
            while (log_posterior_1 < log_posterior_0-e_0)
                rejection(j, 3) = rejection(j, 3)+1;
                if (gamma_1 < gamma_0)
                    LL = gamma_1;
                else
                    RR = gamma_1;
                end
        
                gamma_1 = LL+rand(1, 1)*(RR-LL);
                theta_mv_0(5+j) = -(1/3)*log(gamma_1);
                log_posterior_1 = log_posterior_mv(theta_mv_0, theta_sh_0, eta_0, zeta_0, G_degree, G_scale);
            end
            log_posterior_0 = log_posterior_1;    
        end
        G_draw = theta_mv_0(6:8);
    
    end
end