function [theta_sh_0, rejection] = SH_sampler(theta_sh_0, epsilon_0, zeta_0, method, symmetry)

% Default settings
grid_min = 0.02; grid_max = 0.98; nu_min = 4;

if (symmetry == 0)
    J = 1;
    theta_sh_0(2:4) = zeros(3, 1);
else
    J = 4;
end

if (strcmp(method, 'ARMS'))
    % Extra settings
    S_ARMS = 10; N_ARS = 15; N_ARS_add = 4*S_ARMS; 
    
    % Posterior draws
    rejection = zeros(4, 2);
    for j = 1:J
        log_posterior = log_posterior_sh(theta_sh_0, epsilon_0, zeta_0);

        % The grid is constructed under the transformations nu |-> nu_min/nu
        % and beta |-> exp(beta)/(1+exp(beta))
        % (log-posterior includes a term for flat prior)
        theta_sh_aux = theta_sh_0;
        shape_grid = zeros(N_ARS+N_ARS_add, 1);
        L_shape_grid = zeros(N_ARS+N_ARS_add, 1);
        shape_grid(1:N_ARS) = linspace(grid_min, grid_max, N_ARS)';    
        for n_ARS = 1:N_ARS
            if (j == 1)
                theta_sh_aux(j) = nu_min/shape_grid(n_ARS);
            else
                theta_sh_aux(j) = log(shape_grid(n_ARS)/(1-shape_grid(n_ARS)));            
            end
            L_shape_grid(n_ARS) = log_posterior_sh(theta_sh_aux, epsilon_0, zeta_0);
        end
        [shape_ARS, h_shape_ARS] = ARMS_pwexp_draw(shape_grid(1:N_ARS), L_shape_grid(1:N_ARS));
        if (j == 1)
            theta_sh_aux(j) = nu_min/shape_ARS;
        else
            theta_sh_aux(j) = log(shape_ARS/(1-shape_ARS));                            
        end
        log_posterior_aux = log_posterior_sh(theta_sh_aux, epsilon_0, zeta_0);
    
        for s_ARMS = 1:S_ARMS

            while (log(rand(1, 1)) > log_posterior_aux-h_shape_ARS) && (rejection(j, 1) < N_ARS_add)
                rejection(j, 1) = rejection(j, 1)+1;            
                N_ARS_aux = N_ARS+rejection(j, 1);
                shape_grid(N_ARS_aux) = shape_ARS;
                L_shape_grid(N_ARS_aux) = log_posterior_aux;
                [shape_ARS, h_shape_ARS] = ARMS_pwexp_draw(shape_grid(1:N_ARS_aux), L_shape_grid(1:N_ARS_aux));
                if (j == 1)
                    theta_sh_aux(j) = nu_min/shape_ARS;
                else
                    theta_sh_aux(j) = log(shape_ARS/(1-shape_ARS));                            
                end
                log_posterior_aux = log_posterior_sh(theta_sh_aux, epsilon_0, zeta_0);            
            end    
                    
            if (log(rand(1, 1)) < log_posterior_aux-log_posterior)
                theta_sh_0 = theta_sh_aux;
                log_posterior = log_posterior_aux;
            else
                rejection(j, 2) = rejection(j, 2)+1;
            end             
        end
    end
    rejection(:, 1) = rejection(:, 1)/S_ARMS; rejection(:, 2) = rejection(:, 2)/S_ARMS;
    
else
    % Extra settings
    w_SLICE = 1/50; m_SLICE = 10; 

    % Posterior draws
    rejection = zeros(4, 3);
    log_posterior_0 = log_posterior_sh(theta_sh_0, epsilon_0, zeta_0);
    for j = 1:J
        e_0 = -log(rand(1, 1));
        m_LL = floor(rand(1, 1)*m_SLICE);
        m_RR = m_SLICE-1-m_LL;
    
        % The steps are defined under the transformations nu |-> nu_min/nu
        % and beta |-> exp(beta)/(1+exp(beta))
        if (j == 1)
            shape_0 = nu_min/theta_sh_0(j);
        else
            shape_0 = 1/(1+exp(-theta_sh_0(j)));
        end
        LL = max(shape_0-rand(1, 1)*w_SLICE, grid_min);
        RR = min(LL+w_SLICE, grid_max);

        theta_sh_LL = theta_sh_0;
        theta_sh_RR = theta_sh_0;
        if (j == 1)
            theta_sh_LL(j) = nu_min/LL;
            theta_sh_RR(j) = nu_min/RR;
        else
            theta_sh_LL(j) = log(LL/(1-LL));
            theta_sh_RR(j) = log(RR/(1-RR));
        end
        L_shape_LL = log_posterior_sh(theta_sh_LL, epsilon_0, zeta_0);
        L_shape_RR = log_posterior_sh(theta_sh_RR, epsilon_0, zeta_0);
    
        while (L_shape_LL > log_posterior_0-e_0) && (m_LL > 0)
            rejection(j, 1) = rejection(j, 1)+1;
            m_LL = m_LL-1;
            LL = max(LL-w_SLICE, grid_min);
            if (j == 1)
                theta_sh_LL(j) = nu_min/LL;
            else
                theta_sh_LL(j) = log(LL/(1-LL));
            end
            L_shape_LL = log_posterior_sh(theta_sh_LL, epsilon_0, zeta_0);
        end    
    
        while (L_shape_RR > log_posterior_0-e_0) && (m_RR > 0)
            rejection(j, 2) = rejection(j, 2)+1;
            m_RR = m_RR-1;
            RR = min(RR+w_SLICE, grid_max);
            if (j == 1)
                theta_sh_RR(j) = nu_min/RR;
            else
                theta_sh_RR(j) = log(RR/(1-RR));
            end
            L_shape_RR = log_posterior_sh(theta_sh_RR, epsilon_0, zeta_0);
        end    
    
        shape_1 = LL+rand(1, 1)*(RR-LL);
        if (j == 1)
            theta_sh_0(j) = nu_min/shape_1;
        else
            theta_sh_0(j) = log(shape_1/(1-shape_1));
        end
        log_posterior_1 = log_posterior_sh(theta_sh_0, epsilon_0, zeta_0);
    
        while (log_posterior_1 < log_posterior_0-e_0)
            rejection(j, 3) = rejection(j, 3)+1;
            if (shape_1 < shape_0)
                LL = shape_1;
            else
                RR = shape_1;
            end
        
            shape_1 = LL+rand(1, 1)*(RR-LL);
            if (j == 1)
                theta_sh_0(j) = nu_min/shape_1;
            else
                theta_sh_0(j) = log(shape_1/(1-shape_1));
            end
            log_posterior_1 = log_posterior_sh(theta_sh_0, epsilon_0, zeta_0);
        end
        log_posterior_0 = log_posterior_1;
    end   
    
end