function [xi_0, epsilon_0] = SS_sampler(theta_mv_0, theta_sh_0, zeta_0, Y)

% Default settings
N = 2; M = 4; K = 3; T = size(Y, 2);
kappa = 1e3; tolerance_b = 1e-3;

% Parameters
C_mv = [(1-theta_mv_0(3))*theta_mv_0(1); ...
        0; ...
        (1-theta_mv_0(4))*theta_mv_0(2)/2; ...
       -(1-theta_mv_0(5))*theta_mv_0(2)/2];
F_mv = [1+theta_mv_0(3), -theta_mv_0(3), 0, 0; ...
        1, 0, 0, 0; ...
        0, 0, theta_mv_0(4), 0; ...
        0, 0, 0, theta_mv_0(5)];
G_mv = [sqrt(theta_mv_0(6)), 0, 0; ...
        0, 0, 0; ...
        0, sqrt(theta_mv_0(7)), 0; ...
        0, 0, sqrt(theta_mv_0(8))];
H = [ones(2,1), zeros(2,1), eye(2)];   
Ex = zeros(M, 1);
Vx_mv = [(theta_mv_0(6)/(1-theta_mv_0(3)^2))*kappa*ones(2, 2), zeros(2,2);...
         zeros(2,2), ...
         diag([theta_mv_0(7)/(1-theta_mv_0(4)^2),...
               theta_mv_0(8)/(1-theta_mv_0(5)^2)])];

nu_sh = theta_sh_0(1);
b_sh = theta_sh_0(2:4);
if (b_sh'*b_sh < tolerance_b) 
    a_sh = 1;
    b_sh = zeros(K, 1);
    Upsilon_sh = (nu_sh-2)*eye(K);
else
    a_sh = (-(nu_sh-4) + sqrt((nu_sh-4)^2+8*(nu_sh-4)*(b_sh'*b_sh)))/(4*(b_sh'*b_sh));
    Upsilon_sh = (nu_sh-2)*(eye(K) + (a_sh-1)/(b_sh'*b_sh)*(b_sh*b_sh'));
end               
Upsilon_sqrt_sh = chol(Upsilon_sh, 'lower');    

% (i) Simulation
X_plus = zeros(M, T);  % This is {x_t^+}_{t=1}^T in the notes
Y_plus = zeros(N, T);  % This is {y_t^+}_{t=1}^T in the notes

X_plus(:, 1) = Ex;
Y_plus(:, 1) = Y(:, 1) - H*X_plus(:, 1);         
for t = 2:T
    X_plus(:, t) = (C_mv + G_mv*((1/zeta_0(1, t-1))*Upsilon_sh*b_sh - a_sh*b_sh)) ...
                   + F_mv*X_plus(:, t-1) ...
                   + sqrt((1/zeta_0(1, t-1)))*G_mv*Upsilon_sqrt_sh*normrnd(0, 1, K, 1); 
    Y_plus(:, t) = Y(:, t) - H*X_plus(:, t);                    
end
    
% (ii) Kalman filter and smoother
Xf = zeros(M, T);                                                          % Filtered state vector
Xu = zeros(M, T-1);                                                        % Updated state vector, Xu_{t|t} = E[X_t|y_1 ... y_t]
X_hat = zeros(M, T);                                                       % Smoothed state vector
Pf = zeros(M, M, T);                                                       % MSE of filtered state
Pu = zeros(M, M, T-1);                                                     % MSE of updated state, Pu_(t|t) = E[(X_t-Xu_(t|t))(X_t-Xu_(t|t))']

Xf(:, 1) = Ex;
Pf(:, :, 1) = Vx_mv;

for t = 1:T-1
    Kt = (Pf(:, :, t)*H') * pinv(H*Pf(:, :, t)*H');
    Pu(:, :, t) = (eye(M) - Kt*H) * Pf(:, :, t);
    Pf(:, :, t+1) = F_mv*Pu(:, :, t)*F_mv' + (1/zeta_0(1, t))*G_mv*Upsilon_sh*(G_mv');
    Xu(:, t) = Xf(:, t) + Kt*(Y_plus(:, t) - H*Xf(:, t));
    Xf(:, t+1) = F_mv*Xu(:, t);
end

Kt = (Pf(:, :, T)*H') * pinv(H*Pf(:, :, T)*H');
X_hat(:, T) = Xf(:, T) + Kt*(Y_plus(:, T) - H*Xf(:, T));

for t = 1:T-1
    J = (Pu(:, :, T-t)*F_mv') * pinv(Pf(:, :, T-t+1));
    X_hat(:, T-t) = Xu(:, T-t) + J*(X_hat(:, T-t+1) - Xf(:, T-t+1));
end
        
% (iii) Variable construction
xi_0 = X_plus + X_hat;
epsilon_0 = (pinv(G_mv'*G_mv)*G_mv')*(xi_0(:, 2:T) - C_mv - F_mv*xi_0(:, 1:(T-1)));