function L = log_posterior_mv(theta_mv_0, theta_sh_0, eta_0, zeta_0, G_degree, G_scale)

% Default settings
K = 3; T = size(eta_0, 2);
tolerance_b = 1e-3;

% Parameters
gx = theta_mv_0(6); gx_degree = G_degree(1); gx_scale = G_scale(1);
g1 = theta_mv_0(7); g1_degree = G_degree(2); g1_scale = G_scale(2);
g2 = theta_mv_0(8); g2_degree = G_degree(3); g2_scale = G_scale(3);

nu_sh = theta_sh_0(1);
b_sh = theta_sh_0(2:4);
if (b_sh'*b_sh < tolerance_b) 
    a_sh = 1;
    b_sh = zeros(K, 1);
    Upsilon_sh = (nu_sh-2)*eye(K);
else
    a_sh = (-(nu_sh-4) + sqrt((nu_sh-4)^2+8*(nu_sh-4)*(b_sh'*b_sh)))/(4*(b_sh'*b_sh));
    Upsilon_sh = (nu_sh-2)*(eye(K) + (a_sh-1)/(b_sh'*b_sh)*(b_sh*b_sh'));
end    
Upsilon_sqrt_sh = chol(Upsilon_sh, 'lower');
mean_eta_0 = -(a_sh*b_sh)*sqrt(zeta_0) + Upsilon_sh*b_sh./sqrt(zeta_0);

% Log-posterior density (log-likelihood + log-prior)
eta_star_0 = pinv(Upsilon_sqrt_sh)*(diag(1./sqrt([gx, g1, g2]))*eta_0-mean_eta_0);

L = -(T/2)*(log(gx)+log(g1)+log(g2)) ...
    - (1/2)*sum(sum(eta_star_0.^2, 1), 2);

L = L + 3*(gx+g1+g2) ...
    - (1/2)*(gx_degree*log(gx)+(gx_scale/gx)) ...
    - (1/2)*(g1_degree*log(g1)+(g1_scale/g1)) ...
    - (1/2)*(g2_degree*log(g2)+(g2_scale/g2));