function L = log_posterior_sh(theta_sh_0, epsilon_0, zeta_0)

% Default settings
[K, T] = size(epsilon_0);
tolerance_b = 1e-3;

% Parameters
nu_sh = theta_sh_0(1);
b_sh = theta_sh_0(2:4);
if (b_sh'*b_sh < tolerance_b) 
    a_sh = 1;
    b_sh = zeros(K, 1);
    Upsilon_sh = (nu_sh-2)*eye(K);
else
    a_sh = (-(nu_sh-4) + sqrt((nu_sh-4)^2+8*(nu_sh-4)*(b_sh'*b_sh)))/(4*(b_sh'*b_sh));
    Upsilon_sh = (nu_sh-2)*(eye(K) + (a_sh-1)/(b_sh'*b_sh)*(b_sh*b_sh'));
end
Upsilon_sqrt_sh = chol(Upsilon_sh, 'lower');

% Log-posterior density (log-likelihood + log-prior)
epsilon_star_0 = pinv(Upsilon_sqrt_sh) ...
                 *(epsilon_0.*sqrt(zeta_0)+(a_sh*b_sh)*sqrt(zeta_0)-Upsilon_sh*b_sh./sqrt(zeta_0));

L = - (T/2)*log(det(Upsilon_sh)) ...
    - (1/2)*sum(sum(epsilon_star_0.^2, 1), 2) ...
    - (T*nu_sh/2)*log(2) - T*log(gamma(nu_sh/2)) ...
    + (nu_sh/2)*sum(log(zeta_0), 2);