(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     66338,       1742]
NotebookOptionsPosition[     65255,       1704]
NotebookOutlinePosition[     65599,       1719]
CellTagsIndexPosition[     65556,       1716]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"DESCRIPTION", ":", "\[IndentingNewLine]", "   ", 
    RowBox[{
     RowBox[{
     "This", " ", "code", " ", "derives", " ", "analytically", " ", "the", 
      " ", "AR"}], "-", 
     RowBox[{
     "VMA", " ", "representation", " ", "of", " ", "the", " ", "Wiener"}], 
     "-", 
     RowBox[{
     "Kolmogorov", " ", "filter", " ", "for", " ", "the", " ", "Univariate", 
      " ", "Stationary", " ", "model"}]}]}], " ", "*)"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "INFORMATION", ":", "\[IndentingNewLine]", "   ", "Authors", ":", " ", 
     "Almuzara"}], ",", " ", "Amengual", ",", " ", 
    RowBox[{"Sentana", ".", " ", "\[IndentingNewLine]", "   ", 
     RowBox[{"Date", ":", " ", 
      RowBox[{"January", " ", "2016."}]}]}]}], " ", "*)"}], 
  "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.6628948982807956`*^9, 3.6628949680650163`*^9}, {
  3.66296616780527*^9, 3.6629661680613413`*^9}, {3.662978670351843*^9, 
  3.662978779808587*^9}, {3.6629788408156223`*^9, 3.6629788893892717`*^9}, {
  3.6629811909523296`*^9, 3.662981196913023*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Parameters", " ", "and", " ", 
    RowBox[{"matrices", ":", " ", 
     RowBox[{"ax", " ", "denotes", " ", 
      SubscriptBox["\[Alpha]", "x"]}]}]}], ",", " ", 
   RowBox[{"a1", " ", "denotes", " ", 
    SubscriptBox["\[Alpha]", "1"]}], ",", " ", 
   RowBox[{"a2", " ", "denotes", " ", 
    SubscriptBox["\[Alpha]", "2"]}], ",", " ", 
   RowBox[{"gx", " ", "denotes", " ", 
    SubscriptBox["\[Gamma]", "x"]}], ",", " ", 
   RowBox[{"g1", " ", "denotes", " ", 
    SubscriptBox["\[Gamma]", "1"], " ", "and", " ", "g2", " ", "denotes", " ", 
    SubscriptBox["\[Gamma]", "2"]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.647623625484875*^9, 3.647623679428853*^9}, {
  3.647623744618757*^9, 3.647623746444059*^9}, {3.647623907139451*^9, 
  3.6476239215552177`*^9}, {3.6476239647475214`*^9, 3.64762399752555*^9}, {
  3.647624280105523*^9, 3.6476242804087305`*^9}, {3.6628950281726074`*^9, 
  3.662895071369152*^9}, {3.6628951499021463`*^9, 3.6628951879452314`*^9}, {
  3.6628953074685345`*^9, 3.6628953169687414`*^9}, {3.662895379215222*^9, 
  3.6628954382822266`*^9}, {3.6629789432439756`*^9, 3.662978957254816*^9}, {
  3.6632298027285366`*^9, 3.6632299721132836`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"H", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "0", ",", "1"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"F", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "+", "ax"}], ",", 
       RowBox[{"-", "ax"}], ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"G", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Sqrt", "[", "gx", "]"}], ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"Sqrt", "[", "g1", "]"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", 
       RowBox[{"Sqrt", "[", "g2", "]"}]}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"M", "=", " ", 
   RowBox[{"Part", "[", 
    RowBox[{"Part", "[", 
     RowBox[{
      RowBox[{"Dimensions", "[", "G", "]"}], ",", "1"}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"K", " ", "=", " ", 
   RowBox[{"Part", "[", 
    RowBox[{"Part", "[", 
     RowBox[{
      RowBox[{"Dimensions", "[", "G", "]"}], ",", "2"}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"MatrixForm", "[", "H", "]"}], "\[IndentingNewLine]", 
 RowBox[{"MatrixForm", "[", "F", "]"}], "\[IndentingNewLine]", 
 RowBox[{"MatrixForm", "[", "G", "]"}]}], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGBQBGIQXdL73MBO97Ujxwx7axD9T/GoK4j2fMrjBqLNLnL7
gOgazzwwbXQnYjaIDnOpnwOiF9rKXOgC0lUskpdAdPJkra8g+k3oZTAt55zX
7Wf62nHToQ9g2qgj4kOP/WvHJc8Ef4NopeoIpl4grbCWjQNE+ytNVQXRbcw7
1EG0E0eSWR+QvudeCaYDUqfZgWjF5GQvEG2S+9UfRL9ZyxMOou0/H8sB0UXq
Yvkg2kyXuxhEh8xaUAGit0vKN4LosDNmYFpi9929IFom4sdhEP1BV/EMiH6l
Me06iI5KlH2/G0hrxU//BKJzS3cr7gHSnFUPwfSOD3sfZnm9dpw3PfcJiOY3
+qGTDaT9Ap4Zg+idvlnbQfQ+h+uHQHT8dA7JHCA97aojmA5YH3NoEpBWWSd0
GERf2qonNxlIX/kZBaY/2D18C6KZhJPegegluUlKU4H0N9EMMM23KyWbz/u1
47k1CTkgOoL1ajOI5t98qRtETzg3fQKIPnHnAJg29XN2FwXSfZWPwPSKRdax
ouWvHefO6QfTALs/9wE=
  "]],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "1", "0"},
     {"1", "0", "0", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.662896647759904*^9, 3.66289666373624*^9}, 
   3.6628973549988527`*^9, 3.66296621941621*^9, 3.662976537402906*^9, 
   3.6629772673851724`*^9, 3.6629789320743837`*^9, 3.6629813958252125`*^9, 
   3.6630054229664507`*^9, 3.6630116885959477`*^9, 3.663230047094572*^9, {
   3.663230081796793*^9, 3.663230087004974*^9}, 3.66324379454397*^9, 
   3.6632438402129993`*^9, 3.6632571094684424`*^9, 3.663260422264299*^9, 
   3.6638534500766087`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"1", "+", "ax"}], 
      RowBox[{"-", "ax"}], "0", "0"},
     {"1", "0", "0", "0"},
     {"0", "0", "a1", "0"},
     {"0", "0", "0", "a2"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.662896647759904*^9, 3.66289666373624*^9}, 
   3.6628973549988527`*^9, 3.66296621941621*^9, 3.662976537402906*^9, 
   3.6629772673851724`*^9, 3.6629789320743837`*^9, 3.6629813958252125`*^9, 
   3.6630054229664507`*^9, 3.6630116885959477`*^9, 3.663230047094572*^9, {
   3.663230081796793*^9, 3.663230087004974*^9}, 3.66324379454397*^9, 
   3.6632438402129993`*^9, 3.6632571094684424`*^9, 3.663260422264299*^9, 
   3.663853450080611*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      SqrtBox["gx"], "0", "0"},
     {"0", "0", "0"},
     {"0", 
      SqrtBox["g1"], "0"},
     {"0", "0", 
      SqrtBox["g2"]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.662896647759904*^9, 3.66289666373624*^9}, 
   3.6628973549988527`*^9, 3.66296621941621*^9, 3.662976537402906*^9, 
   3.6629772673851724`*^9, 3.6629789320743837`*^9, 3.6629813958252125`*^9, 
   3.6630054229664507`*^9, 3.6630116885959477`*^9, 3.663230047094572*^9, {
   3.663230081796793*^9, 3.663230087004974*^9}, 3.66324379454397*^9, 
   3.6632438402129993`*^9, 3.6632571094684424`*^9, 3.663260422264299*^9, 
   3.663853450082612*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Wiener", "-", 
   RowBox[{"Kolmogorov", " ", "filter", " ", "operator"}]}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.6476240180011415`*^9, 3.647624042738632*^9}, {
  3.6476242732024016`*^9, 3.64762427355265*^9}, {3.6629789694851665`*^9, 
  3.6629789734868603`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"FZ", " ", "=", " ", 
   RowBox[{"Simplify", "[", 
    RowBox[{"Inverse", "[", 
     RowBox[{
      RowBox[{"IdentityMatrix", "[", "M", "]"}], "-", 
      RowBox[{"F", " ", "z"}]}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"trFZ", " ", "=", " ", 
   RowBox[{"Simplify", "[", 
    RowBox[{"Inverse", "[", 
     RowBox[{
      RowBox[{"IdentityMatrix", "[", "M", "]"}], "-", 
      RowBox[{
       RowBox[{"Transpose", "[", "F", "]"}], " ", "/", " ", "z"}]}], "]"}], 
    "]"}]}], " ", ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"HFGZ", " ", "=", " ", 
   RowBox[{"Simplify", "[", 
    RowBox[{"H", " ", ".", "FZ", ".", " ", "G"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"trHFGZ", " ", "=", " ", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"Transpose", "[", "G", "]"}], ".", "trFZ", " ", ".", 
     RowBox[{"Transpose", "[", "H", "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Inv", " ", "=", " ", 
   RowBox[{"Simplify", "[", 
    RowBox[{"Inverse", "[", 
     RowBox[{"HFGZ", ".", "trHFGZ"}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FilterOperator", " ", "=", " ", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{"trHFGZ", ".", "Inv", ".", "HFGZ"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"MatrixForm", "[", "FilterOperator", "]"}]}], "Input",
 CellChangeTimes->{
  3.6475992104670815`*^9, {3.64759925768474*^9, 3.6475992792958922`*^9}, {
   3.6475998720305862`*^9, 3.6475999097757287`*^9}, {3.6475999668358593`*^9, 
   3.6476000318692055`*^9}, {3.647600192205982*^9, 3.6476001928244214`*^9}, {
   3.6476011206209345`*^9, 3.6476011254053297`*^9}, 3.6476744128343725`*^9, {
   3.6476779521110115`*^9, 3.6476779583364277`*^9}, 3.6476782268344655`*^9, {
   3.6477558483074036`*^9, 3.647755852314245*^9}, {3.6477559388464823`*^9, 
   3.647755941916661*^9}, {3.6518356031630297`*^9, 3.65183561250214*^9}, {
   3.651835742185629*^9, 3.6518357624970255`*^9}, {3.651836001529255*^9, 
   3.6518360110880585`*^9}, {3.6518361451424923`*^9, 3.651836146198236*^9}, {
   3.6518362590910273`*^9, 3.6518362632780237`*^9}, {3.65719793485481*^9, 
   3.6571979508352985`*^9}, {3.6571981361209283`*^9, 3.657198137878173*^9}, {
   3.657198285597821*^9, 3.657198395803625*^9}, {3.6571984629207563`*^9, 
   3.6571984760110536`*^9}, 3.6571985312998*^9, {3.657200303728074*^9, 
   3.657200323885412*^9}, {3.657200589895139*^9, 3.6572006201682644`*^9}, 
   3.6572007324480524`*^9, {3.6572007651085916`*^9, 3.657200765745878*^9}, {
   3.657200878592039*^9, 3.6572008809170084`*^9}, {3.6572009828912644`*^9, 
   3.657200983559746*^9}, {3.6629059100893073`*^9, 3.6629059632537794`*^9}, {
   3.6629060003758636`*^9, 3.662906007679474*^9}, {3.662906038236329*^9, 
   3.6629060526144023`*^9}, {3.6629061583181996`*^9, 3.662906162399375*^9}, {
   3.6629062163559685`*^9, 3.6629063014210796`*^9}, {3.6629063389963093`*^9, 
   3.662906455067329*^9}, {3.6629064933977375`*^9, 3.662906501191131*^9}, {
   3.662906532462473*^9, 3.6629066019411783`*^9}, {3.662906710345071*^9, 
   3.662906744765162*^9}, {3.6629067813338313`*^9, 3.6629069255476694`*^9}, 
   3.66290695718175*^9, {3.662907079358303*^9, 3.6629070795079765`*^9}, {
   3.662907167114703*^9, 3.6629072315383015`*^9}, {3.6629072640682287`*^9, 
   3.6629072849985685`*^9}, {3.662927494680483*^9, 3.662927503722223*^9}, {
   3.6629275668709235`*^9, 3.6629275714714813`*^9}, 3.6629276048831096`*^9, {
   3.6629276875490065`*^9, 3.6629276880616107`*^9}, {3.6629277304077954`*^9, 
   3.662927757398076*^9}, {3.6629281371154985`*^9, 3.662928141149809*^9}, {
   3.6629284496879587`*^9, 3.662928492858763*^9}, {3.6629287020143814`*^9, 
   3.662928702590583*^9}, {3.662929131695198*^9, 3.6629291435960245`*^9}, {
   3.6629291842073526`*^9, 3.662929246526797*^9}, {3.6629293157681804`*^9, 
   3.662929320917018*^9}, {3.6629293582647142`*^9, 3.6629293848568935`*^9}, {
   3.662929703972972*^9, 3.6629297121568375`*^9}, {3.662929746935916*^9, 
   3.6629297651537685`*^9}, {3.6629298726641884`*^9, 
   3.6629298932771244`*^9}, {3.662929931393495*^9, 3.6629299442900586`*^9}, {
   3.6629302752792673`*^9, 3.6629302898113165`*^9}, {3.6629303301005917`*^9, 
   3.662930365187873*^9}, {3.66293049423112*^9, 3.662930520875293*^9}, {
   3.6629306012371225`*^9, 3.6629306018278055`*^9}, {3.6629309296135163`*^9, 
   3.662930938175066*^9}, {3.6629310170135603`*^9, 3.6629310234895906`*^9}, {
   3.6629662329095364`*^9, 3.6629662527071505`*^9}, {3.6629767370769424`*^9, 
   3.662976741135004*^9}, {3.6629767967983055`*^9, 3.6629768650673637`*^9}, {
   3.662976895715925*^9, 3.662976900798212*^9}, {3.6629769467634735`*^9, 
   3.662977062247163*^9}, {3.6629771040219946`*^9, 3.6629771348235245`*^9}, {
   3.6629771973605967`*^9, 3.6629772503541884`*^9}, {3.6629777163662357`*^9, 
   3.66297773554643*^9}, {3.66298434457905*^9, 3.6629843692901306`*^9}, {
   3.6632301062260094`*^9, 3.6632301211667385`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox[
       RowBox[{"gx", " ", "z", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"a2", " ", "g1"}], "+", 
          RowBox[{"a1", " ", "g2"}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"g1", "+", 
             RowBox[{
              SuperscriptBox["a2", "2"], " ", "g1"}], "+", "g2", "+", 
             RowBox[{
              SuperscriptBox["a1", "2"], " ", "g2"}]}], ")"}], " ", "z"}], 
          "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"a2", " ", "g1"}], "+", 
             RowBox[{"a1", " ", "g2"}]}], ")"}], " ", 
           SuperscriptBox["z", "2"]}]}], ")"}]}], 
       RowBox[{
        RowBox[{
         SuperscriptBox["ax", "2"], " ", "g1", " ", "g2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", "z"}], "-", 
        RowBox[{"ax", " ", "g1", " ", "g2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           SuperscriptBox["z", "2"]}], ")"}]}], "+", 
        RowBox[{"z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "g2"}], " ", "gx", " ", 
            RowBox[{"(", 
             RowBox[{"a1", "-", "z"}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", 
              RowBox[{"a1", " ", "z"}]}], ")"}]}], "+", 
           RowBox[{"g1", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"g2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"]}], "-", 
              RowBox[{"gx", " ", 
               RowBox[{"(", 
                RowBox[{"a2", "-", "z"}], ")"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", "1"}], "+", 
                 RowBox[{"a2", " ", "z"}]}], ")"}]}]}], ")"}]}]}], ")"}]}]}]], 
      FractionBox[
       RowBox[{
        SqrtBox["g1"], " ", "g2", " ", 
        SqrtBox["gx"], " ", 
        RowBox[{"(", 
         RowBox[{"a1", "-", "z"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", "z"}], ")"}], " ", "z", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          RowBox[{"ax", " ", "z"}]}], ")"}]}], 
       RowBox[{
        RowBox[{
         SuperscriptBox["ax", "2"], " ", "g1", " ", "g2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", "z"}], "-", 
        RowBox[{"ax", " ", "g1", " ", "g2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           SuperscriptBox["z", "2"]}], ")"}]}], "+", 
        RowBox[{"z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "g2"}], " ", "gx", " ", 
            RowBox[{"(", 
             RowBox[{"a1", "-", "z"}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", 
              RowBox[{"a1", " ", "z"}]}], ")"}]}], "+", 
           RowBox[{"g1", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"g2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"]}], "-", 
              RowBox[{"gx", " ", 
               RowBox[{"(", 
                RowBox[{"a2", "-", "z"}], ")"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", "1"}], "+", 
                 RowBox[{"a2", " ", "z"}]}], ")"}]}]}], ")"}]}]}], ")"}]}]}]], 
      FractionBox[
       RowBox[{"g1", " ", 
        SqrtBox["g2"], " ", 
        SqrtBox["gx"], " ", 
        RowBox[{"(", 
         RowBox[{"a2", "-", "z"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", "z"}], ")"}], " ", "z", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          RowBox[{"ax", " ", "z"}]}], ")"}]}], 
       RowBox[{
        RowBox[{
         SuperscriptBox["ax", "2"], " ", "g1", " ", "g2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", "z"}], "-", 
        RowBox[{"ax", " ", "g1", " ", "g2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           SuperscriptBox["z", "2"]}], ")"}]}], "+", 
        RowBox[{"z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "g2"}], " ", "gx", " ", 
            RowBox[{"(", 
             RowBox[{"a1", "-", "z"}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", 
              RowBox[{"a1", " ", "z"}]}], ")"}]}], "+", 
           RowBox[{"g1", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"g2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"]}], "-", 
              RowBox[{"gx", " ", 
               RowBox[{"(", 
                RowBox[{"a2", "-", "z"}], ")"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", "1"}], "+", 
                 RowBox[{"a2", " ", "z"}]}], ")"}]}]}], ")"}]}]}], ")"}]}]}]]},
     {
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         SqrtBox["g1"], " ", "g2", " ", 
         SqrtBox["gx"], " ", 
         RowBox[{"(", 
          RowBox[{"ax", "-", "z"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "+", "z"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "+", 
           RowBox[{"a1", " ", "z"}]}], ")"}]}], 
        RowBox[{
         RowBox[{
          SuperscriptBox["ax", "2"], " ", "g1", " ", "g2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", "z"}], "-", 
         RowBox[{"ax", " ", "g1", " ", "g2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{"1", "+", 
            SuperscriptBox["z", "2"]}], ")"}]}], "+", 
         RowBox[{"z", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "g2"}], " ", "gx", " ", 
             RowBox[{"(", 
              RowBox[{"a1", "-", "z"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", 
               RowBox[{"a1", " ", "z"}]}], ")"}]}], "+", 
            RowBox[{"g1", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"]}], "-", 
               RowBox[{"gx", " ", 
                RowBox[{"(", 
                 RowBox[{"a2", "-", "z"}], ")"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "1"}], "+", 
                  RowBox[{"a2", " ", "z"}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}]}]]}], 
      FractionBox[
       RowBox[{"g1", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox["ax", "2"], " ", "g2", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", "z"}], "-", 
          RowBox[{"ax", " ", "g2", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", 
           RowBox[{"(", 
            RowBox[{"1", "+", 
             SuperscriptBox["z", "2"]}], ")"}]}], "+", 
          RowBox[{"z", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"]}], "-", 
             RowBox[{"gx", " ", 
              RowBox[{"(", 
               RowBox[{"a2", "-", "z"}], ")"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", "1"}], "+", 
                RowBox[{"a2", " ", "z"}]}], ")"}]}]}], ")"}]}]}], ")"}]}], 
       RowBox[{
        RowBox[{
         SuperscriptBox["ax", "2"], " ", "g1", " ", "g2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", "z"}], "-", 
        RowBox[{"ax", " ", "g1", " ", "g2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           SuperscriptBox["z", "2"]}], ")"}]}], "+", 
        RowBox[{"z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "g2"}], " ", "gx", " ", 
            RowBox[{"(", 
             RowBox[{"a1", "-", "z"}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", 
              RowBox[{"a1", " ", "z"}]}], ")"}]}], "+", 
           RowBox[{"g1", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"g2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"]}], "-", 
              RowBox[{"gx", " ", 
               RowBox[{"(", 
                RowBox[{"a2", "-", "z"}], ")"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", "1"}], "+", 
                 RowBox[{"a2", " ", "z"}]}], ")"}]}]}], ")"}]}]}], ")"}]}]}]], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         SqrtBox["g1"], " ", 
         SqrtBox["g2"], " ", "gx", " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "a2"}], "+", "z"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "+", 
           RowBox[{"a1", " ", "z"}]}], ")"}]}], 
        RowBox[{
         RowBox[{
          SuperscriptBox["ax", "2"], " ", "g1", " ", "g2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", "z"}], "-", 
         RowBox[{"ax", " ", "g1", " ", "g2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{"1", "+", 
            SuperscriptBox["z", "2"]}], ")"}]}], "+", 
         RowBox[{"z", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "g2"}], " ", "gx", " ", 
             RowBox[{"(", 
              RowBox[{"a1", "-", "z"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", 
               RowBox[{"a1", " ", "z"}]}], ")"}]}], "+", 
            RowBox[{"g1", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"]}], "-", 
               RowBox[{"gx", " ", 
                RowBox[{"(", 
                 RowBox[{"a2", "-", "z"}], ")"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "1"}], "+", 
                  RowBox[{"a2", " ", "z"}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}]}]]}]},
     {
      RowBox[{"-", 
       FractionBox[
        RowBox[{"g1", " ", 
         SqrtBox["g2"], " ", 
         SqrtBox["gx"], " ", 
         RowBox[{"(", 
          RowBox[{"ax", "-", "z"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "+", "z"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "+", 
           RowBox[{"a2", " ", "z"}]}], ")"}]}], 
        RowBox[{
         RowBox[{
          SuperscriptBox["ax", "2"], " ", "g1", " ", "g2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", "z"}], "-", 
         RowBox[{"ax", " ", "g1", " ", "g2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{"1", "+", 
            SuperscriptBox["z", "2"]}], ")"}]}], "+", 
         RowBox[{"z", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "g2"}], " ", "gx", " ", 
             RowBox[{"(", 
              RowBox[{"a1", "-", "z"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", 
               RowBox[{"a1", " ", "z"}]}], ")"}]}], "+", 
            RowBox[{"g1", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"]}], "-", 
               RowBox[{"gx", " ", 
                RowBox[{"(", 
                 RowBox[{"a2", "-", "z"}], ")"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "1"}], "+", 
                  RowBox[{"a2", " ", "z"}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}]}]]}], 
      FractionBox[
       RowBox[{
        SqrtBox["g1"], " ", 
        SqrtBox["g2"], " ", "gx", " ", 
        RowBox[{"(", 
         RowBox[{"a1", "-", "z"}], ")"}], " ", "z", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          RowBox[{"a2", " ", "z"}]}], ")"}]}], 
       RowBox[{
        RowBox[{
         SuperscriptBox["ax", "2"], " ", "g1", " ", "g2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", "z"}], "-", 
        RowBox[{"ax", " ", "g1", " ", "g2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           SuperscriptBox["z", "2"]}], ")"}]}], "+", 
        RowBox[{"z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "g2"}], " ", "gx", " ", 
            RowBox[{"(", 
             RowBox[{"a1", "-", "z"}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", 
              RowBox[{"a1", " ", "z"}]}], ")"}]}], "+", 
           RowBox[{"g1", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"g2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"]}], "-", 
              RowBox[{"gx", " ", 
               RowBox[{"(", 
                RowBox[{"a2", "-", "z"}], ")"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", "1"}], "+", 
                 RowBox[{"a2", " ", "z"}]}], ")"}]}]}], ")"}]}]}], ")"}]}]}]], 
      FractionBox[
       RowBox[{"g2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox["ax", "2"], " ", "g1", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", "z"}], "-", 
          RowBox[{"ax", " ", "g1", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", 
           RowBox[{"(", 
            RowBox[{"1", "+", 
             SuperscriptBox["z", "2"]}], ")"}]}], "+", 
          RowBox[{"z", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g1", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"]}], "-", 
             RowBox[{"gx", " ", 
              RowBox[{"(", 
               RowBox[{"a1", "-", "z"}], ")"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", "1"}], "+", 
                RowBox[{"a1", " ", "z"}]}], ")"}]}]}], ")"}]}]}], ")"}]}], 
       RowBox[{
        RowBox[{
         SuperscriptBox["ax", "2"], " ", "g1", " ", "g2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", "z"}], "-", 
        RowBox[{"ax", " ", "g1", " ", "g2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           SuperscriptBox["z", "2"]}], ")"}]}], "+", 
        RowBox[{"z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "g2"}], " ", "gx", " ", 
            RowBox[{"(", 
             RowBox[{"a1", "-", "z"}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", 
              RowBox[{"a1", " ", "z"}]}], ")"}]}], "+", 
           RowBox[{"g1", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"g2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"]}], "-", 
              RowBox[{"gx", " ", 
               RowBox[{"(", 
                RowBox[{"a2", "-", "z"}], ")"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", "1"}], "+", 
                 RowBox[{"a2", " ", "z"}]}], ")"}]}]}], ")"}]}]}], ")"}]}]}]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.657198477239955*^9, 3.6571985344110346`*^9, 3.657200059475477*^9, {
   3.657200378655695*^9, 3.6572003898306065`*^9}, 3.6572006332238393`*^9, 
   3.657200689428085*^9, 3.657200733692941*^9, 3.6572007695936155`*^9, 
   3.657200883076521*^9, 3.6572009849267373`*^9, {3.657201365353122*^9, 
   3.6572013665990305`*^9}, 3.657290014264945*^9, 3.657294105813134*^9, 
   3.657294660425838*^9, 3.65737532453069*^9, 3.657808021711049*^9, 
   3.658399810467373*^9, 3.658409204053833*^9, 3.662906930126816*^9, 
   3.6629069610773134`*^9, 3.6629070807508564`*^9, {3.66290720190792*^9, 
   3.6629072324635115`*^9}, {3.6629072650804405`*^9, 3.6629072855418186`*^9}, 
   3.66292750429879*^9, 3.66292757202594*^9, 3.6629276054813747`*^9, 
   3.6629276885940895`*^9, {3.6629277328309903`*^9, 3.662927757957762*^9}, 
   3.662928142068545*^9, 3.6629284603974113`*^9, 3.6629284936722*^9, 
   3.6629287040538793`*^9, 3.662929150827276*^9, 3.662929197808546*^9, 
   3.662929247168793*^9, 3.6629293213772087`*^9, {3.662929370669673*^9, 
   3.6629293851891723`*^9}, 3.6629297125378017`*^9, {3.662929754824152*^9, 
   3.662929765551173*^9}, {3.6629298776692314`*^9, 3.6629298949093256`*^9}, 
   3.6629299377928925`*^9, 3.6629302942005043`*^9, {3.6629303373117495`*^9, 
   3.662930366232767*^9}, {3.6629305091568966`*^9, 3.662930521289603*^9}, 
   3.6629306043056946`*^9, {3.662930931035269*^9, 3.662930939139721*^9}, 
   3.662931024115382*^9, 3.6629662802754407`*^9, 3.662976547749796*^9, 
   3.662976741858532*^9, {3.662976803551194*^9, 3.6629768409940577`*^9}, 
   3.662976913338895*^9, 3.662976987066358*^9, {3.662977044357403*^9, 
   3.6629770631362596`*^9}, 3.662977135363942*^9, 3.662977212566908*^9, {
   3.6629772470024176`*^9, 3.6629772801279573`*^9}, 3.6629777369247594`*^9, 
   3.6629789628917*^9, 3.6629814063837223`*^9, {3.6629843486747055`*^9, 
   3.6629843705951986`*^9}, 3.663005428418194*^9, 3.663011693803231*^9, 
   3.6632301255406203`*^9, {3.6632437867649703`*^9, 3.66324381546671*^9}, 
   3.663243848815324*^9, 3.663257116135817*^9, 3.6632604320693216`*^9, 
   3.6638534544188957`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Coefficient", " ", "matrices", " ", "of", " ", "the", " ", "AR"}],
    "-", 
   RowBox[{
   "VMA", " ", "representation", " ", "of", " ", "the", " ", "filter"}]}], 
  " ", "*)"}]], "Input",
 CellChangeTimes->{{3.6476240604175353`*^9, 3.6476241970539894`*^9}, {
  3.6629789843322773`*^9, 3.662978990700488*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"AR", " ", "=", " ", 
   RowBox[{"Apply", "[", 
    RowBox[{"PolynomialLCM", ",", 
     RowBox[{"Flatten", "[", 
      RowBox[{"Denominator", "[", "FilterOperator", "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MA", " ", "=", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{"AR", " ", "FilterOperator"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"p", "=", 
   RowBox[{"Exponent", "[", 
    RowBox[{"AR", ",", "z"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"q", " ", "=", " ", 
   RowBox[{"Apply", "[", 
    RowBox[{"Max", ",", 
     RowBox[{"Flatten", "[", 
      RowBox[{"Exponent", "[", 
       RowBox[{"MA", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"MatrixForm", "[", "AR", "]"}], "\[IndentingNewLine]", 
 RowBox[{"MatrixForm", "[", "MA", "]"}]}], "Input",
 CellChangeTimes->{{3.6476009146655984`*^9, 3.647600978852153*^9}, {
   3.6476012227683716`*^9, 3.6476012274697065`*^9}, {3.647601374805441*^9, 
   3.647601376067331*^9}, 3.6476014175395465`*^9, {3.647601483280203*^9, 
   3.647601502518262*^9}, 3.6476219375838847`*^9, {3.6476782828742447`*^9, 
   3.6476782884111724`*^9}, 3.6477559794020634`*^9, {3.6477561669102945`*^9, 
   3.647756168097701*^9}, {3.647756221730812*^9, 3.6477562228606195`*^9}, {
   3.651826803195713*^9, 3.651826804305202*^9}, 3.651836915443204*^9, {
   3.6518371514439564`*^9, 3.65183715258276*^9}, 3.651837386662074*^9, {
   3.6571986561644473`*^9, 3.65719868258453*^9}, {3.6571987369821353`*^9, 
   3.6571987537102227`*^9}, {3.6571988125713396`*^9, 3.657198814317581*^9}, {
   3.657198912697051*^9, 3.657198927580622*^9}, {3.6571989577843094`*^9, 
   3.657199007924385*^9}, {3.657199069096694*^9, 3.657199078465376*^9}, {
   3.6571993020184526`*^9, 3.657199304838825*^9}, {3.6572008981892524`*^9, 
   3.6572009284451895`*^9}, {3.6572009977127905`*^9, 
   3.6572010007951264`*^9}, {3.657201428255642*^9, 3.6572014283897104`*^9}, {
   3.6572014735896587`*^9, 3.657201485467328*^9}, {3.657202140625455*^9, 
   3.657202173138871*^9}, {3.6572941325885596`*^9, 3.6572941363138676`*^9}, {
   3.657294659394863*^9, 3.657294662902009*^9}, {3.66290781355459*^9, 
   3.662907824373844*^9}, {3.6629080408307247`*^9, 3.662908064283663*^9}, {
   3.6629081534477224`*^9, 3.6629081642888002`*^9}, {3.6629083870108223`*^9, 
   3.6629084070834265`*^9}, {3.6629084546159678`*^9, 
   3.6629084857904024`*^9}, {3.662908531169262*^9, 3.662908535952967*^9}, {
   3.662927462350629*^9, 3.6629274701346073`*^9}, {3.662927534003213*^9, 
   3.6629275588039484`*^9}, {3.662928708457289*^9, 3.662928753169586*^9}, {
   3.6629288186706576`*^9, 3.662928836643362*^9}, {3.662928965657549*^9, 
   3.6629289731932216`*^9}, {3.6629310425814085`*^9, 3.662931074187353*^9}, {
   3.662931197082601*^9, 3.6629312524525805`*^9}, {3.6629313430724373`*^9, 
   3.6629313485802975`*^9}, {3.6629314519603615`*^9, 
   3.6629315101745405`*^9}, {3.6629315516831846`*^9, 3.662931576511213*^9}, {
   3.6629317342735453`*^9, 3.662931742382759*^9}, {3.662976678374733*^9, 
   3.662976727455181*^9}, {3.6629776042559023`*^9, 3.6629776724268756`*^9}, {
   3.6629777067151594`*^9, 3.6629777083240414`*^9}, {3.662979003956295*^9, 
   3.6629790116167336`*^9}, {3.663230204941824*^9, 3.6632302064257193`*^9}, {
   3.663230252306595*^9, 3.663230254256442*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{
   RowBox[{"ax", " ", "g1", " ", "g2"}], "-", 
   RowBox[{"g1", " ", "g2", " ", "z"}], "-", 
   RowBox[{"2", " ", "ax", " ", "g1", " ", "g2", " ", "z"}], "-", 
   RowBox[{
    SuperscriptBox["ax", "2"], " ", "g1", " ", "g2", " ", "z"}], "-", 
   RowBox[{"a2", " ", "g1", " ", "gx", " ", "z"}], "-", 
   RowBox[{"a1", " ", "g2", " ", "gx", " ", "z"}], "+", 
   RowBox[{"2", " ", "g1", " ", "g2", " ", 
    SuperscriptBox["z", "2"]}], "+", 
   RowBox[{"2", " ", "ax", " ", "g1", " ", "g2", " ", 
    SuperscriptBox["z", "2"]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["ax", "2"], " ", "g1", " ", "g2", " ", 
    SuperscriptBox["z", "2"]}], "+", 
   RowBox[{"g1", " ", "gx", " ", 
    SuperscriptBox["z", "2"]}], "+", 
   RowBox[{
    SuperscriptBox["a2", "2"], " ", "g1", " ", "gx", " ", 
    SuperscriptBox["z", "2"]}], "+", 
   RowBox[{"g2", " ", "gx", " ", 
    SuperscriptBox["z", "2"]}], "+", 
   RowBox[{
    SuperscriptBox["a1", "2"], " ", "g2", " ", "gx", " ", 
    SuperscriptBox["z", "2"]}], "-", 
   RowBox[{"g1", " ", "g2", " ", 
    SuperscriptBox["z", "3"]}], "-", 
   RowBox[{"2", " ", "ax", " ", "g1", " ", "g2", " ", 
    SuperscriptBox["z", "3"]}], "-", 
   RowBox[{
    SuperscriptBox["ax", "2"], " ", "g1", " ", "g2", " ", 
    SuperscriptBox["z", "3"]}], "-", 
   RowBox[{"a2", " ", "g1", " ", "gx", " ", 
    SuperscriptBox["z", "3"]}], "-", 
   RowBox[{"a1", " ", "g2", " ", "gx", " ", 
    SuperscriptBox["z", "3"]}], "+", 
   RowBox[{"ax", " ", "g1", " ", "g2", " ", 
    SuperscriptBox["z", "4"]}]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.6629314706828394`*^9, 3.6629315105810843`*^9, {3.6629315552943525`*^9, 
   3.662931579948064*^9}, 3.6629317432642994`*^9, 3.6629765522295427`*^9, {
   3.662976694576005*^9, 3.662976727816635*^9}, 3.6629768526841726`*^9, 
   3.662976921304592*^9, 3.6629771527690563`*^9, 3.6629773001356335`*^9, {
   3.6629776215234427`*^9, 3.662977636079604*^9}, 3.662977672837504*^9, 
   3.662977744630142*^9, 3.6629790122248573`*^9, 3.662981411171348*^9, 
   3.6629843835592546`*^9, 3.6630054333199215`*^9, 3.6630116990988235`*^9, 
   3.663230149602437*^9, 3.6632302073567176`*^9, 3.663230254647768*^9, 
   3.663243853003194*^9, 3.663257119187874*^9, 3.6632604365217257`*^9, 
   3.6638534549676905`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       RowBox[{"-", "gx"}], " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"a2", " ", "g1"}], "+", 
         RowBox[{"a1", " ", "g2"}], "-", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"g1", "+", 
            RowBox[{
             SuperscriptBox["a2", "2"], " ", "g1"}], "+", "g2", "+", 
            RowBox[{
             SuperscriptBox["a1", "2"], " ", "g2"}]}], ")"}], " ", "z"}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"a2", " ", "g1"}], "+", 
            RowBox[{"a1", " ", "g2"}]}], ")"}], " ", 
          SuperscriptBox["z", "2"]}]}], ")"}]}], 
      RowBox[{
       RowBox[{"-", 
        SqrtBox["g1"]}], " ", "g2", " ", 
       SqrtBox["gx"], " ", 
       RowBox[{"(", 
        RowBox[{"a1", "-", "z"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "z"}], ")"}], " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         RowBox[{"ax", " ", "z"}]}], ")"}]}], 
      RowBox[{
       RowBox[{"-", "g1"}], " ", 
       SqrtBox["g2"], " ", 
       SqrtBox["gx"], " ", 
       RowBox[{"(", 
        RowBox[{"a2", "-", "z"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "z"}], ")"}], " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         RowBox[{"ax", " ", "z"}]}], ")"}]}]},
     {
      RowBox[{
       SqrtBox["g1"], " ", "g2", " ", 
       SqrtBox["gx"], " ", 
       RowBox[{"(", 
        RowBox[{"ax", "-", "z"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "z"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         RowBox[{"a1", " ", "z"}]}], ")"}]}], 
      RowBox[{
       RowBox[{"-", "g1"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["ax", "2"], " ", "g2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", "z"}], "-", 
         RowBox[{"ax", " ", "g2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{"1", "+", 
            SuperscriptBox["z", "2"]}], ")"}]}], "+", 
         RowBox[{"z", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"g2", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"]}], "-", 
            RowBox[{"gx", " ", 
             RowBox[{"(", 
              RowBox[{"a2", "-", "z"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", 
               RowBox[{"a2", " ", "z"}]}], ")"}]}]}], ")"}]}]}], ")"}]}], 
      RowBox[{
       SqrtBox["g1"], " ", 
       SqrtBox["g2"], " ", "gx", " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "a2"}], "+", "z"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         RowBox[{"a1", " ", "z"}]}], ")"}]}]},
     {
      RowBox[{"g1", " ", 
       SqrtBox["g2"], " ", 
       SqrtBox["gx"], " ", 
       RowBox[{"(", 
        RowBox[{"ax", "-", "z"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "z"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         RowBox[{"a2", " ", "z"}]}], ")"}]}], 
      RowBox[{
       RowBox[{"-", 
        SqrtBox["g1"]}], " ", 
       SqrtBox["g2"], " ", "gx", " ", 
       RowBox[{"(", 
        RowBox[{"a1", "-", "z"}], ")"}], " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         RowBox[{"a2", " ", "z"}]}], ")"}]}], 
      RowBox[{
       RowBox[{"-", "g2"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["ax", "2"], " ", "g1", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", "z"}], "-", 
         RowBox[{"ax", " ", "g1", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{"1", "+", 
            SuperscriptBox["z", "2"]}], ")"}]}], "+", 
         RowBox[{"z", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"g1", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", "1"}], "+", "z"}], ")"}], "2"]}], "-", 
            RowBox[{"gx", " ", 
             RowBox[{"(", 
              RowBox[{"a1", "-", "z"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", 
               RowBox[{"a1", " ", "z"}]}], ")"}]}]}], ")"}]}]}], ")"}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.6629314706828394`*^9, 3.6629315105810843`*^9, {3.6629315552943525`*^9, 
   3.662931579948064*^9}, 3.6629317432642994`*^9, 3.6629765522295427`*^9, {
   3.662976694576005*^9, 3.662976727816635*^9}, 3.6629768526841726`*^9, 
   3.662976921304592*^9, 3.6629771527690563`*^9, 3.6629773001356335`*^9, {
   3.6629776215234427`*^9, 3.662977636079604*^9}, 3.662977672837504*^9, 
   3.662977744630142*^9, 3.6629790122248573`*^9, 3.662981411171348*^9, 
   3.6629843835592546`*^9, 3.6630054333199215`*^9, 3.6630116990988235`*^9, 
   3.663230149602437*^9, 3.6632302073567176`*^9, 3.663230254647768*^9, 
   3.663243853003194*^9, 3.663257119187874*^9, 3.6632604365217257`*^9, 
   3.6638534549716887`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Collecting", " ", "coefficient", " ", "matrices", " ", "for", " ", "input",
    " ", "in", " ", "MATLAB", " ", "code"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.662979017594861*^9, 3.6629790585978756`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"aux0", " ", "=", " ", 
   RowBox[{"Coefficient", "[", 
    RowBox[{"AR", ",", "z", ",", "0"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"PHI", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Coefficient", "[", 
       RowBox[{"AR", ",", "z", ",", "1"}], "]"}]}], "/", "aux0"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Do", "[", 
   RowBox[{
    RowBox[{"PHI", " ", "=", " ", 
     RowBox[{"Join", "[", 
      RowBox[{"PHI", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Coefficient", "[", 
           RowBox[{"AR", ",", "z", ",", "j"}], "]"}]}], "/", "aux0"}], 
        "}"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "2", ",", "p", ",", "1"}], "}"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"THETA", " ", "=", " ", 
   RowBox[{
    RowBox[{"Coefficient", "[", 
     RowBox[{"MA", ",", "z", ",", "0"}], "]"}], "/", "aux0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Do", "[", 
   RowBox[{
    RowBox[{"THETA", " ", "=", " ", 
     RowBox[{"Join", "[", 
      RowBox[{"THETA", ",", 
       RowBox[{
        RowBox[{"Coefficient", "[", 
         RowBox[{"MA", ",", "z", ",", "j"}], "]"}], "/", "aux0"}], ",", "2"}],
       "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "q"}], "}"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"MatrixForm", "[", "PHI", "]"}], "\[IndentingNewLine]", 
 RowBox[{"MatrixForm", "[", "THETA", "]"}]}], "Input",
 CellChangeTimes->{{3.662977381699546*^9, 3.6629774394012303`*^9}, {
   3.662977477132804*^9, 3.6629775235756197`*^9}, {3.6629777631076593`*^9, 
   3.6629777826314583`*^9}, {3.662977829416805*^9, 3.6629778432215524`*^9}, {
   3.662977900945115*^9, 3.6629779248376284`*^9}, {3.66297804394165*^9, 
   3.662978154237462*^9}, {3.6629782081039667`*^9, 3.6629782154534473`*^9}, {
   3.662978269163667*^9, 3.662978313772671*^9}, {3.6629783683786983`*^9, 
   3.66297847095091*^9}, {3.662979066152646*^9, 3.6629790944635463`*^9}, {
   3.6629792423447704`*^9, 3.662979255149622*^9}, {3.6629809045785823`*^9, 
   3.6629809125429645`*^9}, {3.6629843939085054`*^9, 3.662984400794976*^9}, {
   3.663005148085739*^9, 3.66300530254438*^9}, {3.6630053709418087`*^9, 
   3.6630054173629375`*^9}, {3.663005515929472*^9, 3.663005577081358*^9}, 
   3.663005617071506*^9, {3.6630057068408427`*^9, 3.6630057743059206`*^9}, {
   3.663006966077234*^9, 3.663007009021063*^9}, {3.663009834695299*^9, 
   3.663009835701502*^9}, {3.663009944110761*^9, 3.663010011821021*^9}, {
   3.6630101817076445`*^9, 3.6630101889773445`*^9}, {3.663010255252433*^9, 
   3.66301027275802*^9}, {3.663010452358927*^9, 3.663010457138939*^9}, {
   3.6630113571297245`*^9, 3.663011439823928*^9}, {3.663011555119067*^9, 
   3.6630116484175134`*^9}, {3.6630117389035473`*^9, 3.6630118098682966`*^9}, 
   3.663257153380538*^9, {3.663432408215067*^9, 3.6634324117700596`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       FractionBox[
        RowBox[{
         RowBox[{"g1", " ", "g2"}], "+", 
         RowBox[{"2", " ", "ax", " ", "g1", " ", "g2"}], "+", 
         RowBox[{
          SuperscriptBox["ax", "2"], " ", "g1", " ", "g2"}], "+", 
         RowBox[{"a2", " ", "g1", " ", "gx"}], "+", 
         RowBox[{"a1", " ", "g2", " ", "gx"}]}], 
        RowBox[{"ax", " ", "g1", " ", "g2"}]]},
      {
       FractionBox[
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", "g1", " ", "g2"}], "-", 
         RowBox[{"2", " ", "ax", " ", "g1", " ", "g2"}], "-", 
         RowBox[{"2", " ", 
          SuperscriptBox["ax", "2"], " ", "g1", " ", "g2"}], "-", 
         RowBox[{"g1", " ", "gx"}], "-", 
         RowBox[{
          SuperscriptBox["a2", "2"], " ", "g1", " ", "gx"}], "-", 
         RowBox[{"g2", " ", "gx"}], "-", 
         RowBox[{
          SuperscriptBox["a1", "2"], " ", "g2", " ", "gx"}]}], 
        RowBox[{"ax", " ", "g1", " ", "g2"}]]},
      {
       FractionBox[
        RowBox[{
         RowBox[{"g1", " ", "g2"}], "+", 
         RowBox[{"2", " ", "ax", " ", "g1", " ", "g2"}], "+", 
         RowBox[{
          SuperscriptBox["ax", "2"], " ", "g1", " ", "g2"}], "+", 
         RowBox[{"a2", " ", "g1", " ", "gx"}], "+", 
         RowBox[{"a1", " ", "g2", " ", "gx"}]}], 
        RowBox[{"ax", " ", "g1", " ", "g2"}]]},
      {
       RowBox[{"-", "1"}]}
     },
     GridBoxAlignment->{
      "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
        "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.6629790947110786`*^9, 3.6629791273946033`*^9}, 
   3.6629792581797132`*^9, 3.6629809148377185`*^9, 3.662981418283529*^9, {
   3.662984390249856*^9, 3.662984401438779*^9}, 3.66300530346682*^9, 
   3.6630054384669523`*^9, {3.6630055196874027`*^9, 3.663005567586235*^9}, 
   3.6630056175059614`*^9, {3.663005713439952*^9, 3.663005775774328*^9}, 
   3.663006938690409*^9, 3.663007011933202*^9, {3.6630099473678446`*^9, 
   3.6630100125650396`*^9}, 3.6630102817730885`*^9, 3.6630104577431355`*^9, {
   3.6630114128462424`*^9, 3.663011440325344*^9}, {3.663011586961319*^9, 
   3.663011649648838*^9}, {3.6630117256629434`*^9, 3.663011761073002*^9}, {
   3.663011801171922*^9, 3.663011821666754*^9}, 3.6632301820006757`*^9, 
   3.6632302620032935`*^9, 3.6632438604859514`*^9, 3.66325715379369*^9, 
   3.6632604429103103`*^9, 3.6638534589036913`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"a2", " ", "g1"}], "+", 
           RowBox[{"a1", " ", "g2"}]}], ")"}], " ", "gx"}], 
        RowBox[{"ax", " ", "g1", " ", "g2"}]]}], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"a1", " ", 
         SqrtBox["gx"]}], 
        RowBox[{"ax", " ", 
         SqrtBox["g1"]}]]}], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"a2", " ", 
         SqrtBox["gx"]}], 
        RowBox[{"ax", " ", 
         SqrtBox["g2"]}]]}], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "g1"}], "-", 
           RowBox[{
            SuperscriptBox["a2", "2"], " ", "g1"}], "-", "g2", "-", 
           RowBox[{
            SuperscriptBox["a1", "2"], " ", "g2"}]}], ")"}], " ", "gx"}], 
        RowBox[{"ax", " ", "g1", " ", "g2"}]]}], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "-", "a1", "-", 
           RowBox[{"a1", " ", "ax"}]}], ")"}], " ", 
         SqrtBox["gx"]}], 
        RowBox[{"ax", " ", 
         SqrtBox["g1"]}]]}], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "-", "a2", "-", 
           RowBox[{"a2", " ", "ax"}]}], ")"}], " ", 
         SqrtBox["gx"]}], 
        RowBox[{"ax", " ", 
         SqrtBox["g2"]}]]}], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"a2", " ", "g1"}], "+", 
           RowBox[{"a1", " ", "g2"}]}], ")"}], " ", "gx"}], 
        RowBox[{"ax", " ", "g1", " ", "g2"}]]}], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "+", "ax", "+", 
           RowBox[{"a1", " ", "ax"}]}], ")"}], " ", 
         SqrtBox["gx"]}], 
        RowBox[{"ax", " ", 
         SqrtBox["g1"]}]]}], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "+", "ax", "+", 
           RowBox[{"a2", " ", "ax"}]}], ")"}], " ", 
         SqrtBox["gx"]}], 
        RowBox[{"ax", " ", 
         SqrtBox["g2"]}]]}], "0", 
      FractionBox[
       SqrtBox["gx"], 
       SqrtBox["g1"]], 
      FractionBox[
       SqrtBox["gx"], 
       SqrtBox["g2"]]},
     {
      FractionBox[
       SqrtBox["gx"], 
       SqrtBox["g1"]], "1", "0", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "-", "ax", "-", 
          RowBox[{"a1", " ", "ax"}]}], ")"}], " ", 
        SqrtBox["gx"]}], 
       RowBox[{"ax", " ", 
        SqrtBox["g1"]}]], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"g2", "+", 
         RowBox[{"2", " ", "ax", " ", "g2"}], "+", 
         RowBox[{
          SuperscriptBox["ax", "2"], " ", "g2"}], "+", 
         RowBox[{"a2", " ", "gx"}]}], 
        RowBox[{"ax", " ", "g2"}]]}], 
      FractionBox[
       RowBox[{"a2", " ", "gx"}], 
       RowBox[{"ax", " ", 
        SqrtBox["g1"], " ", 
        SqrtBox["g2"]}]], 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "+", "a1", "+", 
          RowBox[{"a1", " ", "ax"}]}], ")"}], " ", 
        SqrtBox["gx"]}], 
       RowBox[{"ax", " ", 
        SqrtBox["g1"]}]], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", "g2"}], "-", 
         RowBox[{"2", " ", "ax", " ", "g2"}], "-", 
         RowBox[{"2", " ", 
          SuperscriptBox["ax", "2"], " ", "g2"}], "-", "gx", "-", 
         RowBox[{
          SuperscriptBox["a2", "2"], " ", "gx"}]}], 
        RowBox[{"ax", " ", "g2"}]]}], 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "-", 
          RowBox[{"a1", " ", "a2"}]}], ")"}], " ", "gx"}], 
       RowBox[{"ax", " ", 
        SqrtBox["g1"], " ", 
        SqrtBox["g2"]}]], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"a1", " ", 
         SqrtBox["gx"]}], 
        RowBox[{"ax", " ", 
         SqrtBox["g1"]}]]}], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"g2", "+", 
         RowBox[{"2", " ", "ax", " ", "g2"}], "+", 
         RowBox[{
          SuperscriptBox["ax", "2"], " ", "g2"}], "+", 
         RowBox[{"a2", " ", "gx"}]}], 
        RowBox[{"ax", " ", "g2"}]]}], 
      FractionBox[
       RowBox[{"a1", " ", "gx"}], 
       RowBox[{"ax", " ", 
        SqrtBox["g1"], " ", 
        SqrtBox["g2"]}]], "0", "1", "0"},
     {
      FractionBox[
       SqrtBox["gx"], 
       SqrtBox["g2"]], "0", "1", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "-", "ax", "-", 
          RowBox[{"a2", " ", "ax"}]}], ")"}], " ", 
        SqrtBox["gx"]}], 
       RowBox[{"ax", " ", 
        SqrtBox["g2"]}]], 
      FractionBox[
       RowBox[{"a1", " ", "gx"}], 
       RowBox[{"ax", " ", 
        SqrtBox["g1"], " ", 
        SqrtBox["g2"]}]], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"g1", "+", 
         RowBox[{"2", " ", "ax", " ", "g1"}], "+", 
         RowBox[{
          SuperscriptBox["ax", "2"], " ", "g1"}], "+", 
         RowBox[{"a1", " ", "gx"}]}], 
        RowBox[{"ax", " ", "g1"}]]}], 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "+", "a2", "+", 
          RowBox[{"a2", " ", "ax"}]}], ")"}], " ", 
        SqrtBox["gx"]}], 
       RowBox[{"ax", " ", 
        SqrtBox["g2"]}]], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "+", 
           RowBox[{"a1", " ", "a2"}]}], ")"}], " ", "gx"}], 
        RowBox[{"ax", " ", 
         SqrtBox["g1"], " ", 
         SqrtBox["g2"]}]]}], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", "g1"}], "-", 
         RowBox[{"2", " ", "ax", " ", "g1"}], "-", 
         RowBox[{"2", " ", 
          SuperscriptBox["ax", "2"], " ", "g1"}], "-", "gx", "-", 
         RowBox[{
          SuperscriptBox["a1", "2"], " ", "gx"}]}], 
        RowBox[{"ax", " ", "g1"}]]}], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"a2", " ", 
         SqrtBox["gx"]}], 
        RowBox[{"ax", " ", 
         SqrtBox["g2"]}]]}], 
      FractionBox[
       RowBox[{"a2", " ", "gx"}], 
       RowBox[{"ax", " ", 
        SqrtBox["g1"], " ", 
        SqrtBox["g2"]}]], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"g1", "+", 
         RowBox[{"2", " ", "ax", " ", "g1"}], "+", 
         RowBox[{
          SuperscriptBox["ax", "2"], " ", "g1"}], "+", 
         RowBox[{"a1", " ", "gx"}]}], 
        RowBox[{"ax", " ", "g1"}]]}], "0", "0", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.6629790947110786`*^9, 3.6629791273946033`*^9}, 
   3.6629792581797132`*^9, 3.6629809148377185`*^9, 3.662981418283529*^9, {
   3.662984390249856*^9, 3.662984401438779*^9}, 3.66300530346682*^9, 
   3.6630054384669523`*^9, {3.6630055196874027`*^9, 3.663005567586235*^9}, 
   3.6630056175059614`*^9, {3.663005713439952*^9, 3.663005775774328*^9}, 
   3.663006938690409*^9, 3.663007011933202*^9, {3.6630099473678446`*^9, 
   3.6630100125650396`*^9}, 3.6630102817730885`*^9, 3.6630104577431355`*^9, {
   3.6630114128462424`*^9, 3.663011440325344*^9}, {3.663011586961319*^9, 
   3.663011649648838*^9}, {3.6630117256629434`*^9, 3.663011761073002*^9}, {
   3.663011801171922*^9, 3.663011821666754*^9}, 3.6632301820006757`*^9, 
   3.6632302620032935`*^9, 3.6632438604859514`*^9, 3.66325715379369*^9, 
   3.6632604429103103`*^9, 3.6638534589091997`*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"AppendTo", "[", 
   RowBox[{"$Path", ",", 
    RowBox[{"FindFile", "[", "\"\<ToMatlab.m\>\"", "]"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Get", "[", "\"\<ToMatlab.m\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"file", " ", "=", " ", 
   RowBox[{"OpenWrite", "[", "\"\<Bivariate_WKfilter.m\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Write", "[", 
   RowBox[{"file", ",", 
    RowBox[{
    "OutputForm", "[", 
     "\"\<function [PHI,THETA] = Bivariate_WKfilter(ax,gx,g1,g2) \\n\>\"", 
     "]"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"WriteMatlab", "[", 
   RowBox[{"PHI", ",", "file", ",", "\"\<PHI\>\""}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Write", "[", 
   RowBox[{"file", ",", 
    RowBox[{"OutputForm", "[", "\"\< \>\"", "]"}]}], "]"}], " ", 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"WriteMatlab", "[", 
   RowBox[{"THETA", ",", "file", ",", "\"\<THETA\>\""}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Close", "[", "file", "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.662979479512642*^9, 3.6629794837090983`*^9}, {
   3.662979726357485*^9, 3.662979726608963*^9}, {3.6629798586581717`*^9, 
   3.6629798712489147`*^9}, {3.662979916026958*^9, 3.6629799450278835`*^9}, {
   3.662980064914152*^9, 3.6629801036765566`*^9}, {3.6629801841792965`*^9, 
   3.6629802270347443`*^9}, {3.662980464780523*^9, 3.6629805009357986`*^9}, {
   3.662980659157852*^9, 3.662980694413725*^9}, {3.6629807647190895`*^9, 
   3.6629807728209896`*^9}, {3.6629809371559772`*^9, 3.662980955100498*^9}, {
   3.6632574738333006`*^9, 3.6632576131453347`*^9}, {3.6632577175520077`*^9, 
   3.663257752482092*^9}, {3.663257881042793*^9, 3.663257891756526*^9}, {
   3.6632579266010075`*^9, 3.663257927046226*^9}, {3.663257981677081*^9, 
   3.663258012322712*^9}, {3.663258076166355*^9, 3.6632581687877817`*^9}, 
   3.6632583378733273`*^9, {3.663258826892976*^9, 3.663258865926511*^9}, {
   3.6632589407157626`*^9, 3.6632589753246655`*^9}, {3.6632591790971317`*^9, 
   3.6632591815916433`*^9}, {3.66325924798759*^9, 3.6632592765713425`*^9}, {
   3.6632593723976974`*^9, 3.6632593940790844`*^9}, {3.663259514068592*^9, 
   3.663259521136835*^9}, {3.66325962128137*^9, 3.6632596254280753`*^9}, {
   3.6632599405156765`*^9, 3.663259963296932*^9}, {3.663260008222562*^9, 
   3.663260011257368*^9}, {3.6632602017199917`*^9, 3.663260295659356*^9}, {
   3.6632603584754972`*^9, 3.6632603627969127`*^9}, {3.6632605846977663`*^9, 
   3.6632605928925633`*^9}, {3.6632610587462654`*^9, 
   3.6632611034419694`*^9}, {3.6632611414615345`*^9, 3.663261172743718*^9}, {
   3.6632612091534076`*^9, 3.6632612900603175`*^9}, 3.6863127110445232`*^9}]
},
WindowSize->{1264, 583},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
FrontEndVersion->"10.3 para Microsoft Windows (64-bit) (October 9, 2015)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[557, 20, 1175, 27, 152, "Input"],
Cell[1735, 49, 1230, 23, 31, "Input"],
Cell[CellGroupData[{
Cell[2990, 76, 2511, 69, 278, "Input"],
Cell[5504, 147, 1064, 24, 42, "Output"],
Cell[6571, 173, 1169, 28, 72, "Output"],
Cell[7743, 203, 1147, 29, 90, "Output"]
}, Open  ]],
Cell[8905, 235, 318, 7, 31, "Input"],
Cell[CellGroupData[{
Cell[9248, 246, 4978, 88, 276, "Input"],
Cell[14229, 336, 21273, 582, 132, "Output"]
}, Open  ]],
Cell[35517, 921, 368, 9, 31, "Input"],
Cell[CellGroupData[{
Cell[35910, 934, 3382, 60, 238, "Input"],
Cell[39295, 996, 2342, 52, 52, "Output"],
Cell[41640, 1050, 6314, 184, 84, "Output"]
}, Open  ]],
Cell[47969, 1237, 260, 5, 31, "Input"],
Cell[CellGroupData[{
Cell[48254, 1246, 2982, 66, 276, "Input"],
Cell[51239, 1314, 2853, 66, 120, "Output"],
Cell[54095, 1382, 8221, 257, 127, "Output"]
}, Open  ]],
Cell[62331, 1642, 2920, 60, 316, "Input"]
}
]
*)

(* End of internal cache information *)
